# Generated by the gRPC Python protocol compiler plugin. DO NOT EDIT!
"""Client and server classes corresponding to protobuf-defined services."""
import grpc

from nidaqmx._stubs import nidaqmx_pb2 as nidaqmx__pb2


class NiDAQmxStub(object):
    """Missing associated documentation comment in .proto file."""

    def __init__(self, channel):
        """Constructor.

        Args:
            channel: A grpc.Channel.
        """
        self.AddCDAQSyncConnection = channel.unary_unary(
                '/nidaqmx_grpc.NiDAQmx/AddCDAQSyncConnection',
                request_serializer=nidaqmx__pb2.AddCDAQSyncConnectionRequest.SerializeToString,
                response_deserializer=nidaqmx__pb2.AddCDAQSyncConnectionResponse.FromString,
                )
        self.AddGlobalChansToTask = channel.unary_unary(
                '/nidaqmx_grpc.NiDAQmx/AddGlobalChansToTask',
                request_serializer=nidaqmx__pb2.AddGlobalChansToTaskRequest.SerializeToString,
                response_deserializer=nidaqmx__pb2.AddGlobalChansToTaskResponse.FromString,
                )
        self.AddNetworkDevice = channel.unary_unary(
                '/nidaqmx_grpc.NiDAQmx/AddNetworkDevice',
                request_serializer=nidaqmx__pb2.AddNetworkDeviceRequest.SerializeToString,
                response_deserializer=nidaqmx__pb2.AddNetworkDeviceResponse.FromString,
                )
        self.AreConfiguredCDAQSyncPortsDisconnected = channel.unary_unary(
                '/nidaqmx_grpc.NiDAQmx/AreConfiguredCDAQSyncPortsDisconnected',
                request_serializer=nidaqmx__pb2.AreConfiguredCDAQSyncPortsDisconnectedRequest.SerializeToString,
                response_deserializer=nidaqmx__pb2.AreConfiguredCDAQSyncPortsDisconnectedResponse.FromString,
                )
        self.AutoConfigureCDAQSyncConnections = channel.unary_unary(
                '/nidaqmx_grpc.NiDAQmx/AutoConfigureCDAQSyncConnections',
                request_serializer=nidaqmx__pb2.AutoConfigureCDAQSyncConnectionsRequest.SerializeToString,
                response_deserializer=nidaqmx__pb2.AutoConfigureCDAQSyncConnectionsResponse.FromString,
                )
        self.CalculateReversePolyCoeff = channel.unary_unary(
                '/nidaqmx_grpc.NiDAQmx/CalculateReversePolyCoeff',
                request_serializer=nidaqmx__pb2.CalculateReversePolyCoeffRequest.SerializeToString,
                response_deserializer=nidaqmx__pb2.CalculateReversePolyCoeffResponse.FromString,
                )
        self.CfgAnlgEdgeRefTrig = channel.unary_unary(
                '/nidaqmx_grpc.NiDAQmx/CfgAnlgEdgeRefTrig',
                request_serializer=nidaqmx__pb2.CfgAnlgEdgeRefTrigRequest.SerializeToString,
                response_deserializer=nidaqmx__pb2.CfgAnlgEdgeRefTrigResponse.FromString,
                )
        self.CfgAnlgEdgeStartTrig = channel.unary_unary(
                '/nidaqmx_grpc.NiDAQmx/CfgAnlgEdgeStartTrig',
                request_serializer=nidaqmx__pb2.CfgAnlgEdgeStartTrigRequest.SerializeToString,
                response_deserializer=nidaqmx__pb2.CfgAnlgEdgeStartTrigResponse.FromString,
                )
        self.CfgAnlgMultiEdgeRefTrig = channel.unary_unary(
                '/nidaqmx_grpc.NiDAQmx/CfgAnlgMultiEdgeRefTrig',
                request_serializer=nidaqmx__pb2.CfgAnlgMultiEdgeRefTrigRequest.SerializeToString,
                response_deserializer=nidaqmx__pb2.CfgAnlgMultiEdgeRefTrigResponse.FromString,
                )
        self.CfgAnlgMultiEdgeStartTrig = channel.unary_unary(
                '/nidaqmx_grpc.NiDAQmx/CfgAnlgMultiEdgeStartTrig',
                request_serializer=nidaqmx__pb2.CfgAnlgMultiEdgeStartTrigRequest.SerializeToString,
                response_deserializer=nidaqmx__pb2.CfgAnlgMultiEdgeStartTrigResponse.FromString,
                )
        self.CfgAnlgWindowRefTrig = channel.unary_unary(
                '/nidaqmx_grpc.NiDAQmx/CfgAnlgWindowRefTrig',
                request_serializer=nidaqmx__pb2.CfgAnlgWindowRefTrigRequest.SerializeToString,
                response_deserializer=nidaqmx__pb2.CfgAnlgWindowRefTrigResponse.FromString,
                )
        self.CfgAnlgWindowStartTrig = channel.unary_unary(
                '/nidaqmx_grpc.NiDAQmx/CfgAnlgWindowStartTrig',
                request_serializer=nidaqmx__pb2.CfgAnlgWindowStartTrigRequest.SerializeToString,
                response_deserializer=nidaqmx__pb2.CfgAnlgWindowStartTrigResponse.FromString,
                )
        self.CfgBurstHandshakingTimingExportClock = channel.unary_unary(
                '/nidaqmx_grpc.NiDAQmx/CfgBurstHandshakingTimingExportClock',
                request_serializer=nidaqmx__pb2.CfgBurstHandshakingTimingExportClockRequest.SerializeToString,
                response_deserializer=nidaqmx__pb2.CfgBurstHandshakingTimingExportClockResponse.FromString,
                )
        self.CfgBurstHandshakingTimingImportClock = channel.unary_unary(
                '/nidaqmx_grpc.NiDAQmx/CfgBurstHandshakingTimingImportClock',
                request_serializer=nidaqmx__pb2.CfgBurstHandshakingTimingImportClockRequest.SerializeToString,
                response_deserializer=nidaqmx__pb2.CfgBurstHandshakingTimingImportClockResponse.FromString,
                )
        self.CfgChangeDetectionTiming = channel.unary_unary(
                '/nidaqmx_grpc.NiDAQmx/CfgChangeDetectionTiming',
                request_serializer=nidaqmx__pb2.CfgChangeDetectionTimingRequest.SerializeToString,
                response_deserializer=nidaqmx__pb2.CfgChangeDetectionTimingResponse.FromString,
                )
        self.CfgDigEdgeRefTrig = channel.unary_unary(
                '/nidaqmx_grpc.NiDAQmx/CfgDigEdgeRefTrig',
                request_serializer=nidaqmx__pb2.CfgDigEdgeRefTrigRequest.SerializeToString,
                response_deserializer=nidaqmx__pb2.CfgDigEdgeRefTrigResponse.FromString,
                )
        self.CfgDigEdgeStartTrig = channel.unary_unary(
                '/nidaqmx_grpc.NiDAQmx/CfgDigEdgeStartTrig',
                request_serializer=nidaqmx__pb2.CfgDigEdgeStartTrigRequest.SerializeToString,
                response_deserializer=nidaqmx__pb2.CfgDigEdgeStartTrigResponse.FromString,
                )
        self.CfgDigPatternRefTrig = channel.unary_unary(
                '/nidaqmx_grpc.NiDAQmx/CfgDigPatternRefTrig',
                request_serializer=nidaqmx__pb2.CfgDigPatternRefTrigRequest.SerializeToString,
                response_deserializer=nidaqmx__pb2.CfgDigPatternRefTrigResponse.FromString,
                )
        self.CfgDigPatternStartTrig = channel.unary_unary(
                '/nidaqmx_grpc.NiDAQmx/CfgDigPatternStartTrig',
                request_serializer=nidaqmx__pb2.CfgDigPatternStartTrigRequest.SerializeToString,
                response_deserializer=nidaqmx__pb2.CfgDigPatternStartTrigResponse.FromString,
                )
        self.CfgHandshakingTiming = channel.unary_unary(
                '/nidaqmx_grpc.NiDAQmx/CfgHandshakingTiming',
                request_serializer=nidaqmx__pb2.CfgHandshakingTimingRequest.SerializeToString,
                response_deserializer=nidaqmx__pb2.CfgHandshakingTimingResponse.FromString,
                )
        self.CfgImplicitTiming = channel.unary_unary(
                '/nidaqmx_grpc.NiDAQmx/CfgImplicitTiming',
                request_serializer=nidaqmx__pb2.CfgImplicitTimingRequest.SerializeToString,
                response_deserializer=nidaqmx__pb2.CfgImplicitTimingResponse.FromString,
                )
        self.CfgInputBuffer = channel.unary_unary(
                '/nidaqmx_grpc.NiDAQmx/CfgInputBuffer',
                request_serializer=nidaqmx__pb2.CfgInputBufferRequest.SerializeToString,
                response_deserializer=nidaqmx__pb2.CfgInputBufferResponse.FromString,
                )
        self.CfgOutputBuffer = channel.unary_unary(
                '/nidaqmx_grpc.NiDAQmx/CfgOutputBuffer',
                request_serializer=nidaqmx__pb2.CfgOutputBufferRequest.SerializeToString,
                response_deserializer=nidaqmx__pb2.CfgOutputBufferResponse.FromString,
                )
        self.CfgPipelinedSampClkTiming = channel.unary_unary(
                '/nidaqmx_grpc.NiDAQmx/CfgPipelinedSampClkTiming',
                request_serializer=nidaqmx__pb2.CfgPipelinedSampClkTimingRequest.SerializeToString,
                response_deserializer=nidaqmx__pb2.CfgPipelinedSampClkTimingResponse.FromString,
                )
        self.CfgSampClkTiming = channel.unary_unary(
                '/nidaqmx_grpc.NiDAQmx/CfgSampClkTiming',
                request_serializer=nidaqmx__pb2.CfgSampClkTimingRequest.SerializeToString,
                response_deserializer=nidaqmx__pb2.CfgSampClkTimingResponse.FromString,
                )
        self.CfgTimeStartTrig = channel.unary_unary(
                '/nidaqmx_grpc.NiDAQmx/CfgTimeStartTrig',
                request_serializer=nidaqmx__pb2.CfgTimeStartTrigRequest.SerializeToString,
                response_deserializer=nidaqmx__pb2.CfgTimeStartTrigResponse.FromString,
                )
        self.CfgWatchdogAOExpirStates = channel.unary_unary(
                '/nidaqmx_grpc.NiDAQmx/CfgWatchdogAOExpirStates',
                request_serializer=nidaqmx__pb2.CfgWatchdogAOExpirStatesRequest.SerializeToString,
                response_deserializer=nidaqmx__pb2.CfgWatchdogAOExpirStatesResponse.FromString,
                )
        self.CfgWatchdogCOExpirStates = channel.unary_unary(
                '/nidaqmx_grpc.NiDAQmx/CfgWatchdogCOExpirStates',
                request_serializer=nidaqmx__pb2.CfgWatchdogCOExpirStatesRequest.SerializeToString,
                response_deserializer=nidaqmx__pb2.CfgWatchdogCOExpirStatesResponse.FromString,
                )
        self.CfgWatchdogDOExpirStates = channel.unary_unary(
                '/nidaqmx_grpc.NiDAQmx/CfgWatchdogDOExpirStates',
                request_serializer=nidaqmx__pb2.CfgWatchdogDOExpirStatesRequest.SerializeToString,
                response_deserializer=nidaqmx__pb2.CfgWatchdogDOExpirStatesResponse.FromString,
                )
        self.ClearTEDS = channel.unary_unary(
                '/nidaqmx_grpc.NiDAQmx/ClearTEDS',
                request_serializer=nidaqmx__pb2.ClearTEDSRequest.SerializeToString,
                response_deserializer=nidaqmx__pb2.ClearTEDSResponse.FromString,
                )
        self.ClearTask = channel.unary_unary(
                '/nidaqmx_grpc.NiDAQmx/ClearTask',
                request_serializer=nidaqmx__pb2.ClearTaskRequest.SerializeToString,
                response_deserializer=nidaqmx__pb2.ClearTaskResponse.FromString,
                )
        self.ConfigureLogging = channel.unary_unary(
                '/nidaqmx_grpc.NiDAQmx/ConfigureLogging',
                request_serializer=nidaqmx__pb2.ConfigureLoggingRequest.SerializeToString,
                response_deserializer=nidaqmx__pb2.ConfigureLoggingResponse.FromString,
                )
        self.ConfigureTEDS = channel.unary_unary(
                '/nidaqmx_grpc.NiDAQmx/ConfigureTEDS',
                request_serializer=nidaqmx__pb2.ConfigureTEDSRequest.SerializeToString,
                response_deserializer=nidaqmx__pb2.ConfigureTEDSResponse.FromString,
                )
        self.ConnectTerms = channel.unary_unary(
                '/nidaqmx_grpc.NiDAQmx/ConnectTerms',
                request_serializer=nidaqmx__pb2.ConnectTermsRequest.SerializeToString,
                response_deserializer=nidaqmx__pb2.ConnectTermsResponse.FromString,
                )
        self.ControlWatchdogTask = channel.unary_unary(
                '/nidaqmx_grpc.NiDAQmx/ControlWatchdogTask',
                request_serializer=nidaqmx__pb2.ControlWatchdogTaskRequest.SerializeToString,
                response_deserializer=nidaqmx__pb2.ControlWatchdogTaskResponse.FromString,
                )
        self.CreateAIAccel4WireDCVoltageChan = channel.unary_unary(
                '/nidaqmx_grpc.NiDAQmx/CreateAIAccel4WireDCVoltageChan',
                request_serializer=nidaqmx__pb2.CreateAIAccel4WireDCVoltageChanRequest.SerializeToString,
                response_deserializer=nidaqmx__pb2.CreateAIAccel4WireDCVoltageChanResponse.FromString,
                )
        self.CreateAIAccelChan = channel.unary_unary(
                '/nidaqmx_grpc.NiDAQmx/CreateAIAccelChan',
                request_serializer=nidaqmx__pb2.CreateAIAccelChanRequest.SerializeToString,
                response_deserializer=nidaqmx__pb2.CreateAIAccelChanResponse.FromString,
                )
        self.CreateAIAccelChargeChan = channel.unary_unary(
                '/nidaqmx_grpc.NiDAQmx/CreateAIAccelChargeChan',
                request_serializer=nidaqmx__pb2.CreateAIAccelChargeChanRequest.SerializeToString,
                response_deserializer=nidaqmx__pb2.CreateAIAccelChargeChanResponse.FromString,
                )
        self.CreateAIBridgeChan = channel.unary_unary(
                '/nidaqmx_grpc.NiDAQmx/CreateAIBridgeChan',
                request_serializer=nidaqmx__pb2.CreateAIBridgeChanRequest.SerializeToString,
                response_deserializer=nidaqmx__pb2.CreateAIBridgeChanResponse.FromString,
                )
        self.CreateAIChargeChan = channel.unary_unary(
                '/nidaqmx_grpc.NiDAQmx/CreateAIChargeChan',
                request_serializer=nidaqmx__pb2.CreateAIChargeChanRequest.SerializeToString,
                response_deserializer=nidaqmx__pb2.CreateAIChargeChanResponse.FromString,
                )
        self.CreateAICurrentChan = channel.unary_unary(
                '/nidaqmx_grpc.NiDAQmx/CreateAICurrentChan',
                request_serializer=nidaqmx__pb2.CreateAICurrentChanRequest.SerializeToString,
                response_deserializer=nidaqmx__pb2.CreateAICurrentChanResponse.FromString,
                )
        self.CreateAICurrentRMSChan = channel.unary_unary(
                '/nidaqmx_grpc.NiDAQmx/CreateAICurrentRMSChan',
                request_serializer=nidaqmx__pb2.CreateAICurrentRMSChanRequest.SerializeToString,
                response_deserializer=nidaqmx__pb2.CreateAICurrentRMSChanResponse.FromString,
                )
        self.CreateAIForceBridgePolynomialChan = channel.unary_unary(
                '/nidaqmx_grpc.NiDAQmx/CreateAIForceBridgePolynomialChan',
                request_serializer=nidaqmx__pb2.CreateAIForceBridgePolynomialChanRequest.SerializeToString,
                response_deserializer=nidaqmx__pb2.CreateAIForceBridgePolynomialChanResponse.FromString,
                )
        self.CreateAIForceBridgeTableChan = channel.unary_unary(
                '/nidaqmx_grpc.NiDAQmx/CreateAIForceBridgeTableChan',
                request_serializer=nidaqmx__pb2.CreateAIForceBridgeTableChanRequest.SerializeToString,
                response_deserializer=nidaqmx__pb2.CreateAIForceBridgeTableChanResponse.FromString,
                )
        self.CreateAIForceBridgeTwoPointLinChan = channel.unary_unary(
                '/nidaqmx_grpc.NiDAQmx/CreateAIForceBridgeTwoPointLinChan',
                request_serializer=nidaqmx__pb2.CreateAIForceBridgeTwoPointLinChanRequest.SerializeToString,
                response_deserializer=nidaqmx__pb2.CreateAIForceBridgeTwoPointLinChanResponse.FromString,
                )
        self.CreateAIForceIEPEChan = channel.unary_unary(
                '/nidaqmx_grpc.NiDAQmx/CreateAIForceIEPEChan',
                request_serializer=nidaqmx__pb2.CreateAIForceIEPEChanRequest.SerializeToString,
                response_deserializer=nidaqmx__pb2.CreateAIForceIEPEChanResponse.FromString,
                )
        self.CreateAIFreqVoltageChan = channel.unary_unary(
                '/nidaqmx_grpc.NiDAQmx/CreateAIFreqVoltageChan',
                request_serializer=nidaqmx__pb2.CreateAIFreqVoltageChanRequest.SerializeToString,
                response_deserializer=nidaqmx__pb2.CreateAIFreqVoltageChanResponse.FromString,
                )
        self.CreateAIMicrophoneChan = channel.unary_unary(
                '/nidaqmx_grpc.NiDAQmx/CreateAIMicrophoneChan',
                request_serializer=nidaqmx__pb2.CreateAIMicrophoneChanRequest.SerializeToString,
                response_deserializer=nidaqmx__pb2.CreateAIMicrophoneChanResponse.FromString,
                )
        self.CreateAIPosEddyCurrProxProbeChan = channel.unary_unary(
                '/nidaqmx_grpc.NiDAQmx/CreateAIPosEddyCurrProxProbeChan',
                request_serializer=nidaqmx__pb2.CreateAIPosEddyCurrProxProbeChanRequest.SerializeToString,
                response_deserializer=nidaqmx__pb2.CreateAIPosEddyCurrProxProbeChanResponse.FromString,
                )
        self.CreateAIPosLVDTChan = channel.unary_unary(
                '/nidaqmx_grpc.NiDAQmx/CreateAIPosLVDTChan',
                request_serializer=nidaqmx__pb2.CreateAIPosLVDTChanRequest.SerializeToString,
                response_deserializer=nidaqmx__pb2.CreateAIPosLVDTChanResponse.FromString,
                )
        self.CreateAIPosRVDTChan = channel.unary_unary(
                '/nidaqmx_grpc.NiDAQmx/CreateAIPosRVDTChan',
                request_serializer=nidaqmx__pb2.CreateAIPosRVDTChanRequest.SerializeToString,
                response_deserializer=nidaqmx__pb2.CreateAIPosRVDTChanResponse.FromString,
                )
        self.CreateAIPowerChan = channel.unary_unary(
                '/nidaqmx_grpc.NiDAQmx/CreateAIPowerChan',
                request_serializer=nidaqmx__pb2.CreateAIPowerChanRequest.SerializeToString,
                response_deserializer=nidaqmx__pb2.CreateAIPowerChanResponse.FromString,
                )
        self.CreateAIPressureBridgePolynomialChan = channel.unary_unary(
                '/nidaqmx_grpc.NiDAQmx/CreateAIPressureBridgePolynomialChan',
                request_serializer=nidaqmx__pb2.CreateAIPressureBridgePolynomialChanRequest.SerializeToString,
                response_deserializer=nidaqmx__pb2.CreateAIPressureBridgePolynomialChanResponse.FromString,
                )
        self.CreateAIPressureBridgeTableChan = channel.unary_unary(
                '/nidaqmx_grpc.NiDAQmx/CreateAIPressureBridgeTableChan',
                request_serializer=nidaqmx__pb2.CreateAIPressureBridgeTableChanRequest.SerializeToString,
                response_deserializer=nidaqmx__pb2.CreateAIPressureBridgeTableChanResponse.FromString,
                )
        self.CreateAIPressureBridgeTwoPointLinChan = channel.unary_unary(
                '/nidaqmx_grpc.NiDAQmx/CreateAIPressureBridgeTwoPointLinChan',
                request_serializer=nidaqmx__pb2.CreateAIPressureBridgeTwoPointLinChanRequest.SerializeToString,
                response_deserializer=nidaqmx__pb2.CreateAIPressureBridgeTwoPointLinChanResponse.FromString,
                )
        self.CreateAIRTDChan = channel.unary_unary(
                '/nidaqmx_grpc.NiDAQmx/CreateAIRTDChan',
                request_serializer=nidaqmx__pb2.CreateAIRTDChanRequest.SerializeToString,
                response_deserializer=nidaqmx__pb2.CreateAIRTDChanResponse.FromString,
                )
        self.CreateAIResistanceChan = channel.unary_unary(
                '/nidaqmx_grpc.NiDAQmx/CreateAIResistanceChan',
                request_serializer=nidaqmx__pb2.CreateAIResistanceChanRequest.SerializeToString,
                response_deserializer=nidaqmx__pb2.CreateAIResistanceChanResponse.FromString,
                )
        self.CreateAIRosetteStrainGageChan = channel.unary_unary(
                '/nidaqmx_grpc.NiDAQmx/CreateAIRosetteStrainGageChan',
                request_serializer=nidaqmx__pb2.CreateAIRosetteStrainGageChanRequest.SerializeToString,
                response_deserializer=nidaqmx__pb2.CreateAIRosetteStrainGageChanResponse.FromString,
                )
        self.CreateAIStrainGageChan = channel.unary_unary(
                '/nidaqmx_grpc.NiDAQmx/CreateAIStrainGageChan',
                request_serializer=nidaqmx__pb2.CreateAIStrainGageChanRequest.SerializeToString,
                response_deserializer=nidaqmx__pb2.CreateAIStrainGageChanResponse.FromString,
                )
        self.CreateAITempBuiltInSensorChan = channel.unary_unary(
                '/nidaqmx_grpc.NiDAQmx/CreateAITempBuiltInSensorChan',
                request_serializer=nidaqmx__pb2.CreateAITempBuiltInSensorChanRequest.SerializeToString,
                response_deserializer=nidaqmx__pb2.CreateAITempBuiltInSensorChanResponse.FromString,
                )
        self.CreateAIThrmcplChan = channel.unary_unary(
                '/nidaqmx_grpc.NiDAQmx/CreateAIThrmcplChan',
                request_serializer=nidaqmx__pb2.CreateAIThrmcplChanRequest.SerializeToString,
                response_deserializer=nidaqmx__pb2.CreateAIThrmcplChanResponse.FromString,
                )
        self.CreateAIThrmstrChanIex = channel.unary_unary(
                '/nidaqmx_grpc.NiDAQmx/CreateAIThrmstrChanIex',
                request_serializer=nidaqmx__pb2.CreateAIThrmstrChanIexRequest.SerializeToString,
                response_deserializer=nidaqmx__pb2.CreateAIThrmstrChanIexResponse.FromString,
                )
        self.CreateAIThrmstrChanVex = channel.unary_unary(
                '/nidaqmx_grpc.NiDAQmx/CreateAIThrmstrChanVex',
                request_serializer=nidaqmx__pb2.CreateAIThrmstrChanVexRequest.SerializeToString,
                response_deserializer=nidaqmx__pb2.CreateAIThrmstrChanVexResponse.FromString,
                )
        self.CreateAITorqueBridgePolynomialChan = channel.unary_unary(
                '/nidaqmx_grpc.NiDAQmx/CreateAITorqueBridgePolynomialChan',
                request_serializer=nidaqmx__pb2.CreateAITorqueBridgePolynomialChanRequest.SerializeToString,
                response_deserializer=nidaqmx__pb2.CreateAITorqueBridgePolynomialChanResponse.FromString,
                )
        self.CreateAITorqueBridgeTableChan = channel.unary_unary(
                '/nidaqmx_grpc.NiDAQmx/CreateAITorqueBridgeTableChan',
                request_serializer=nidaqmx__pb2.CreateAITorqueBridgeTableChanRequest.SerializeToString,
                response_deserializer=nidaqmx__pb2.CreateAITorqueBridgeTableChanResponse.FromString,
                )
        self.CreateAITorqueBridgeTwoPointLinChan = channel.unary_unary(
                '/nidaqmx_grpc.NiDAQmx/CreateAITorqueBridgeTwoPointLinChan',
                request_serializer=nidaqmx__pb2.CreateAITorqueBridgeTwoPointLinChanRequest.SerializeToString,
                response_deserializer=nidaqmx__pb2.CreateAITorqueBridgeTwoPointLinChanResponse.FromString,
                )
        self.CreateAIVelocityIEPEChan = channel.unary_unary(
                '/nidaqmx_grpc.NiDAQmx/CreateAIVelocityIEPEChan',
                request_serializer=nidaqmx__pb2.CreateAIVelocityIEPEChanRequest.SerializeToString,
                response_deserializer=nidaqmx__pb2.CreateAIVelocityIEPEChanResponse.FromString,
                )
        self.CreateAIVoltageChan = channel.unary_unary(
                '/nidaqmx_grpc.NiDAQmx/CreateAIVoltageChan',
                request_serializer=nidaqmx__pb2.CreateAIVoltageChanRequest.SerializeToString,
                response_deserializer=nidaqmx__pb2.CreateAIVoltageChanResponse.FromString,
                )
        self.CreateAIVoltageChanWithExcit = channel.unary_unary(
                '/nidaqmx_grpc.NiDAQmx/CreateAIVoltageChanWithExcit',
                request_serializer=nidaqmx__pb2.CreateAIVoltageChanWithExcitRequest.SerializeToString,
                response_deserializer=nidaqmx__pb2.CreateAIVoltageChanWithExcitResponse.FromString,
                )
        self.CreateAIVoltageRMSChan = channel.unary_unary(
                '/nidaqmx_grpc.NiDAQmx/CreateAIVoltageRMSChan',
                request_serializer=nidaqmx__pb2.CreateAIVoltageRMSChanRequest.SerializeToString,
                response_deserializer=nidaqmx__pb2.CreateAIVoltageRMSChanResponse.FromString,
                )
        self.CreateAOCurrentChan = channel.unary_unary(
                '/nidaqmx_grpc.NiDAQmx/CreateAOCurrentChan',
                request_serializer=nidaqmx__pb2.CreateAOCurrentChanRequest.SerializeToString,
                response_deserializer=nidaqmx__pb2.CreateAOCurrentChanResponse.FromString,
                )
        self.CreateAOFuncGenChan = channel.unary_unary(
                '/nidaqmx_grpc.NiDAQmx/CreateAOFuncGenChan',
                request_serializer=nidaqmx__pb2.CreateAOFuncGenChanRequest.SerializeToString,
                response_deserializer=nidaqmx__pb2.CreateAOFuncGenChanResponse.FromString,
                )
        self.CreateAOVoltageChan = channel.unary_unary(
                '/nidaqmx_grpc.NiDAQmx/CreateAOVoltageChan',
                request_serializer=nidaqmx__pb2.CreateAOVoltageChanRequest.SerializeToString,
                response_deserializer=nidaqmx__pb2.CreateAOVoltageChanResponse.FromString,
                )
        self.CreateCIAngEncoderChan = channel.unary_unary(
                '/nidaqmx_grpc.NiDAQmx/CreateCIAngEncoderChan',
                request_serializer=nidaqmx__pb2.CreateCIAngEncoderChanRequest.SerializeToString,
                response_deserializer=nidaqmx__pb2.CreateCIAngEncoderChanResponse.FromString,
                )
        self.CreateCIAngVelocityChan = channel.unary_unary(
                '/nidaqmx_grpc.NiDAQmx/CreateCIAngVelocityChan',
                request_serializer=nidaqmx__pb2.CreateCIAngVelocityChanRequest.SerializeToString,
                response_deserializer=nidaqmx__pb2.CreateCIAngVelocityChanResponse.FromString,
                )
        self.CreateCICountEdgesChan = channel.unary_unary(
                '/nidaqmx_grpc.NiDAQmx/CreateCICountEdgesChan',
                request_serializer=nidaqmx__pb2.CreateCICountEdgesChanRequest.SerializeToString,
                response_deserializer=nidaqmx__pb2.CreateCICountEdgesChanResponse.FromString,
                )
        self.CreateCIDutyCycleChan = channel.unary_unary(
                '/nidaqmx_grpc.NiDAQmx/CreateCIDutyCycleChan',
                request_serializer=nidaqmx__pb2.CreateCIDutyCycleChanRequest.SerializeToString,
                response_deserializer=nidaqmx__pb2.CreateCIDutyCycleChanResponse.FromString,
                )
        self.CreateCIFreqChan = channel.unary_unary(
                '/nidaqmx_grpc.NiDAQmx/CreateCIFreqChan',
                request_serializer=nidaqmx__pb2.CreateCIFreqChanRequest.SerializeToString,
                response_deserializer=nidaqmx__pb2.CreateCIFreqChanResponse.FromString,
                )
        self.CreateCIGPSTimestampChan = channel.unary_unary(
                '/nidaqmx_grpc.NiDAQmx/CreateCIGPSTimestampChan',
                request_serializer=nidaqmx__pb2.CreateCIGPSTimestampChanRequest.SerializeToString,
                response_deserializer=nidaqmx__pb2.CreateCIGPSTimestampChanResponse.FromString,
                )
        self.CreateCILinEncoderChan = channel.unary_unary(
                '/nidaqmx_grpc.NiDAQmx/CreateCILinEncoderChan',
                request_serializer=nidaqmx__pb2.CreateCILinEncoderChanRequest.SerializeToString,
                response_deserializer=nidaqmx__pb2.CreateCILinEncoderChanResponse.FromString,
                )
        self.CreateCILinVelocityChan = channel.unary_unary(
                '/nidaqmx_grpc.NiDAQmx/CreateCILinVelocityChan',
                request_serializer=nidaqmx__pb2.CreateCILinVelocityChanRequest.SerializeToString,
                response_deserializer=nidaqmx__pb2.CreateCILinVelocityChanResponse.FromString,
                )
        self.CreateCIPeriodChan = channel.unary_unary(
                '/nidaqmx_grpc.NiDAQmx/CreateCIPeriodChan',
                request_serializer=nidaqmx__pb2.CreateCIPeriodChanRequest.SerializeToString,
                response_deserializer=nidaqmx__pb2.CreateCIPeriodChanResponse.FromString,
                )
        self.CreateCIPulseChanFreq = channel.unary_unary(
                '/nidaqmx_grpc.NiDAQmx/CreateCIPulseChanFreq',
                request_serializer=nidaqmx__pb2.CreateCIPulseChanFreqRequest.SerializeToString,
                response_deserializer=nidaqmx__pb2.CreateCIPulseChanFreqResponse.FromString,
                )
        self.CreateCIPulseChanTicks = channel.unary_unary(
                '/nidaqmx_grpc.NiDAQmx/CreateCIPulseChanTicks',
                request_serializer=nidaqmx__pb2.CreateCIPulseChanTicksRequest.SerializeToString,
                response_deserializer=nidaqmx__pb2.CreateCIPulseChanTicksResponse.FromString,
                )
        self.CreateCIPulseChanTime = channel.unary_unary(
                '/nidaqmx_grpc.NiDAQmx/CreateCIPulseChanTime',
                request_serializer=nidaqmx__pb2.CreateCIPulseChanTimeRequest.SerializeToString,
                response_deserializer=nidaqmx__pb2.CreateCIPulseChanTimeResponse.FromString,
                )
        self.CreateCIPulseWidthChan = channel.unary_unary(
                '/nidaqmx_grpc.NiDAQmx/CreateCIPulseWidthChan',
                request_serializer=nidaqmx__pb2.CreateCIPulseWidthChanRequest.SerializeToString,
                response_deserializer=nidaqmx__pb2.CreateCIPulseWidthChanResponse.FromString,
                )
        self.CreateCISemiPeriodChan = channel.unary_unary(
                '/nidaqmx_grpc.NiDAQmx/CreateCISemiPeriodChan',
                request_serializer=nidaqmx__pb2.CreateCISemiPeriodChanRequest.SerializeToString,
                response_deserializer=nidaqmx__pb2.CreateCISemiPeriodChanResponse.FromString,
                )
        self.CreateCITwoEdgeSepChan = channel.unary_unary(
                '/nidaqmx_grpc.NiDAQmx/CreateCITwoEdgeSepChan',
                request_serializer=nidaqmx__pb2.CreateCITwoEdgeSepChanRequest.SerializeToString,
                response_deserializer=nidaqmx__pb2.CreateCITwoEdgeSepChanResponse.FromString,
                )
        self.CreateCOPulseChanFreq = channel.unary_unary(
                '/nidaqmx_grpc.NiDAQmx/CreateCOPulseChanFreq',
                request_serializer=nidaqmx__pb2.CreateCOPulseChanFreqRequest.SerializeToString,
                response_deserializer=nidaqmx__pb2.CreateCOPulseChanFreqResponse.FromString,
                )
        self.CreateCOPulseChanTicks = channel.unary_unary(
                '/nidaqmx_grpc.NiDAQmx/CreateCOPulseChanTicks',
                request_serializer=nidaqmx__pb2.CreateCOPulseChanTicksRequest.SerializeToString,
                response_deserializer=nidaqmx__pb2.CreateCOPulseChanTicksResponse.FromString,
                )
        self.CreateCOPulseChanTime = channel.unary_unary(
                '/nidaqmx_grpc.NiDAQmx/CreateCOPulseChanTime',
                request_serializer=nidaqmx__pb2.CreateCOPulseChanTimeRequest.SerializeToString,
                response_deserializer=nidaqmx__pb2.CreateCOPulseChanTimeResponse.FromString,
                )
        self.CreateDIChan = channel.unary_unary(
                '/nidaqmx_grpc.NiDAQmx/CreateDIChan',
                request_serializer=nidaqmx__pb2.CreateDIChanRequest.SerializeToString,
                response_deserializer=nidaqmx__pb2.CreateDIChanResponse.FromString,
                )
        self.CreateDOChan = channel.unary_unary(
                '/nidaqmx_grpc.NiDAQmx/CreateDOChan',
                request_serializer=nidaqmx__pb2.CreateDOChanRequest.SerializeToString,
                response_deserializer=nidaqmx__pb2.CreateDOChanResponse.FromString,
                )
        self.CreateLinScale = channel.unary_unary(
                '/nidaqmx_grpc.NiDAQmx/CreateLinScale',
                request_serializer=nidaqmx__pb2.CreateLinScaleRequest.SerializeToString,
                response_deserializer=nidaqmx__pb2.CreateLinScaleResponse.FromString,
                )
        self.CreateMapScale = channel.unary_unary(
                '/nidaqmx_grpc.NiDAQmx/CreateMapScale',
                request_serializer=nidaqmx__pb2.CreateMapScaleRequest.SerializeToString,
                response_deserializer=nidaqmx__pb2.CreateMapScaleResponse.FromString,
                )
        self.CreatePolynomialScale = channel.unary_unary(
                '/nidaqmx_grpc.NiDAQmx/CreatePolynomialScale',
                request_serializer=nidaqmx__pb2.CreatePolynomialScaleRequest.SerializeToString,
                response_deserializer=nidaqmx__pb2.CreatePolynomialScaleResponse.FromString,
                )
        self.CreateTEDSAIAccelChan = channel.unary_unary(
                '/nidaqmx_grpc.NiDAQmx/CreateTEDSAIAccelChan',
                request_serializer=nidaqmx__pb2.CreateTEDSAIAccelChanRequest.SerializeToString,
                response_deserializer=nidaqmx__pb2.CreateTEDSAIAccelChanResponse.FromString,
                )
        self.CreateTEDSAIBridgeChan = channel.unary_unary(
                '/nidaqmx_grpc.NiDAQmx/CreateTEDSAIBridgeChan',
                request_serializer=nidaqmx__pb2.CreateTEDSAIBridgeChanRequest.SerializeToString,
                response_deserializer=nidaqmx__pb2.CreateTEDSAIBridgeChanResponse.FromString,
                )
        self.CreateTEDSAICurrentChan = channel.unary_unary(
                '/nidaqmx_grpc.NiDAQmx/CreateTEDSAICurrentChan',
                request_serializer=nidaqmx__pb2.CreateTEDSAICurrentChanRequest.SerializeToString,
                response_deserializer=nidaqmx__pb2.CreateTEDSAICurrentChanResponse.FromString,
                )
        self.CreateTEDSAIForceBridgeChan = channel.unary_unary(
                '/nidaqmx_grpc.NiDAQmx/CreateTEDSAIForceBridgeChan',
                request_serializer=nidaqmx__pb2.CreateTEDSAIForceBridgeChanRequest.SerializeToString,
                response_deserializer=nidaqmx__pb2.CreateTEDSAIForceBridgeChanResponse.FromString,
                )
        self.CreateTEDSAIForceIEPEChan = channel.unary_unary(
                '/nidaqmx_grpc.NiDAQmx/CreateTEDSAIForceIEPEChan',
                request_serializer=nidaqmx__pb2.CreateTEDSAIForceIEPEChanRequest.SerializeToString,
                response_deserializer=nidaqmx__pb2.CreateTEDSAIForceIEPEChanResponse.FromString,
                )
        self.CreateTEDSAIMicrophoneChan = channel.unary_unary(
                '/nidaqmx_grpc.NiDAQmx/CreateTEDSAIMicrophoneChan',
                request_serializer=nidaqmx__pb2.CreateTEDSAIMicrophoneChanRequest.SerializeToString,
                response_deserializer=nidaqmx__pb2.CreateTEDSAIMicrophoneChanResponse.FromString,
                )
        self.CreateTEDSAIPosLVDTChan = channel.unary_unary(
                '/nidaqmx_grpc.NiDAQmx/CreateTEDSAIPosLVDTChan',
                request_serializer=nidaqmx__pb2.CreateTEDSAIPosLVDTChanRequest.SerializeToString,
                response_deserializer=nidaqmx__pb2.CreateTEDSAIPosLVDTChanResponse.FromString,
                )
        self.CreateTEDSAIPosRVDTChan = channel.unary_unary(
                '/nidaqmx_grpc.NiDAQmx/CreateTEDSAIPosRVDTChan',
                request_serializer=nidaqmx__pb2.CreateTEDSAIPosRVDTChanRequest.SerializeToString,
                response_deserializer=nidaqmx__pb2.CreateTEDSAIPosRVDTChanResponse.FromString,
                )
        self.CreateTEDSAIPressureBridgeChan = channel.unary_unary(
                '/nidaqmx_grpc.NiDAQmx/CreateTEDSAIPressureBridgeChan',
                request_serializer=nidaqmx__pb2.CreateTEDSAIPressureBridgeChanRequest.SerializeToString,
                response_deserializer=nidaqmx__pb2.CreateTEDSAIPressureBridgeChanResponse.FromString,
                )
        self.CreateTEDSAIRTDChan = channel.unary_unary(
                '/nidaqmx_grpc.NiDAQmx/CreateTEDSAIRTDChan',
                request_serializer=nidaqmx__pb2.CreateTEDSAIRTDChanRequest.SerializeToString,
                response_deserializer=nidaqmx__pb2.CreateTEDSAIRTDChanResponse.FromString,
                )
        self.CreateTEDSAIResistanceChan = channel.unary_unary(
                '/nidaqmx_grpc.NiDAQmx/CreateTEDSAIResistanceChan',
                request_serializer=nidaqmx__pb2.CreateTEDSAIResistanceChanRequest.SerializeToString,
                response_deserializer=nidaqmx__pb2.CreateTEDSAIResistanceChanResponse.FromString,
                )
        self.CreateTEDSAIStrainGageChan = channel.unary_unary(
                '/nidaqmx_grpc.NiDAQmx/CreateTEDSAIStrainGageChan',
                request_serializer=nidaqmx__pb2.CreateTEDSAIStrainGageChanRequest.SerializeToString,
                response_deserializer=nidaqmx__pb2.CreateTEDSAIStrainGageChanResponse.FromString,
                )
        self.CreateTEDSAIThrmcplChan = channel.unary_unary(
                '/nidaqmx_grpc.NiDAQmx/CreateTEDSAIThrmcplChan',
                request_serializer=nidaqmx__pb2.CreateTEDSAIThrmcplChanRequest.SerializeToString,
                response_deserializer=nidaqmx__pb2.CreateTEDSAIThrmcplChanResponse.FromString,
                )
        self.CreateTEDSAIThrmstrChanIex = channel.unary_unary(
                '/nidaqmx_grpc.NiDAQmx/CreateTEDSAIThrmstrChanIex',
                request_serializer=nidaqmx__pb2.CreateTEDSAIThrmstrChanIexRequest.SerializeToString,
                response_deserializer=nidaqmx__pb2.CreateTEDSAIThrmstrChanIexResponse.FromString,
                )
        self.CreateTEDSAIThrmstrChanVex = channel.unary_unary(
                '/nidaqmx_grpc.NiDAQmx/CreateTEDSAIThrmstrChanVex',
                request_serializer=nidaqmx__pb2.CreateTEDSAIThrmstrChanVexRequest.SerializeToString,
                response_deserializer=nidaqmx__pb2.CreateTEDSAIThrmstrChanVexResponse.FromString,
                )
        self.CreateTEDSAITorqueBridgeChan = channel.unary_unary(
                '/nidaqmx_grpc.NiDAQmx/CreateTEDSAITorqueBridgeChan',
                request_serializer=nidaqmx__pb2.CreateTEDSAITorqueBridgeChanRequest.SerializeToString,
                response_deserializer=nidaqmx__pb2.CreateTEDSAITorqueBridgeChanResponse.FromString,
                )
        self.CreateTEDSAIVoltageChan = channel.unary_unary(
                '/nidaqmx_grpc.NiDAQmx/CreateTEDSAIVoltageChan',
                request_serializer=nidaqmx__pb2.CreateTEDSAIVoltageChanRequest.SerializeToString,
                response_deserializer=nidaqmx__pb2.CreateTEDSAIVoltageChanResponse.FromString,
                )
        self.CreateTEDSAIVoltageChanWithExcit = channel.unary_unary(
                '/nidaqmx_grpc.NiDAQmx/CreateTEDSAIVoltageChanWithExcit',
                request_serializer=nidaqmx__pb2.CreateTEDSAIVoltageChanWithExcitRequest.SerializeToString,
                response_deserializer=nidaqmx__pb2.CreateTEDSAIVoltageChanWithExcitResponse.FromString,
                )
        self.CreateTableScale = channel.unary_unary(
                '/nidaqmx_grpc.NiDAQmx/CreateTableScale',
                request_serializer=nidaqmx__pb2.CreateTableScaleRequest.SerializeToString,
                response_deserializer=nidaqmx__pb2.CreateTableScaleResponse.FromString,
                )
        self.CreateTask = channel.unary_unary(
                '/nidaqmx_grpc.NiDAQmx/CreateTask',
                request_serializer=nidaqmx__pb2.CreateTaskRequest.SerializeToString,
                response_deserializer=nidaqmx__pb2.CreateTaskResponse.FromString,
                )
        self.CreateWatchdogTimerTask = channel.unary_unary(
                '/nidaqmx_grpc.NiDAQmx/CreateWatchdogTimerTask',
                request_serializer=nidaqmx__pb2.CreateWatchdogTimerTaskRequest.SerializeToString,
                response_deserializer=nidaqmx__pb2.CreateWatchdogTimerTaskResponse.FromString,
                )
        self.CreateWatchdogTimerTaskEx = channel.unary_unary(
                '/nidaqmx_grpc.NiDAQmx/CreateWatchdogTimerTaskEx',
                request_serializer=nidaqmx__pb2.CreateWatchdogTimerTaskExRequest.SerializeToString,
                response_deserializer=nidaqmx__pb2.CreateWatchdogTimerTaskExResponse.FromString,
                )
        self.DeleteNetworkDevice = channel.unary_unary(
                '/nidaqmx_grpc.NiDAQmx/DeleteNetworkDevice',
                request_serializer=nidaqmx__pb2.DeleteNetworkDeviceRequest.SerializeToString,
                response_deserializer=nidaqmx__pb2.DeleteNetworkDeviceResponse.FromString,
                )
        self.DeleteSavedGlobalChan = channel.unary_unary(
                '/nidaqmx_grpc.NiDAQmx/DeleteSavedGlobalChan',
                request_serializer=nidaqmx__pb2.DeleteSavedGlobalChanRequest.SerializeToString,
                response_deserializer=nidaqmx__pb2.DeleteSavedGlobalChanResponse.FromString,
                )
        self.DeleteSavedScale = channel.unary_unary(
                '/nidaqmx_grpc.NiDAQmx/DeleteSavedScale',
                request_serializer=nidaqmx__pb2.DeleteSavedScaleRequest.SerializeToString,
                response_deserializer=nidaqmx__pb2.DeleteSavedScaleResponse.FromString,
                )
        self.DeleteSavedTask = channel.unary_unary(
                '/nidaqmx_grpc.NiDAQmx/DeleteSavedTask',
                request_serializer=nidaqmx__pb2.DeleteSavedTaskRequest.SerializeToString,
                response_deserializer=nidaqmx__pb2.DeleteSavedTaskResponse.FromString,
                )
        self.DeviceSupportsCal = channel.unary_unary(
                '/nidaqmx_grpc.NiDAQmx/DeviceSupportsCal',
                request_serializer=nidaqmx__pb2.DeviceSupportsCalRequest.SerializeToString,
                response_deserializer=nidaqmx__pb2.DeviceSupportsCalResponse.FromString,
                )
        self.DisableRefTrig = channel.unary_unary(
                '/nidaqmx_grpc.NiDAQmx/DisableRefTrig',
                request_serializer=nidaqmx__pb2.DisableRefTrigRequest.SerializeToString,
                response_deserializer=nidaqmx__pb2.DisableRefTrigResponse.FromString,
                )
        self.DisableStartTrig = channel.unary_unary(
                '/nidaqmx_grpc.NiDAQmx/DisableStartTrig',
                request_serializer=nidaqmx__pb2.DisableStartTrigRequest.SerializeToString,
                response_deserializer=nidaqmx__pb2.DisableStartTrigResponse.FromString,
                )
        self.DisconnectTerms = channel.unary_unary(
                '/nidaqmx_grpc.NiDAQmx/DisconnectTerms',
                request_serializer=nidaqmx__pb2.DisconnectTermsRequest.SerializeToString,
                response_deserializer=nidaqmx__pb2.DisconnectTermsResponse.FromString,
                )
        self.ExportSignal = channel.unary_unary(
                '/nidaqmx_grpc.NiDAQmx/ExportSignal',
                request_serializer=nidaqmx__pb2.ExportSignalRequest.SerializeToString,
                response_deserializer=nidaqmx__pb2.ExportSignalResponse.FromString,
                )
        self.GetAIChanCalCalDate = channel.unary_unary(
                '/nidaqmx_grpc.NiDAQmx/GetAIChanCalCalDate',
                request_serializer=nidaqmx__pb2.GetAIChanCalCalDateRequest.SerializeToString,
                response_deserializer=nidaqmx__pb2.GetAIChanCalCalDateResponse.FromString,
                )
        self.GetAIChanCalExpDate = channel.unary_unary(
                '/nidaqmx_grpc.NiDAQmx/GetAIChanCalExpDate',
                request_serializer=nidaqmx__pb2.GetAIChanCalExpDateRequest.SerializeToString,
                response_deserializer=nidaqmx__pb2.GetAIChanCalExpDateResponse.FromString,
                )
        self.GetAnalogPowerUpStates = channel.unary_unary(
                '/nidaqmx_grpc.NiDAQmx/GetAnalogPowerUpStates',
                request_serializer=nidaqmx__pb2.GetAnalogPowerUpStatesRequest.SerializeToString,
                response_deserializer=nidaqmx__pb2.GetAnalogPowerUpStatesResponse.FromString,
                )
        self.GetAnalogPowerUpStatesWithOutputType = channel.unary_unary(
                '/nidaqmx_grpc.NiDAQmx/GetAnalogPowerUpStatesWithOutputType',
                request_serializer=nidaqmx__pb2.GetAnalogPowerUpStatesWithOutputTypeRequest.SerializeToString,
                response_deserializer=nidaqmx__pb2.GetAnalogPowerUpStatesWithOutputTypeResponse.FromString,
                )
        self.GetArmStartTrigTimestampVal = channel.unary_unary(
                '/nidaqmx_grpc.NiDAQmx/GetArmStartTrigTimestampVal',
                request_serializer=nidaqmx__pb2.GetArmStartTrigTimestampValRequest.SerializeToString,
                response_deserializer=nidaqmx__pb2.GetArmStartTrigTimestampValResponse.FromString,
                )
        self.GetArmStartTrigTrigWhen = channel.unary_unary(
                '/nidaqmx_grpc.NiDAQmx/GetArmStartTrigTrigWhen',
                request_serializer=nidaqmx__pb2.GetArmStartTrigTrigWhenRequest.SerializeToString,
                response_deserializer=nidaqmx__pb2.GetArmStartTrigTrigWhenResponse.FromString,
                )
        self.GetAutoConfiguredCDAQSyncConnections = channel.unary_unary(
                '/nidaqmx_grpc.NiDAQmx/GetAutoConfiguredCDAQSyncConnections',
                request_serializer=nidaqmx__pb2.GetAutoConfiguredCDAQSyncConnectionsRequest.SerializeToString,
                response_deserializer=nidaqmx__pb2.GetAutoConfiguredCDAQSyncConnectionsResponse.FromString,
                )
        self.GetBufferAttributeUInt32 = channel.unary_unary(
                '/nidaqmx_grpc.NiDAQmx/GetBufferAttributeUInt32',
                request_serializer=nidaqmx__pb2.GetBufferAttributeUInt32Request.SerializeToString,
                response_deserializer=nidaqmx__pb2.GetBufferAttributeUInt32Response.FromString,
                )
        self.GetCalInfoAttributeBool = channel.unary_unary(
                '/nidaqmx_grpc.NiDAQmx/GetCalInfoAttributeBool',
                request_serializer=nidaqmx__pb2.GetCalInfoAttributeBoolRequest.SerializeToString,
                response_deserializer=nidaqmx__pb2.GetCalInfoAttributeBoolResponse.FromString,
                )
        self.GetCalInfoAttributeDouble = channel.unary_unary(
                '/nidaqmx_grpc.NiDAQmx/GetCalInfoAttributeDouble',
                request_serializer=nidaqmx__pb2.GetCalInfoAttributeDoubleRequest.SerializeToString,
                response_deserializer=nidaqmx__pb2.GetCalInfoAttributeDoubleResponse.FromString,
                )
        self.GetCalInfoAttributeString = channel.unary_unary(
                '/nidaqmx_grpc.NiDAQmx/GetCalInfoAttributeString',
                request_serializer=nidaqmx__pb2.GetCalInfoAttributeStringRequest.SerializeToString,
                response_deserializer=nidaqmx__pb2.GetCalInfoAttributeStringResponse.FromString,
                )
        self.GetCalInfoAttributeUInt32 = channel.unary_unary(
                '/nidaqmx_grpc.NiDAQmx/GetCalInfoAttributeUInt32',
                request_serializer=nidaqmx__pb2.GetCalInfoAttributeUInt32Request.SerializeToString,
                response_deserializer=nidaqmx__pb2.GetCalInfoAttributeUInt32Response.FromString,
                )
        self.GetChanAttributeBool = channel.unary_unary(
                '/nidaqmx_grpc.NiDAQmx/GetChanAttributeBool',
                request_serializer=nidaqmx__pb2.GetChanAttributeBoolRequest.SerializeToString,
                response_deserializer=nidaqmx__pb2.GetChanAttributeBoolResponse.FromString,
                )
        self.GetChanAttributeDouble = channel.unary_unary(
                '/nidaqmx_grpc.NiDAQmx/GetChanAttributeDouble',
                request_serializer=nidaqmx__pb2.GetChanAttributeDoubleRequest.SerializeToString,
                response_deserializer=nidaqmx__pb2.GetChanAttributeDoubleResponse.FromString,
                )
        self.GetChanAttributeDoubleArray = channel.unary_unary(
                '/nidaqmx_grpc.NiDAQmx/GetChanAttributeDoubleArray',
                request_serializer=nidaqmx__pb2.GetChanAttributeDoubleArrayRequest.SerializeToString,
                response_deserializer=nidaqmx__pb2.GetChanAttributeDoubleArrayResponse.FromString,
                )
        self.GetChanAttributeInt32 = channel.unary_unary(
                '/nidaqmx_grpc.NiDAQmx/GetChanAttributeInt32',
                request_serializer=nidaqmx__pb2.GetChanAttributeInt32Request.SerializeToString,
                response_deserializer=nidaqmx__pb2.GetChanAttributeInt32Response.FromString,
                )
        self.GetChanAttributeString = channel.unary_unary(
                '/nidaqmx_grpc.NiDAQmx/GetChanAttributeString',
                request_serializer=nidaqmx__pb2.GetChanAttributeStringRequest.SerializeToString,
                response_deserializer=nidaqmx__pb2.GetChanAttributeStringResponse.FromString,
                )
        self.GetChanAttributeUInt32 = channel.unary_unary(
                '/nidaqmx_grpc.NiDAQmx/GetChanAttributeUInt32',
                request_serializer=nidaqmx__pb2.GetChanAttributeUInt32Request.SerializeToString,
                response_deserializer=nidaqmx__pb2.GetChanAttributeUInt32Response.FromString,
                )
        self.GetDeviceAttributeBool = channel.unary_unary(
                '/nidaqmx_grpc.NiDAQmx/GetDeviceAttributeBool',
                request_serializer=nidaqmx__pb2.GetDeviceAttributeBoolRequest.SerializeToString,
                response_deserializer=nidaqmx__pb2.GetDeviceAttributeBoolResponse.FromString,
                )
        self.GetDeviceAttributeDouble = channel.unary_unary(
                '/nidaqmx_grpc.NiDAQmx/GetDeviceAttributeDouble',
                request_serializer=nidaqmx__pb2.GetDeviceAttributeDoubleRequest.SerializeToString,
                response_deserializer=nidaqmx__pb2.GetDeviceAttributeDoubleResponse.FromString,
                )
        self.GetDeviceAttributeDoubleArray = channel.unary_unary(
                '/nidaqmx_grpc.NiDAQmx/GetDeviceAttributeDoubleArray',
                request_serializer=nidaqmx__pb2.GetDeviceAttributeDoubleArrayRequest.SerializeToString,
                response_deserializer=nidaqmx__pb2.GetDeviceAttributeDoubleArrayResponse.FromString,
                )
        self.GetDeviceAttributeInt32 = channel.unary_unary(
                '/nidaqmx_grpc.NiDAQmx/GetDeviceAttributeInt32',
                request_serializer=nidaqmx__pb2.GetDeviceAttributeInt32Request.SerializeToString,
                response_deserializer=nidaqmx__pb2.GetDeviceAttributeInt32Response.FromString,
                )
        self.GetDeviceAttributeInt32Array = channel.unary_unary(
                '/nidaqmx_grpc.NiDAQmx/GetDeviceAttributeInt32Array',
                request_serializer=nidaqmx__pb2.GetDeviceAttributeInt32ArrayRequest.SerializeToString,
                response_deserializer=nidaqmx__pb2.GetDeviceAttributeInt32ArrayResponse.FromString,
                )
        self.GetDeviceAttributeString = channel.unary_unary(
                '/nidaqmx_grpc.NiDAQmx/GetDeviceAttributeString',
                request_serializer=nidaqmx__pb2.GetDeviceAttributeStringRequest.SerializeToString,
                response_deserializer=nidaqmx__pb2.GetDeviceAttributeStringResponse.FromString,
                )
        self.GetDeviceAttributeUInt32 = channel.unary_unary(
                '/nidaqmx_grpc.NiDAQmx/GetDeviceAttributeUInt32',
                request_serializer=nidaqmx__pb2.GetDeviceAttributeUInt32Request.SerializeToString,
                response_deserializer=nidaqmx__pb2.GetDeviceAttributeUInt32Response.FromString,
                )
        self.GetDeviceAttributeUInt32Array = channel.unary_unary(
                '/nidaqmx_grpc.NiDAQmx/GetDeviceAttributeUInt32Array',
                request_serializer=nidaqmx__pb2.GetDeviceAttributeUInt32ArrayRequest.SerializeToString,
                response_deserializer=nidaqmx__pb2.GetDeviceAttributeUInt32ArrayResponse.FromString,
                )
        self.GetDigitalLogicFamilyPowerUpState = channel.unary_unary(
                '/nidaqmx_grpc.NiDAQmx/GetDigitalLogicFamilyPowerUpState',
                request_serializer=nidaqmx__pb2.GetDigitalLogicFamilyPowerUpStateRequest.SerializeToString,
                response_deserializer=nidaqmx__pb2.GetDigitalLogicFamilyPowerUpStateResponse.FromString,
                )
        self.GetDigitalPowerUpStates = channel.unary_unary(
                '/nidaqmx_grpc.NiDAQmx/GetDigitalPowerUpStates',
                request_serializer=nidaqmx__pb2.GetDigitalPowerUpStatesRequest.SerializeToString,
                response_deserializer=nidaqmx__pb2.GetDigitalPowerUpStatesResponse.FromString,
                )
        self.GetDigitalPullUpPullDownStates = channel.unary_unary(
                '/nidaqmx_grpc.NiDAQmx/GetDigitalPullUpPullDownStates',
                request_serializer=nidaqmx__pb2.GetDigitalPullUpPullDownStatesRequest.SerializeToString,
                response_deserializer=nidaqmx__pb2.GetDigitalPullUpPullDownStatesResponse.FromString,
                )
        self.GetDisconnectedCDAQSyncPorts = channel.unary_unary(
                '/nidaqmx_grpc.NiDAQmx/GetDisconnectedCDAQSyncPorts',
                request_serializer=nidaqmx__pb2.GetDisconnectedCDAQSyncPortsRequest.SerializeToString,
                response_deserializer=nidaqmx__pb2.GetDisconnectedCDAQSyncPortsResponse.FromString,
                )
        self.GetErrorString = channel.unary_unary(
                '/nidaqmx_grpc.NiDAQmx/GetErrorString',
                request_serializer=nidaqmx__pb2.GetErrorStringRequest.SerializeToString,
                response_deserializer=nidaqmx__pb2.GetErrorStringResponse.FromString,
                )
        self.GetExportedSignalAttributeBool = channel.unary_unary(
                '/nidaqmx_grpc.NiDAQmx/GetExportedSignalAttributeBool',
                request_serializer=nidaqmx__pb2.GetExportedSignalAttributeBoolRequest.SerializeToString,
                response_deserializer=nidaqmx__pb2.GetExportedSignalAttributeBoolResponse.FromString,
                )
        self.GetExportedSignalAttributeDouble = channel.unary_unary(
                '/nidaqmx_grpc.NiDAQmx/GetExportedSignalAttributeDouble',
                request_serializer=nidaqmx__pb2.GetExportedSignalAttributeDoubleRequest.SerializeToString,
                response_deserializer=nidaqmx__pb2.GetExportedSignalAttributeDoubleResponse.FromString,
                )
        self.GetExportedSignalAttributeInt32 = channel.unary_unary(
                '/nidaqmx_grpc.NiDAQmx/GetExportedSignalAttributeInt32',
                request_serializer=nidaqmx__pb2.GetExportedSignalAttributeInt32Request.SerializeToString,
                response_deserializer=nidaqmx__pb2.GetExportedSignalAttributeInt32Response.FromString,
                )
        self.GetExportedSignalAttributeString = channel.unary_unary(
                '/nidaqmx_grpc.NiDAQmx/GetExportedSignalAttributeString',
                request_serializer=nidaqmx__pb2.GetExportedSignalAttributeStringRequest.SerializeToString,
                response_deserializer=nidaqmx__pb2.GetExportedSignalAttributeStringResponse.FromString,
                )
        self.GetExportedSignalAttributeUInt32 = channel.unary_unary(
                '/nidaqmx_grpc.NiDAQmx/GetExportedSignalAttributeUInt32',
                request_serializer=nidaqmx__pb2.GetExportedSignalAttributeUInt32Request.SerializeToString,
                response_deserializer=nidaqmx__pb2.GetExportedSignalAttributeUInt32Response.FromString,
                )
        self.GetExtCalLastDateAndTime = channel.unary_unary(
                '/nidaqmx_grpc.NiDAQmx/GetExtCalLastDateAndTime',
                request_serializer=nidaqmx__pb2.GetExtCalLastDateAndTimeRequest.SerializeToString,
                response_deserializer=nidaqmx__pb2.GetExtCalLastDateAndTimeResponse.FromString,
                )
        self.GetFirstSampClkWhen = channel.unary_unary(
                '/nidaqmx_grpc.NiDAQmx/GetFirstSampClkWhen',
                request_serializer=nidaqmx__pb2.GetFirstSampClkWhenRequest.SerializeToString,
                response_deserializer=nidaqmx__pb2.GetFirstSampClkWhenResponse.FromString,
                )
        self.GetFirstSampTimestampVal = channel.unary_unary(
                '/nidaqmx_grpc.NiDAQmx/GetFirstSampTimestampVal',
                request_serializer=nidaqmx__pb2.GetFirstSampTimestampValRequest.SerializeToString,
                response_deserializer=nidaqmx__pb2.GetFirstSampTimestampValResponse.FromString,
                )
        self.GetNthTaskChannel = channel.unary_unary(
                '/nidaqmx_grpc.NiDAQmx/GetNthTaskChannel',
                request_serializer=nidaqmx__pb2.GetNthTaskChannelRequest.SerializeToString,
                response_deserializer=nidaqmx__pb2.GetNthTaskChannelResponse.FromString,
                )
        self.GetNthTaskDevice = channel.unary_unary(
                '/nidaqmx_grpc.NiDAQmx/GetNthTaskDevice',
                request_serializer=nidaqmx__pb2.GetNthTaskDeviceRequest.SerializeToString,
                response_deserializer=nidaqmx__pb2.GetNthTaskDeviceResponse.FromString,
                )
        self.GetNthTaskReadChannel = channel.unary_unary(
                '/nidaqmx_grpc.NiDAQmx/GetNthTaskReadChannel',
                request_serializer=nidaqmx__pb2.GetNthTaskReadChannelRequest.SerializeToString,
                response_deserializer=nidaqmx__pb2.GetNthTaskReadChannelResponse.FromString,
                )
        self.GetPersistedChanAttributeBool = channel.unary_unary(
                '/nidaqmx_grpc.NiDAQmx/GetPersistedChanAttributeBool',
                request_serializer=nidaqmx__pb2.GetPersistedChanAttributeBoolRequest.SerializeToString,
                response_deserializer=nidaqmx__pb2.GetPersistedChanAttributeBoolResponse.FromString,
                )
        self.GetPersistedChanAttributeString = channel.unary_unary(
                '/nidaqmx_grpc.NiDAQmx/GetPersistedChanAttributeString',
                request_serializer=nidaqmx__pb2.GetPersistedChanAttributeStringRequest.SerializeToString,
                response_deserializer=nidaqmx__pb2.GetPersistedChanAttributeStringResponse.FromString,
                )
        self.GetPersistedScaleAttributeBool = channel.unary_unary(
                '/nidaqmx_grpc.NiDAQmx/GetPersistedScaleAttributeBool',
                request_serializer=nidaqmx__pb2.GetPersistedScaleAttributeBoolRequest.SerializeToString,
                response_deserializer=nidaqmx__pb2.GetPersistedScaleAttributeBoolResponse.FromString,
                )
        self.GetPersistedScaleAttributeString = channel.unary_unary(
                '/nidaqmx_grpc.NiDAQmx/GetPersistedScaleAttributeString',
                request_serializer=nidaqmx__pb2.GetPersistedScaleAttributeStringRequest.SerializeToString,
                response_deserializer=nidaqmx__pb2.GetPersistedScaleAttributeStringResponse.FromString,
                )
        self.GetPersistedTaskAttributeBool = channel.unary_unary(
                '/nidaqmx_grpc.NiDAQmx/GetPersistedTaskAttributeBool',
                request_serializer=nidaqmx__pb2.GetPersistedTaskAttributeBoolRequest.SerializeToString,
                response_deserializer=nidaqmx__pb2.GetPersistedTaskAttributeBoolResponse.FromString,
                )
        self.GetPersistedTaskAttributeString = channel.unary_unary(
                '/nidaqmx_grpc.NiDAQmx/GetPersistedTaskAttributeString',
                request_serializer=nidaqmx__pb2.GetPersistedTaskAttributeStringRequest.SerializeToString,
                response_deserializer=nidaqmx__pb2.GetPersistedTaskAttributeStringResponse.FromString,
                )
        self.GetPhysicalChanAttributeBool = channel.unary_unary(
                '/nidaqmx_grpc.NiDAQmx/GetPhysicalChanAttributeBool',
                request_serializer=nidaqmx__pb2.GetPhysicalChanAttributeBoolRequest.SerializeToString,
                response_deserializer=nidaqmx__pb2.GetPhysicalChanAttributeBoolResponse.FromString,
                )
        self.GetPhysicalChanAttributeBytes = channel.unary_unary(
                '/nidaqmx_grpc.NiDAQmx/GetPhysicalChanAttributeBytes',
                request_serializer=nidaqmx__pb2.GetPhysicalChanAttributeBytesRequest.SerializeToString,
                response_deserializer=nidaqmx__pb2.GetPhysicalChanAttributeBytesResponse.FromString,
                )
        self.GetPhysicalChanAttributeDouble = channel.unary_unary(
                '/nidaqmx_grpc.NiDAQmx/GetPhysicalChanAttributeDouble',
                request_serializer=nidaqmx__pb2.GetPhysicalChanAttributeDoubleRequest.SerializeToString,
                response_deserializer=nidaqmx__pb2.GetPhysicalChanAttributeDoubleResponse.FromString,
                )
        self.GetPhysicalChanAttributeDoubleArray = channel.unary_unary(
                '/nidaqmx_grpc.NiDAQmx/GetPhysicalChanAttributeDoubleArray',
                request_serializer=nidaqmx__pb2.GetPhysicalChanAttributeDoubleArrayRequest.SerializeToString,
                response_deserializer=nidaqmx__pb2.GetPhysicalChanAttributeDoubleArrayResponse.FromString,
                )
        self.GetPhysicalChanAttributeInt32 = channel.unary_unary(
                '/nidaqmx_grpc.NiDAQmx/GetPhysicalChanAttributeInt32',
                request_serializer=nidaqmx__pb2.GetPhysicalChanAttributeInt32Request.SerializeToString,
                response_deserializer=nidaqmx__pb2.GetPhysicalChanAttributeInt32Response.FromString,
                )
        self.GetPhysicalChanAttributeInt32Array = channel.unary_unary(
                '/nidaqmx_grpc.NiDAQmx/GetPhysicalChanAttributeInt32Array',
                request_serializer=nidaqmx__pb2.GetPhysicalChanAttributeInt32ArrayRequest.SerializeToString,
                response_deserializer=nidaqmx__pb2.GetPhysicalChanAttributeInt32ArrayResponse.FromString,
                )
        self.GetPhysicalChanAttributeString = channel.unary_unary(
                '/nidaqmx_grpc.NiDAQmx/GetPhysicalChanAttributeString',
                request_serializer=nidaqmx__pb2.GetPhysicalChanAttributeStringRequest.SerializeToString,
                response_deserializer=nidaqmx__pb2.GetPhysicalChanAttributeStringResponse.FromString,
                )
        self.GetPhysicalChanAttributeUInt32 = channel.unary_unary(
                '/nidaqmx_grpc.NiDAQmx/GetPhysicalChanAttributeUInt32',
                request_serializer=nidaqmx__pb2.GetPhysicalChanAttributeUInt32Request.SerializeToString,
                response_deserializer=nidaqmx__pb2.GetPhysicalChanAttributeUInt32Response.FromString,
                )
        self.GetPhysicalChanAttributeUInt32Array = channel.unary_unary(
                '/nidaqmx_grpc.NiDAQmx/GetPhysicalChanAttributeUInt32Array',
                request_serializer=nidaqmx__pb2.GetPhysicalChanAttributeUInt32ArrayRequest.SerializeToString,
                response_deserializer=nidaqmx__pb2.GetPhysicalChanAttributeUInt32ArrayResponse.FromString,
                )
        self.GetReadAttributeBool = channel.unary_unary(
                '/nidaqmx_grpc.NiDAQmx/GetReadAttributeBool',
                request_serializer=nidaqmx__pb2.GetReadAttributeBoolRequest.SerializeToString,
                response_deserializer=nidaqmx__pb2.GetReadAttributeBoolResponse.FromString,
                )
        self.GetReadAttributeDouble = channel.unary_unary(
                '/nidaqmx_grpc.NiDAQmx/GetReadAttributeDouble',
                request_serializer=nidaqmx__pb2.GetReadAttributeDoubleRequest.SerializeToString,
                response_deserializer=nidaqmx__pb2.GetReadAttributeDoubleResponse.FromString,
                )
        self.GetReadAttributeInt32 = channel.unary_unary(
                '/nidaqmx_grpc.NiDAQmx/GetReadAttributeInt32',
                request_serializer=nidaqmx__pb2.GetReadAttributeInt32Request.SerializeToString,
                response_deserializer=nidaqmx__pb2.GetReadAttributeInt32Response.FromString,
                )
        self.GetReadAttributeString = channel.unary_unary(
                '/nidaqmx_grpc.NiDAQmx/GetReadAttributeString',
                request_serializer=nidaqmx__pb2.GetReadAttributeStringRequest.SerializeToString,
                response_deserializer=nidaqmx__pb2.GetReadAttributeStringResponse.FromString,
                )
        self.GetReadAttributeUInt32 = channel.unary_unary(
                '/nidaqmx_grpc.NiDAQmx/GetReadAttributeUInt32',
                request_serializer=nidaqmx__pb2.GetReadAttributeUInt32Request.SerializeToString,
                response_deserializer=nidaqmx__pb2.GetReadAttributeUInt32Response.FromString,
                )
        self.GetReadAttributeUInt64 = channel.unary_unary(
                '/nidaqmx_grpc.NiDAQmx/GetReadAttributeUInt64',
                request_serializer=nidaqmx__pb2.GetReadAttributeUInt64Request.SerializeToString,
                response_deserializer=nidaqmx__pb2.GetReadAttributeUInt64Response.FromString,
                )
        self.GetRealTimeAttributeBool = channel.unary_unary(
                '/nidaqmx_grpc.NiDAQmx/GetRealTimeAttributeBool',
                request_serializer=nidaqmx__pb2.GetRealTimeAttributeBoolRequest.SerializeToString,
                response_deserializer=nidaqmx__pb2.GetRealTimeAttributeBoolResponse.FromString,
                )
        self.GetRealTimeAttributeInt32 = channel.unary_unary(
                '/nidaqmx_grpc.NiDAQmx/GetRealTimeAttributeInt32',
                request_serializer=nidaqmx__pb2.GetRealTimeAttributeInt32Request.SerializeToString,
                response_deserializer=nidaqmx__pb2.GetRealTimeAttributeInt32Response.FromString,
                )
        self.GetRealTimeAttributeUInt32 = channel.unary_unary(
                '/nidaqmx_grpc.NiDAQmx/GetRealTimeAttributeUInt32',
                request_serializer=nidaqmx__pb2.GetRealTimeAttributeUInt32Request.SerializeToString,
                response_deserializer=nidaqmx__pb2.GetRealTimeAttributeUInt32Response.FromString,
                )
        self.GetRefTrigTimestampVal = channel.unary_unary(
                '/nidaqmx_grpc.NiDAQmx/GetRefTrigTimestampVal',
                request_serializer=nidaqmx__pb2.GetRefTrigTimestampValRequest.SerializeToString,
                response_deserializer=nidaqmx__pb2.GetRefTrigTimestampValResponse.FromString,
                )
        self.GetScaleAttributeDouble = channel.unary_unary(
                '/nidaqmx_grpc.NiDAQmx/GetScaleAttributeDouble',
                request_serializer=nidaqmx__pb2.GetScaleAttributeDoubleRequest.SerializeToString,
                response_deserializer=nidaqmx__pb2.GetScaleAttributeDoubleResponse.FromString,
                )
        self.GetScaleAttributeDoubleArray = channel.unary_unary(
                '/nidaqmx_grpc.NiDAQmx/GetScaleAttributeDoubleArray',
                request_serializer=nidaqmx__pb2.GetScaleAttributeDoubleArrayRequest.SerializeToString,
                response_deserializer=nidaqmx__pb2.GetScaleAttributeDoubleArrayResponse.FromString,
                )
        self.GetScaleAttributeInt32 = channel.unary_unary(
                '/nidaqmx_grpc.NiDAQmx/GetScaleAttributeInt32',
                request_serializer=nidaqmx__pb2.GetScaleAttributeInt32Request.SerializeToString,
                response_deserializer=nidaqmx__pb2.GetScaleAttributeInt32Response.FromString,
                )
        self.GetScaleAttributeString = channel.unary_unary(
                '/nidaqmx_grpc.NiDAQmx/GetScaleAttributeString',
                request_serializer=nidaqmx__pb2.GetScaleAttributeStringRequest.SerializeToString,
                response_deserializer=nidaqmx__pb2.GetScaleAttributeStringResponse.FromString,
                )
        self.GetSelfCalLastDateAndTime = channel.unary_unary(
                '/nidaqmx_grpc.NiDAQmx/GetSelfCalLastDateAndTime',
                request_serializer=nidaqmx__pb2.GetSelfCalLastDateAndTimeRequest.SerializeToString,
                response_deserializer=nidaqmx__pb2.GetSelfCalLastDateAndTimeResponse.FromString,
                )
        self.GetStartTrigTimestampVal = channel.unary_unary(
                '/nidaqmx_grpc.NiDAQmx/GetStartTrigTimestampVal',
                request_serializer=nidaqmx__pb2.GetStartTrigTimestampValRequest.SerializeToString,
                response_deserializer=nidaqmx__pb2.GetStartTrigTimestampValResponse.FromString,
                )
        self.GetStartTrigTrigWhen = channel.unary_unary(
                '/nidaqmx_grpc.NiDAQmx/GetStartTrigTrigWhen',
                request_serializer=nidaqmx__pb2.GetStartTrigTrigWhenRequest.SerializeToString,
                response_deserializer=nidaqmx__pb2.GetStartTrigTrigWhenResponse.FromString,
                )
        self.GetSyncPulseTimeWhen = channel.unary_unary(
                '/nidaqmx_grpc.NiDAQmx/GetSyncPulseTimeWhen',
                request_serializer=nidaqmx__pb2.GetSyncPulseTimeWhenRequest.SerializeToString,
                response_deserializer=nidaqmx__pb2.GetSyncPulseTimeWhenResponse.FromString,
                )
        self.GetSystemInfoAttributeString = channel.unary_unary(
                '/nidaqmx_grpc.NiDAQmx/GetSystemInfoAttributeString',
                request_serializer=nidaqmx__pb2.GetSystemInfoAttributeStringRequest.SerializeToString,
                response_deserializer=nidaqmx__pb2.GetSystemInfoAttributeStringResponse.FromString,
                )
        self.GetSystemInfoAttributeUInt32 = channel.unary_unary(
                '/nidaqmx_grpc.NiDAQmx/GetSystemInfoAttributeUInt32',
                request_serializer=nidaqmx__pb2.GetSystemInfoAttributeUInt32Request.SerializeToString,
                response_deserializer=nidaqmx__pb2.GetSystemInfoAttributeUInt32Response.FromString,
                )
        self.GetTaskAttributeBool = channel.unary_unary(
                '/nidaqmx_grpc.NiDAQmx/GetTaskAttributeBool',
                request_serializer=nidaqmx__pb2.GetTaskAttributeBoolRequest.SerializeToString,
                response_deserializer=nidaqmx__pb2.GetTaskAttributeBoolResponse.FromString,
                )
        self.GetTaskAttributeString = channel.unary_unary(
                '/nidaqmx_grpc.NiDAQmx/GetTaskAttributeString',
                request_serializer=nidaqmx__pb2.GetTaskAttributeStringRequest.SerializeToString,
                response_deserializer=nidaqmx__pb2.GetTaskAttributeStringResponse.FromString,
                )
        self.GetTaskAttributeUInt32 = channel.unary_unary(
                '/nidaqmx_grpc.NiDAQmx/GetTaskAttributeUInt32',
                request_serializer=nidaqmx__pb2.GetTaskAttributeUInt32Request.SerializeToString,
                response_deserializer=nidaqmx__pb2.GetTaskAttributeUInt32Response.FromString,
                )
        self.GetTimingAttributeBool = channel.unary_unary(
                '/nidaqmx_grpc.NiDAQmx/GetTimingAttributeBool',
                request_serializer=nidaqmx__pb2.GetTimingAttributeBoolRequest.SerializeToString,
                response_deserializer=nidaqmx__pb2.GetTimingAttributeBoolResponse.FromString,
                )
        self.GetTimingAttributeDouble = channel.unary_unary(
                '/nidaqmx_grpc.NiDAQmx/GetTimingAttributeDouble',
                request_serializer=nidaqmx__pb2.GetTimingAttributeDoubleRequest.SerializeToString,
                response_deserializer=nidaqmx__pb2.GetTimingAttributeDoubleResponse.FromString,
                )
        self.GetTimingAttributeExBool = channel.unary_unary(
                '/nidaqmx_grpc.NiDAQmx/GetTimingAttributeExBool',
                request_serializer=nidaqmx__pb2.GetTimingAttributeExBoolRequest.SerializeToString,
                response_deserializer=nidaqmx__pb2.GetTimingAttributeExBoolResponse.FromString,
                )
        self.GetTimingAttributeExDouble = channel.unary_unary(
                '/nidaqmx_grpc.NiDAQmx/GetTimingAttributeExDouble',
                request_serializer=nidaqmx__pb2.GetTimingAttributeExDoubleRequest.SerializeToString,
                response_deserializer=nidaqmx__pb2.GetTimingAttributeExDoubleResponse.FromString,
                )
        self.GetTimingAttributeExInt32 = channel.unary_unary(
                '/nidaqmx_grpc.NiDAQmx/GetTimingAttributeExInt32',
                request_serializer=nidaqmx__pb2.GetTimingAttributeExInt32Request.SerializeToString,
                response_deserializer=nidaqmx__pb2.GetTimingAttributeExInt32Response.FromString,
                )
        self.GetTimingAttributeExString = channel.unary_unary(
                '/nidaqmx_grpc.NiDAQmx/GetTimingAttributeExString',
                request_serializer=nidaqmx__pb2.GetTimingAttributeExStringRequest.SerializeToString,
                response_deserializer=nidaqmx__pb2.GetTimingAttributeExStringResponse.FromString,
                )
        self.GetTimingAttributeExTimestamp = channel.unary_unary(
                '/nidaqmx_grpc.NiDAQmx/GetTimingAttributeExTimestamp',
                request_serializer=nidaqmx__pb2.GetTimingAttributeExTimestampRequest.SerializeToString,
                response_deserializer=nidaqmx__pb2.GetTimingAttributeExTimestampResponse.FromString,
                )
        self.GetTimingAttributeExUInt32 = channel.unary_unary(
                '/nidaqmx_grpc.NiDAQmx/GetTimingAttributeExUInt32',
                request_serializer=nidaqmx__pb2.GetTimingAttributeExUInt32Request.SerializeToString,
                response_deserializer=nidaqmx__pb2.GetTimingAttributeExUInt32Response.FromString,
                )
        self.GetTimingAttributeExUInt64 = channel.unary_unary(
                '/nidaqmx_grpc.NiDAQmx/GetTimingAttributeExUInt64',
                request_serializer=nidaqmx__pb2.GetTimingAttributeExUInt64Request.SerializeToString,
                response_deserializer=nidaqmx__pb2.GetTimingAttributeExUInt64Response.FromString,
                )
        self.GetTimingAttributeInt32 = channel.unary_unary(
                '/nidaqmx_grpc.NiDAQmx/GetTimingAttributeInt32',
                request_serializer=nidaqmx__pb2.GetTimingAttributeInt32Request.SerializeToString,
                response_deserializer=nidaqmx__pb2.GetTimingAttributeInt32Response.FromString,
                )
        self.GetTimingAttributeString = channel.unary_unary(
                '/nidaqmx_grpc.NiDAQmx/GetTimingAttributeString',
                request_serializer=nidaqmx__pb2.GetTimingAttributeStringRequest.SerializeToString,
                response_deserializer=nidaqmx__pb2.GetTimingAttributeStringResponse.FromString,
                )
        self.GetTimingAttributeTimestamp = channel.unary_unary(
                '/nidaqmx_grpc.NiDAQmx/GetTimingAttributeTimestamp',
                request_serializer=nidaqmx__pb2.GetTimingAttributeTimestampRequest.SerializeToString,
                response_deserializer=nidaqmx__pb2.GetTimingAttributeTimestampResponse.FromString,
                )
        self.GetTimingAttributeUInt32 = channel.unary_unary(
                '/nidaqmx_grpc.NiDAQmx/GetTimingAttributeUInt32',
                request_serializer=nidaqmx__pb2.GetTimingAttributeUInt32Request.SerializeToString,
                response_deserializer=nidaqmx__pb2.GetTimingAttributeUInt32Response.FromString,
                )
        self.GetTimingAttributeUInt64 = channel.unary_unary(
                '/nidaqmx_grpc.NiDAQmx/GetTimingAttributeUInt64',
                request_serializer=nidaqmx__pb2.GetTimingAttributeUInt64Request.SerializeToString,
                response_deserializer=nidaqmx__pb2.GetTimingAttributeUInt64Response.FromString,
                )
        self.GetTrigAttributeBool = channel.unary_unary(
                '/nidaqmx_grpc.NiDAQmx/GetTrigAttributeBool',
                request_serializer=nidaqmx__pb2.GetTrigAttributeBoolRequest.SerializeToString,
                response_deserializer=nidaqmx__pb2.GetTrigAttributeBoolResponse.FromString,
                )
        self.GetTrigAttributeDouble = channel.unary_unary(
                '/nidaqmx_grpc.NiDAQmx/GetTrigAttributeDouble',
                request_serializer=nidaqmx__pb2.GetTrigAttributeDoubleRequest.SerializeToString,
                response_deserializer=nidaqmx__pb2.GetTrigAttributeDoubleResponse.FromString,
                )
        self.GetTrigAttributeDoubleArray = channel.unary_unary(
                '/nidaqmx_grpc.NiDAQmx/GetTrigAttributeDoubleArray',
                request_serializer=nidaqmx__pb2.GetTrigAttributeDoubleArrayRequest.SerializeToString,
                response_deserializer=nidaqmx__pb2.GetTrigAttributeDoubleArrayResponse.FromString,
                )
        self.GetTrigAttributeInt32 = channel.unary_unary(
                '/nidaqmx_grpc.NiDAQmx/GetTrigAttributeInt32',
                request_serializer=nidaqmx__pb2.GetTrigAttributeInt32Request.SerializeToString,
                response_deserializer=nidaqmx__pb2.GetTrigAttributeInt32Response.FromString,
                )
        self.GetTrigAttributeInt32Array = channel.unary_unary(
                '/nidaqmx_grpc.NiDAQmx/GetTrigAttributeInt32Array',
                request_serializer=nidaqmx__pb2.GetTrigAttributeInt32ArrayRequest.SerializeToString,
                response_deserializer=nidaqmx__pb2.GetTrigAttributeInt32ArrayResponse.FromString,
                )
        self.GetTrigAttributeString = channel.unary_unary(
                '/nidaqmx_grpc.NiDAQmx/GetTrigAttributeString',
                request_serializer=nidaqmx__pb2.GetTrigAttributeStringRequest.SerializeToString,
                response_deserializer=nidaqmx__pb2.GetTrigAttributeStringResponse.FromString,
                )
        self.GetTrigAttributeTimestamp = channel.unary_unary(
                '/nidaqmx_grpc.NiDAQmx/GetTrigAttributeTimestamp',
                request_serializer=nidaqmx__pb2.GetTrigAttributeTimestampRequest.SerializeToString,
                response_deserializer=nidaqmx__pb2.GetTrigAttributeTimestampResponse.FromString,
                )
        self.GetTrigAttributeUInt32 = channel.unary_unary(
                '/nidaqmx_grpc.NiDAQmx/GetTrigAttributeUInt32',
                request_serializer=nidaqmx__pb2.GetTrigAttributeUInt32Request.SerializeToString,
                response_deserializer=nidaqmx__pb2.GetTrigAttributeUInt32Response.FromString,
                )
        self.GetWatchdogAttributeBool = channel.unary_unary(
                '/nidaqmx_grpc.NiDAQmx/GetWatchdogAttributeBool',
                request_serializer=nidaqmx__pb2.GetWatchdogAttributeBoolRequest.SerializeToString,
                response_deserializer=nidaqmx__pb2.GetWatchdogAttributeBoolResponse.FromString,
                )
        self.GetWatchdogAttributeDouble = channel.unary_unary(
                '/nidaqmx_grpc.NiDAQmx/GetWatchdogAttributeDouble',
                request_serializer=nidaqmx__pb2.GetWatchdogAttributeDoubleRequest.SerializeToString,
                response_deserializer=nidaqmx__pb2.GetWatchdogAttributeDoubleResponse.FromString,
                )
        self.GetWatchdogAttributeInt32 = channel.unary_unary(
                '/nidaqmx_grpc.NiDAQmx/GetWatchdogAttributeInt32',
                request_serializer=nidaqmx__pb2.GetWatchdogAttributeInt32Request.SerializeToString,
                response_deserializer=nidaqmx__pb2.GetWatchdogAttributeInt32Response.FromString,
                )
        self.GetWatchdogAttributeString = channel.unary_unary(
                '/nidaqmx_grpc.NiDAQmx/GetWatchdogAttributeString',
                request_serializer=nidaqmx__pb2.GetWatchdogAttributeStringRequest.SerializeToString,
                response_deserializer=nidaqmx__pb2.GetWatchdogAttributeStringResponse.FromString,
                )
        self.GetWriteAttributeBool = channel.unary_unary(
                '/nidaqmx_grpc.NiDAQmx/GetWriteAttributeBool',
                request_serializer=nidaqmx__pb2.GetWriteAttributeBoolRequest.SerializeToString,
                response_deserializer=nidaqmx__pb2.GetWriteAttributeBoolResponse.FromString,
                )
        self.GetWriteAttributeDouble = channel.unary_unary(
                '/nidaqmx_grpc.NiDAQmx/GetWriteAttributeDouble',
                request_serializer=nidaqmx__pb2.GetWriteAttributeDoubleRequest.SerializeToString,
                response_deserializer=nidaqmx__pb2.GetWriteAttributeDoubleResponse.FromString,
                )
        self.GetWriteAttributeInt32 = channel.unary_unary(
                '/nidaqmx_grpc.NiDAQmx/GetWriteAttributeInt32',
                request_serializer=nidaqmx__pb2.GetWriteAttributeInt32Request.SerializeToString,
                response_deserializer=nidaqmx__pb2.GetWriteAttributeInt32Response.FromString,
                )
        self.GetWriteAttributeString = channel.unary_unary(
                '/nidaqmx_grpc.NiDAQmx/GetWriteAttributeString',
                request_serializer=nidaqmx__pb2.GetWriteAttributeStringRequest.SerializeToString,
                response_deserializer=nidaqmx__pb2.GetWriteAttributeStringResponse.FromString,
                )
        self.GetWriteAttributeUInt32 = channel.unary_unary(
                '/nidaqmx_grpc.NiDAQmx/GetWriteAttributeUInt32',
                request_serializer=nidaqmx__pb2.GetWriteAttributeUInt32Request.SerializeToString,
                response_deserializer=nidaqmx__pb2.GetWriteAttributeUInt32Response.FromString,
                )
        self.GetWriteAttributeUInt64 = channel.unary_unary(
                '/nidaqmx_grpc.NiDAQmx/GetWriteAttributeUInt64',
                request_serializer=nidaqmx__pb2.GetWriteAttributeUInt64Request.SerializeToString,
                response_deserializer=nidaqmx__pb2.GetWriteAttributeUInt64Response.FromString,
                )
        self.IsTaskDone = channel.unary_unary(
                '/nidaqmx_grpc.NiDAQmx/IsTaskDone',
                request_serializer=nidaqmx__pb2.IsTaskDoneRequest.SerializeToString,
                response_deserializer=nidaqmx__pb2.IsTaskDoneResponse.FromString,
                )
        self.LoadTask = channel.unary_unary(
                '/nidaqmx_grpc.NiDAQmx/LoadTask',
                request_serializer=nidaqmx__pb2.LoadTaskRequest.SerializeToString,
                response_deserializer=nidaqmx__pb2.LoadTaskResponse.FromString,
                )
        self.PerformBridgeOffsetNullingCalEx = channel.unary_unary(
                '/nidaqmx_grpc.NiDAQmx/PerformBridgeOffsetNullingCalEx',
                request_serializer=nidaqmx__pb2.PerformBridgeOffsetNullingCalExRequest.SerializeToString,
                response_deserializer=nidaqmx__pb2.PerformBridgeOffsetNullingCalExResponse.FromString,
                )
        self.PerformBridgeShuntCalEx = channel.unary_unary(
                '/nidaqmx_grpc.NiDAQmx/PerformBridgeShuntCalEx',
                request_serializer=nidaqmx__pb2.PerformBridgeShuntCalExRequest.SerializeToString,
                response_deserializer=nidaqmx__pb2.PerformBridgeShuntCalExResponse.FromString,
                )
        self.PerformStrainShuntCalEx = channel.unary_unary(
                '/nidaqmx_grpc.NiDAQmx/PerformStrainShuntCalEx',
                request_serializer=nidaqmx__pb2.PerformStrainShuntCalExRequest.SerializeToString,
                response_deserializer=nidaqmx__pb2.PerformStrainShuntCalExResponse.FromString,
                )
        self.PerformThrmcplLeadOffsetNullingCal = channel.unary_unary(
                '/nidaqmx_grpc.NiDAQmx/PerformThrmcplLeadOffsetNullingCal',
                request_serializer=nidaqmx__pb2.PerformThrmcplLeadOffsetNullingCalRequest.SerializeToString,
                response_deserializer=nidaqmx__pb2.PerformThrmcplLeadOffsetNullingCalResponse.FromString,
                )
        self.ReadAnalogF64 = channel.unary_unary(
                '/nidaqmx_grpc.NiDAQmx/ReadAnalogF64',
                request_serializer=nidaqmx__pb2.ReadAnalogF64Request.SerializeToString,
                response_deserializer=nidaqmx__pb2.ReadAnalogF64Response.FromString,
                )
        self.ReadAnalogScalarF64 = channel.unary_unary(
                '/nidaqmx_grpc.NiDAQmx/ReadAnalogScalarF64',
                request_serializer=nidaqmx__pb2.ReadAnalogScalarF64Request.SerializeToString,
                response_deserializer=nidaqmx__pb2.ReadAnalogScalarF64Response.FromString,
                )
        self.ReadBinaryI16 = channel.unary_unary(
                '/nidaqmx_grpc.NiDAQmx/ReadBinaryI16',
                request_serializer=nidaqmx__pb2.ReadBinaryI16Request.SerializeToString,
                response_deserializer=nidaqmx__pb2.ReadBinaryI16Response.FromString,
                )
        self.ReadBinaryI32 = channel.unary_unary(
                '/nidaqmx_grpc.NiDAQmx/ReadBinaryI32',
                request_serializer=nidaqmx__pb2.ReadBinaryI32Request.SerializeToString,
                response_deserializer=nidaqmx__pb2.ReadBinaryI32Response.FromString,
                )
        self.ReadBinaryU16 = channel.unary_unary(
                '/nidaqmx_grpc.NiDAQmx/ReadBinaryU16',
                request_serializer=nidaqmx__pb2.ReadBinaryU16Request.SerializeToString,
                response_deserializer=nidaqmx__pb2.ReadBinaryU16Response.FromString,
                )
        self.ReadBinaryU32 = channel.unary_unary(
                '/nidaqmx_grpc.NiDAQmx/ReadBinaryU32',
                request_serializer=nidaqmx__pb2.ReadBinaryU32Request.SerializeToString,
                response_deserializer=nidaqmx__pb2.ReadBinaryU32Response.FromString,
                )
        self.ReadCounterF64 = channel.unary_unary(
                '/nidaqmx_grpc.NiDAQmx/ReadCounterF64',
                request_serializer=nidaqmx__pb2.ReadCounterF64Request.SerializeToString,
                response_deserializer=nidaqmx__pb2.ReadCounterF64Response.FromString,
                )
        self.ReadCounterF64Ex = channel.unary_unary(
                '/nidaqmx_grpc.NiDAQmx/ReadCounterF64Ex',
                request_serializer=nidaqmx__pb2.ReadCounterF64ExRequest.SerializeToString,
                response_deserializer=nidaqmx__pb2.ReadCounterF64ExResponse.FromString,
                )
        self.ReadCounterScalarF64 = channel.unary_unary(
                '/nidaqmx_grpc.NiDAQmx/ReadCounterScalarF64',
                request_serializer=nidaqmx__pb2.ReadCounterScalarF64Request.SerializeToString,
                response_deserializer=nidaqmx__pb2.ReadCounterScalarF64Response.FromString,
                )
        self.ReadCounterScalarU32 = channel.unary_unary(
                '/nidaqmx_grpc.NiDAQmx/ReadCounterScalarU32',
                request_serializer=nidaqmx__pb2.ReadCounterScalarU32Request.SerializeToString,
                response_deserializer=nidaqmx__pb2.ReadCounterScalarU32Response.FromString,
                )
        self.ReadCounterU32 = channel.unary_unary(
                '/nidaqmx_grpc.NiDAQmx/ReadCounterU32',
                request_serializer=nidaqmx__pb2.ReadCounterU32Request.SerializeToString,
                response_deserializer=nidaqmx__pb2.ReadCounterU32Response.FromString,
                )
        self.ReadCounterU32Ex = channel.unary_unary(
                '/nidaqmx_grpc.NiDAQmx/ReadCounterU32Ex',
                request_serializer=nidaqmx__pb2.ReadCounterU32ExRequest.SerializeToString,
                response_deserializer=nidaqmx__pb2.ReadCounterU32ExResponse.FromString,
                )
        self.ReadCtrFreq = channel.unary_unary(
                '/nidaqmx_grpc.NiDAQmx/ReadCtrFreq',
                request_serializer=nidaqmx__pb2.ReadCtrFreqRequest.SerializeToString,
                response_deserializer=nidaqmx__pb2.ReadCtrFreqResponse.FromString,
                )
        self.ReadCtrFreqScalar = channel.unary_unary(
                '/nidaqmx_grpc.NiDAQmx/ReadCtrFreqScalar',
                request_serializer=nidaqmx__pb2.ReadCtrFreqScalarRequest.SerializeToString,
                response_deserializer=nidaqmx__pb2.ReadCtrFreqScalarResponse.FromString,
                )
        self.ReadCtrTicks = channel.unary_unary(
                '/nidaqmx_grpc.NiDAQmx/ReadCtrTicks',
                request_serializer=nidaqmx__pb2.ReadCtrTicksRequest.SerializeToString,
                response_deserializer=nidaqmx__pb2.ReadCtrTicksResponse.FromString,
                )
        self.ReadCtrTicksScalar = channel.unary_unary(
                '/nidaqmx_grpc.NiDAQmx/ReadCtrTicksScalar',
                request_serializer=nidaqmx__pb2.ReadCtrTicksScalarRequest.SerializeToString,
                response_deserializer=nidaqmx__pb2.ReadCtrTicksScalarResponse.FromString,
                )
        self.ReadCtrTime = channel.unary_unary(
                '/nidaqmx_grpc.NiDAQmx/ReadCtrTime',
                request_serializer=nidaqmx__pb2.ReadCtrTimeRequest.SerializeToString,
                response_deserializer=nidaqmx__pb2.ReadCtrTimeResponse.FromString,
                )
        self.ReadCtrTimeScalar = channel.unary_unary(
                '/nidaqmx_grpc.NiDAQmx/ReadCtrTimeScalar',
                request_serializer=nidaqmx__pb2.ReadCtrTimeScalarRequest.SerializeToString,
                response_deserializer=nidaqmx__pb2.ReadCtrTimeScalarResponse.FromString,
                )
        self.ReadDigitalLines = channel.unary_unary(
                '/nidaqmx_grpc.NiDAQmx/ReadDigitalLines',
                request_serializer=nidaqmx__pb2.ReadDigitalLinesRequest.SerializeToString,
                response_deserializer=nidaqmx__pb2.ReadDigitalLinesResponse.FromString,
                )
        self.ReadDigitalScalarU32 = channel.unary_unary(
                '/nidaqmx_grpc.NiDAQmx/ReadDigitalScalarU32',
                request_serializer=nidaqmx__pb2.ReadDigitalScalarU32Request.SerializeToString,
                response_deserializer=nidaqmx__pb2.ReadDigitalScalarU32Response.FromString,
                )
        self.ReadDigitalU16 = channel.unary_unary(
                '/nidaqmx_grpc.NiDAQmx/ReadDigitalU16',
                request_serializer=nidaqmx__pb2.ReadDigitalU16Request.SerializeToString,
                response_deserializer=nidaqmx__pb2.ReadDigitalU16Response.FromString,
                )
        self.ReadDigitalU32 = channel.unary_unary(
                '/nidaqmx_grpc.NiDAQmx/ReadDigitalU32',
                request_serializer=nidaqmx__pb2.ReadDigitalU32Request.SerializeToString,
                response_deserializer=nidaqmx__pb2.ReadDigitalU32Response.FromString,
                )
        self.ReadDigitalU8 = channel.unary_unary(
                '/nidaqmx_grpc.NiDAQmx/ReadDigitalU8',
                request_serializer=nidaqmx__pb2.ReadDigitalU8Request.SerializeToString,
                response_deserializer=nidaqmx__pb2.ReadDigitalU8Response.FromString,
                )
        self.ReadPowerBinaryI16 = channel.unary_unary(
                '/nidaqmx_grpc.NiDAQmx/ReadPowerBinaryI16',
                request_serializer=nidaqmx__pb2.ReadPowerBinaryI16Request.SerializeToString,
                response_deserializer=nidaqmx__pb2.ReadPowerBinaryI16Response.FromString,
                )
        self.ReadPowerF64 = channel.unary_unary(
                '/nidaqmx_grpc.NiDAQmx/ReadPowerF64',
                request_serializer=nidaqmx__pb2.ReadPowerF64Request.SerializeToString,
                response_deserializer=nidaqmx__pb2.ReadPowerF64Response.FromString,
                )
        self.ReadPowerScalarF64 = channel.unary_unary(
                '/nidaqmx_grpc.NiDAQmx/ReadPowerScalarF64',
                request_serializer=nidaqmx__pb2.ReadPowerScalarF64Request.SerializeToString,
                response_deserializer=nidaqmx__pb2.ReadPowerScalarF64Response.FromString,
                )
        self.ReadRaw = channel.unary_unary(
                '/nidaqmx_grpc.NiDAQmx/ReadRaw',
                request_serializer=nidaqmx__pb2.ReadRawRequest.SerializeToString,
                response_deserializer=nidaqmx__pb2.ReadRawResponse.FromString,
                )
        self.RegisterDoneEvent = channel.unary_stream(
                '/nidaqmx_grpc.NiDAQmx/RegisterDoneEvent',
                request_serializer=nidaqmx__pb2.RegisterDoneEventRequest.SerializeToString,
                response_deserializer=nidaqmx__pb2.RegisterDoneEventResponse.FromString,
                )
        self.RegisterEveryNSamplesEvent = channel.unary_stream(
                '/nidaqmx_grpc.NiDAQmx/RegisterEveryNSamplesEvent',
                request_serializer=nidaqmx__pb2.RegisterEveryNSamplesEventRequest.SerializeToString,
                response_deserializer=nidaqmx__pb2.RegisterEveryNSamplesEventResponse.FromString,
                )
        self.RegisterSignalEvent = channel.unary_stream(
                '/nidaqmx_grpc.NiDAQmx/RegisterSignalEvent',
                request_serializer=nidaqmx__pb2.RegisterSignalEventRequest.SerializeToString,
                response_deserializer=nidaqmx__pb2.RegisterSignalEventResponse.FromString,
                )
        self.RemoveCDAQSyncConnection = channel.unary_unary(
                '/nidaqmx_grpc.NiDAQmx/RemoveCDAQSyncConnection',
                request_serializer=nidaqmx__pb2.RemoveCDAQSyncConnectionRequest.SerializeToString,
                response_deserializer=nidaqmx__pb2.RemoveCDAQSyncConnectionResponse.FromString,
                )
        self.ReserveNetworkDevice = channel.unary_unary(
                '/nidaqmx_grpc.NiDAQmx/ReserveNetworkDevice',
                request_serializer=nidaqmx__pb2.ReserveNetworkDeviceRequest.SerializeToString,
                response_deserializer=nidaqmx__pb2.ReserveNetworkDeviceResponse.FromString,
                )
        self.ResetBufferAttribute = channel.unary_unary(
                '/nidaqmx_grpc.NiDAQmx/ResetBufferAttribute',
                request_serializer=nidaqmx__pb2.ResetBufferAttributeRequest.SerializeToString,
                response_deserializer=nidaqmx__pb2.ResetBufferAttributeResponse.FromString,
                )
        self.ResetChanAttribute = channel.unary_unary(
                '/nidaqmx_grpc.NiDAQmx/ResetChanAttribute',
                request_serializer=nidaqmx__pb2.ResetChanAttributeRequest.SerializeToString,
                response_deserializer=nidaqmx__pb2.ResetChanAttributeResponse.FromString,
                )
        self.ResetDevice = channel.unary_unary(
                '/nidaqmx_grpc.NiDAQmx/ResetDevice',
                request_serializer=nidaqmx__pb2.ResetDeviceRequest.SerializeToString,
                response_deserializer=nidaqmx__pb2.ResetDeviceResponse.FromString,
                )
        self.ResetExportedSignalAttribute = channel.unary_unary(
                '/nidaqmx_grpc.NiDAQmx/ResetExportedSignalAttribute',
                request_serializer=nidaqmx__pb2.ResetExportedSignalAttributeRequest.SerializeToString,
                response_deserializer=nidaqmx__pb2.ResetExportedSignalAttributeResponse.FromString,
                )
        self.ResetReadAttribute = channel.unary_unary(
                '/nidaqmx_grpc.NiDAQmx/ResetReadAttribute',
                request_serializer=nidaqmx__pb2.ResetReadAttributeRequest.SerializeToString,
                response_deserializer=nidaqmx__pb2.ResetReadAttributeResponse.FromString,
                )
        self.ResetRealTimeAttribute = channel.unary_unary(
                '/nidaqmx_grpc.NiDAQmx/ResetRealTimeAttribute',
                request_serializer=nidaqmx__pb2.ResetRealTimeAttributeRequest.SerializeToString,
                response_deserializer=nidaqmx__pb2.ResetRealTimeAttributeResponse.FromString,
                )
        self.ResetTimingAttribute = channel.unary_unary(
                '/nidaqmx_grpc.NiDAQmx/ResetTimingAttribute',
                request_serializer=nidaqmx__pb2.ResetTimingAttributeRequest.SerializeToString,
                response_deserializer=nidaqmx__pb2.ResetTimingAttributeResponse.FromString,
                )
        self.ResetTimingAttributeEx = channel.unary_unary(
                '/nidaqmx_grpc.NiDAQmx/ResetTimingAttributeEx',
                request_serializer=nidaqmx__pb2.ResetTimingAttributeExRequest.SerializeToString,
                response_deserializer=nidaqmx__pb2.ResetTimingAttributeExResponse.FromString,
                )
        self.ResetTrigAttribute = channel.unary_unary(
                '/nidaqmx_grpc.NiDAQmx/ResetTrigAttribute',
                request_serializer=nidaqmx__pb2.ResetTrigAttributeRequest.SerializeToString,
                response_deserializer=nidaqmx__pb2.ResetTrigAttributeResponse.FromString,
                )
        self.ResetWatchdogAttribute = channel.unary_unary(
                '/nidaqmx_grpc.NiDAQmx/ResetWatchdogAttribute',
                request_serializer=nidaqmx__pb2.ResetWatchdogAttributeRequest.SerializeToString,
                response_deserializer=nidaqmx__pb2.ResetWatchdogAttributeResponse.FromString,
                )
        self.ResetWriteAttribute = channel.unary_unary(
                '/nidaqmx_grpc.NiDAQmx/ResetWriteAttribute',
                request_serializer=nidaqmx__pb2.ResetWriteAttributeRequest.SerializeToString,
                response_deserializer=nidaqmx__pb2.ResetWriteAttributeResponse.FromString,
                )
        self.RestoreLastExtCalConst = channel.unary_unary(
                '/nidaqmx_grpc.NiDAQmx/RestoreLastExtCalConst',
                request_serializer=nidaqmx__pb2.RestoreLastExtCalConstRequest.SerializeToString,
                response_deserializer=nidaqmx__pb2.RestoreLastExtCalConstResponse.FromString,
                )
        self.SaveGlobalChan = channel.unary_unary(
                '/nidaqmx_grpc.NiDAQmx/SaveGlobalChan',
                request_serializer=nidaqmx__pb2.SaveGlobalChanRequest.SerializeToString,
                response_deserializer=nidaqmx__pb2.SaveGlobalChanResponse.FromString,
                )
        self.SaveScale = channel.unary_unary(
                '/nidaqmx_grpc.NiDAQmx/SaveScale',
                request_serializer=nidaqmx__pb2.SaveScaleRequest.SerializeToString,
                response_deserializer=nidaqmx__pb2.SaveScaleResponse.FromString,
                )
        self.SaveTask = channel.unary_unary(
                '/nidaqmx_grpc.NiDAQmx/SaveTask',
                request_serializer=nidaqmx__pb2.SaveTaskRequest.SerializeToString,
                response_deserializer=nidaqmx__pb2.SaveTaskResponse.FromString,
                )
        self.SelfCal = channel.unary_unary(
                '/nidaqmx_grpc.NiDAQmx/SelfCal',
                request_serializer=nidaqmx__pb2.SelfCalRequest.SerializeToString,
                response_deserializer=nidaqmx__pb2.SelfCalResponse.FromString,
                )
        self.SelfTestDevice = channel.unary_unary(
                '/nidaqmx_grpc.NiDAQmx/SelfTestDevice',
                request_serializer=nidaqmx__pb2.SelfTestDeviceRequest.SerializeToString,
                response_deserializer=nidaqmx__pb2.SelfTestDeviceResponse.FromString,
                )
        self.SetAIChanCalCalDate = channel.unary_unary(
                '/nidaqmx_grpc.NiDAQmx/SetAIChanCalCalDate',
                request_serializer=nidaqmx__pb2.SetAIChanCalCalDateRequest.SerializeToString,
                response_deserializer=nidaqmx__pb2.SetAIChanCalCalDateResponse.FromString,
                )
        self.SetAIChanCalExpDate = channel.unary_unary(
                '/nidaqmx_grpc.NiDAQmx/SetAIChanCalExpDate',
                request_serializer=nidaqmx__pb2.SetAIChanCalExpDateRequest.SerializeToString,
                response_deserializer=nidaqmx__pb2.SetAIChanCalExpDateResponse.FromString,
                )
        self.SetAnalogPowerUpStates = channel.unary_unary(
                '/nidaqmx_grpc.NiDAQmx/SetAnalogPowerUpStates',
                request_serializer=nidaqmx__pb2.SetAnalogPowerUpStatesRequest.SerializeToString,
                response_deserializer=nidaqmx__pb2.SetAnalogPowerUpStatesResponse.FromString,
                )
        self.SetAnalogPowerUpStatesWithOutputType = channel.unary_unary(
                '/nidaqmx_grpc.NiDAQmx/SetAnalogPowerUpStatesWithOutputType',
                request_serializer=nidaqmx__pb2.SetAnalogPowerUpStatesWithOutputTypeRequest.SerializeToString,
                response_deserializer=nidaqmx__pb2.SetAnalogPowerUpStatesWithOutputTypeResponse.FromString,
                )
        self.SetArmStartTrigTrigWhen = channel.unary_unary(
                '/nidaqmx_grpc.NiDAQmx/SetArmStartTrigTrigWhen',
                request_serializer=nidaqmx__pb2.SetArmStartTrigTrigWhenRequest.SerializeToString,
                response_deserializer=nidaqmx__pb2.SetArmStartTrigTrigWhenResponse.FromString,
                )
        self.SetBufferAttributeUInt32 = channel.unary_unary(
                '/nidaqmx_grpc.NiDAQmx/SetBufferAttributeUInt32',
                request_serializer=nidaqmx__pb2.SetBufferAttributeUInt32Request.SerializeToString,
                response_deserializer=nidaqmx__pb2.SetBufferAttributeUInt32Response.FromString,
                )
        self.SetCalInfoAttributeBool = channel.unary_unary(
                '/nidaqmx_grpc.NiDAQmx/SetCalInfoAttributeBool',
                request_serializer=nidaqmx__pb2.SetCalInfoAttributeBoolRequest.SerializeToString,
                response_deserializer=nidaqmx__pb2.SetCalInfoAttributeBoolResponse.FromString,
                )
        self.SetCalInfoAttributeDouble = channel.unary_unary(
                '/nidaqmx_grpc.NiDAQmx/SetCalInfoAttributeDouble',
                request_serializer=nidaqmx__pb2.SetCalInfoAttributeDoubleRequest.SerializeToString,
                response_deserializer=nidaqmx__pb2.SetCalInfoAttributeDoubleResponse.FromString,
                )
        self.SetCalInfoAttributeString = channel.unary_unary(
                '/nidaqmx_grpc.NiDAQmx/SetCalInfoAttributeString',
                request_serializer=nidaqmx__pb2.SetCalInfoAttributeStringRequest.SerializeToString,
                response_deserializer=nidaqmx__pb2.SetCalInfoAttributeStringResponse.FromString,
                )
        self.SetCalInfoAttributeUInt32 = channel.unary_unary(
                '/nidaqmx_grpc.NiDAQmx/SetCalInfoAttributeUInt32',
                request_serializer=nidaqmx__pb2.SetCalInfoAttributeUInt32Request.SerializeToString,
                response_deserializer=nidaqmx__pb2.SetCalInfoAttributeUInt32Response.FromString,
                )
        self.SetChanAttributeBool = channel.unary_unary(
                '/nidaqmx_grpc.NiDAQmx/SetChanAttributeBool',
                request_serializer=nidaqmx__pb2.SetChanAttributeBoolRequest.SerializeToString,
                response_deserializer=nidaqmx__pb2.SetChanAttributeBoolResponse.FromString,
                )
        self.SetChanAttributeDouble = channel.unary_unary(
                '/nidaqmx_grpc.NiDAQmx/SetChanAttributeDouble',
                request_serializer=nidaqmx__pb2.SetChanAttributeDoubleRequest.SerializeToString,
                response_deserializer=nidaqmx__pb2.SetChanAttributeDoubleResponse.FromString,
                )
        self.SetChanAttributeDoubleArray = channel.unary_unary(
                '/nidaqmx_grpc.NiDAQmx/SetChanAttributeDoubleArray',
                request_serializer=nidaqmx__pb2.SetChanAttributeDoubleArrayRequest.SerializeToString,
                response_deserializer=nidaqmx__pb2.SetChanAttributeDoubleArrayResponse.FromString,
                )
        self.SetChanAttributeInt32 = channel.unary_unary(
                '/nidaqmx_grpc.NiDAQmx/SetChanAttributeInt32',
                request_serializer=nidaqmx__pb2.SetChanAttributeInt32Request.SerializeToString,
                response_deserializer=nidaqmx__pb2.SetChanAttributeInt32Response.FromString,
                )
        self.SetChanAttributeString = channel.unary_unary(
                '/nidaqmx_grpc.NiDAQmx/SetChanAttributeString',
                request_serializer=nidaqmx__pb2.SetChanAttributeStringRequest.SerializeToString,
                response_deserializer=nidaqmx__pb2.SetChanAttributeStringResponse.FromString,
                )
        self.SetChanAttributeUInt32 = channel.unary_unary(
                '/nidaqmx_grpc.NiDAQmx/SetChanAttributeUInt32',
                request_serializer=nidaqmx__pb2.SetChanAttributeUInt32Request.SerializeToString,
                response_deserializer=nidaqmx__pb2.SetChanAttributeUInt32Response.FromString,
                )
        self.SetDigitalLogicFamilyPowerUpState = channel.unary_unary(
                '/nidaqmx_grpc.NiDAQmx/SetDigitalLogicFamilyPowerUpState',
                request_serializer=nidaqmx__pb2.SetDigitalLogicFamilyPowerUpStateRequest.SerializeToString,
                response_deserializer=nidaqmx__pb2.SetDigitalLogicFamilyPowerUpStateResponse.FromString,
                )
        self.SetDigitalPowerUpStates = channel.unary_unary(
                '/nidaqmx_grpc.NiDAQmx/SetDigitalPowerUpStates',
                request_serializer=nidaqmx__pb2.SetDigitalPowerUpStatesRequest.SerializeToString,
                response_deserializer=nidaqmx__pb2.SetDigitalPowerUpStatesResponse.FromString,
                )
        self.SetDigitalPullUpPullDownStates = channel.unary_unary(
                '/nidaqmx_grpc.NiDAQmx/SetDigitalPullUpPullDownStates',
                request_serializer=nidaqmx__pb2.SetDigitalPullUpPullDownStatesRequest.SerializeToString,
                response_deserializer=nidaqmx__pb2.SetDigitalPullUpPullDownStatesResponse.FromString,
                )
        self.SetExportedSignalAttributeBool = channel.unary_unary(
                '/nidaqmx_grpc.NiDAQmx/SetExportedSignalAttributeBool',
                request_serializer=nidaqmx__pb2.SetExportedSignalAttributeBoolRequest.SerializeToString,
                response_deserializer=nidaqmx__pb2.SetExportedSignalAttributeBoolResponse.FromString,
                )
        self.SetExportedSignalAttributeDouble = channel.unary_unary(
                '/nidaqmx_grpc.NiDAQmx/SetExportedSignalAttributeDouble',
                request_serializer=nidaqmx__pb2.SetExportedSignalAttributeDoubleRequest.SerializeToString,
                response_deserializer=nidaqmx__pb2.SetExportedSignalAttributeDoubleResponse.FromString,
                )
        self.SetExportedSignalAttributeInt32 = channel.unary_unary(
                '/nidaqmx_grpc.NiDAQmx/SetExportedSignalAttributeInt32',
                request_serializer=nidaqmx__pb2.SetExportedSignalAttributeInt32Request.SerializeToString,
                response_deserializer=nidaqmx__pb2.SetExportedSignalAttributeInt32Response.FromString,
                )
        self.SetExportedSignalAttributeString = channel.unary_unary(
                '/nidaqmx_grpc.NiDAQmx/SetExportedSignalAttributeString',
                request_serializer=nidaqmx__pb2.SetExportedSignalAttributeStringRequest.SerializeToString,
                response_deserializer=nidaqmx__pb2.SetExportedSignalAttributeStringResponse.FromString,
                )
        self.SetExportedSignalAttributeUInt32 = channel.unary_unary(
                '/nidaqmx_grpc.NiDAQmx/SetExportedSignalAttributeUInt32',
                request_serializer=nidaqmx__pb2.SetExportedSignalAttributeUInt32Request.SerializeToString,
                response_deserializer=nidaqmx__pb2.SetExportedSignalAttributeUInt32Response.FromString,
                )
        self.SetFirstSampClkWhen = channel.unary_unary(
                '/nidaqmx_grpc.NiDAQmx/SetFirstSampClkWhen',
                request_serializer=nidaqmx__pb2.SetFirstSampClkWhenRequest.SerializeToString,
                response_deserializer=nidaqmx__pb2.SetFirstSampClkWhenResponse.FromString,
                )
        self.SetReadAttributeBool = channel.unary_unary(
                '/nidaqmx_grpc.NiDAQmx/SetReadAttributeBool',
                request_serializer=nidaqmx__pb2.SetReadAttributeBoolRequest.SerializeToString,
                response_deserializer=nidaqmx__pb2.SetReadAttributeBoolResponse.FromString,
                )
        self.SetReadAttributeDouble = channel.unary_unary(
                '/nidaqmx_grpc.NiDAQmx/SetReadAttributeDouble',
                request_serializer=nidaqmx__pb2.SetReadAttributeDoubleRequest.SerializeToString,
                response_deserializer=nidaqmx__pb2.SetReadAttributeDoubleResponse.FromString,
                )
        self.SetReadAttributeInt32 = channel.unary_unary(
                '/nidaqmx_grpc.NiDAQmx/SetReadAttributeInt32',
                request_serializer=nidaqmx__pb2.SetReadAttributeInt32Request.SerializeToString,
                response_deserializer=nidaqmx__pb2.SetReadAttributeInt32Response.FromString,
                )
        self.SetReadAttributeString = channel.unary_unary(
                '/nidaqmx_grpc.NiDAQmx/SetReadAttributeString',
                request_serializer=nidaqmx__pb2.SetReadAttributeStringRequest.SerializeToString,
                response_deserializer=nidaqmx__pb2.SetReadAttributeStringResponse.FromString,
                )
        self.SetReadAttributeUInt32 = channel.unary_unary(
                '/nidaqmx_grpc.NiDAQmx/SetReadAttributeUInt32',
                request_serializer=nidaqmx__pb2.SetReadAttributeUInt32Request.SerializeToString,
                response_deserializer=nidaqmx__pb2.SetReadAttributeUInt32Response.FromString,
                )
        self.SetReadAttributeUInt64 = channel.unary_unary(
                '/nidaqmx_grpc.NiDAQmx/SetReadAttributeUInt64',
                request_serializer=nidaqmx__pb2.SetReadAttributeUInt64Request.SerializeToString,
                response_deserializer=nidaqmx__pb2.SetReadAttributeUInt64Response.FromString,
                )
        self.SetRealTimeAttributeBool = channel.unary_unary(
                '/nidaqmx_grpc.NiDAQmx/SetRealTimeAttributeBool',
                request_serializer=nidaqmx__pb2.SetRealTimeAttributeBoolRequest.SerializeToString,
                response_deserializer=nidaqmx__pb2.SetRealTimeAttributeBoolResponse.FromString,
                )
        self.SetRealTimeAttributeInt32 = channel.unary_unary(
                '/nidaqmx_grpc.NiDAQmx/SetRealTimeAttributeInt32',
                request_serializer=nidaqmx__pb2.SetRealTimeAttributeInt32Request.SerializeToString,
                response_deserializer=nidaqmx__pb2.SetRealTimeAttributeInt32Response.FromString,
                )
        self.SetRealTimeAttributeUInt32 = channel.unary_unary(
                '/nidaqmx_grpc.NiDAQmx/SetRealTimeAttributeUInt32',
                request_serializer=nidaqmx__pb2.SetRealTimeAttributeUInt32Request.SerializeToString,
                response_deserializer=nidaqmx__pb2.SetRealTimeAttributeUInt32Response.FromString,
                )
        self.SetScaleAttributeDouble = channel.unary_unary(
                '/nidaqmx_grpc.NiDAQmx/SetScaleAttributeDouble',
                request_serializer=nidaqmx__pb2.SetScaleAttributeDoubleRequest.SerializeToString,
                response_deserializer=nidaqmx__pb2.SetScaleAttributeDoubleResponse.FromString,
                )
        self.SetScaleAttributeDoubleArray = channel.unary_unary(
                '/nidaqmx_grpc.NiDAQmx/SetScaleAttributeDoubleArray',
                request_serializer=nidaqmx__pb2.SetScaleAttributeDoubleArrayRequest.SerializeToString,
                response_deserializer=nidaqmx__pb2.SetScaleAttributeDoubleArrayResponse.FromString,
                )
        self.SetScaleAttributeInt32 = channel.unary_unary(
                '/nidaqmx_grpc.NiDAQmx/SetScaleAttributeInt32',
                request_serializer=nidaqmx__pb2.SetScaleAttributeInt32Request.SerializeToString,
                response_deserializer=nidaqmx__pb2.SetScaleAttributeInt32Response.FromString,
                )
        self.SetScaleAttributeString = channel.unary_unary(
                '/nidaqmx_grpc.NiDAQmx/SetScaleAttributeString',
                request_serializer=nidaqmx__pb2.SetScaleAttributeStringRequest.SerializeToString,
                response_deserializer=nidaqmx__pb2.SetScaleAttributeStringResponse.FromString,
                )
        self.SetStartTrigTrigWhen = channel.unary_unary(
                '/nidaqmx_grpc.NiDAQmx/SetStartTrigTrigWhen',
                request_serializer=nidaqmx__pb2.SetStartTrigTrigWhenRequest.SerializeToString,
                response_deserializer=nidaqmx__pb2.SetStartTrigTrigWhenResponse.FromString,
                )
        self.SetSyncPulseTimeWhen = channel.unary_unary(
                '/nidaqmx_grpc.NiDAQmx/SetSyncPulseTimeWhen',
                request_serializer=nidaqmx__pb2.SetSyncPulseTimeWhenRequest.SerializeToString,
                response_deserializer=nidaqmx__pb2.SetSyncPulseTimeWhenResponse.FromString,
                )
        self.SetTimingAttributeBool = channel.unary_unary(
                '/nidaqmx_grpc.NiDAQmx/SetTimingAttributeBool',
                request_serializer=nidaqmx__pb2.SetTimingAttributeBoolRequest.SerializeToString,
                response_deserializer=nidaqmx__pb2.SetTimingAttributeBoolResponse.FromString,
                )
        self.SetTimingAttributeDouble = channel.unary_unary(
                '/nidaqmx_grpc.NiDAQmx/SetTimingAttributeDouble',
                request_serializer=nidaqmx__pb2.SetTimingAttributeDoubleRequest.SerializeToString,
                response_deserializer=nidaqmx__pb2.SetTimingAttributeDoubleResponse.FromString,
                )
        self.SetTimingAttributeExBool = channel.unary_unary(
                '/nidaqmx_grpc.NiDAQmx/SetTimingAttributeExBool',
                request_serializer=nidaqmx__pb2.SetTimingAttributeExBoolRequest.SerializeToString,
                response_deserializer=nidaqmx__pb2.SetTimingAttributeExBoolResponse.FromString,
                )
        self.SetTimingAttributeExDouble = channel.unary_unary(
                '/nidaqmx_grpc.NiDAQmx/SetTimingAttributeExDouble',
                request_serializer=nidaqmx__pb2.SetTimingAttributeExDoubleRequest.SerializeToString,
                response_deserializer=nidaqmx__pb2.SetTimingAttributeExDoubleResponse.FromString,
                )
        self.SetTimingAttributeExInt32 = channel.unary_unary(
                '/nidaqmx_grpc.NiDAQmx/SetTimingAttributeExInt32',
                request_serializer=nidaqmx__pb2.SetTimingAttributeExInt32Request.SerializeToString,
                response_deserializer=nidaqmx__pb2.SetTimingAttributeExInt32Response.FromString,
                )
        self.SetTimingAttributeExString = channel.unary_unary(
                '/nidaqmx_grpc.NiDAQmx/SetTimingAttributeExString',
                request_serializer=nidaqmx__pb2.SetTimingAttributeExStringRequest.SerializeToString,
                response_deserializer=nidaqmx__pb2.SetTimingAttributeExStringResponse.FromString,
                )
        self.SetTimingAttributeExTimestamp = channel.unary_unary(
                '/nidaqmx_grpc.NiDAQmx/SetTimingAttributeExTimestamp',
                request_serializer=nidaqmx__pb2.SetTimingAttributeExTimestampRequest.SerializeToString,
                response_deserializer=nidaqmx__pb2.SetTimingAttributeExTimestampResponse.FromString,
                )
        self.SetTimingAttributeExUInt32 = channel.unary_unary(
                '/nidaqmx_grpc.NiDAQmx/SetTimingAttributeExUInt32',
                request_serializer=nidaqmx__pb2.SetTimingAttributeExUInt32Request.SerializeToString,
                response_deserializer=nidaqmx__pb2.SetTimingAttributeExUInt32Response.FromString,
                )
        self.SetTimingAttributeExUInt64 = channel.unary_unary(
                '/nidaqmx_grpc.NiDAQmx/SetTimingAttributeExUInt64',
                request_serializer=nidaqmx__pb2.SetTimingAttributeExUInt64Request.SerializeToString,
                response_deserializer=nidaqmx__pb2.SetTimingAttributeExUInt64Response.FromString,
                )
        self.SetTimingAttributeInt32 = channel.unary_unary(
                '/nidaqmx_grpc.NiDAQmx/SetTimingAttributeInt32',
                request_serializer=nidaqmx__pb2.SetTimingAttributeInt32Request.SerializeToString,
                response_deserializer=nidaqmx__pb2.SetTimingAttributeInt32Response.FromString,
                )
        self.SetTimingAttributeString = channel.unary_unary(
                '/nidaqmx_grpc.NiDAQmx/SetTimingAttributeString',
                request_serializer=nidaqmx__pb2.SetTimingAttributeStringRequest.SerializeToString,
                response_deserializer=nidaqmx__pb2.SetTimingAttributeStringResponse.FromString,
                )
        self.SetTimingAttributeTimestamp = channel.unary_unary(
                '/nidaqmx_grpc.NiDAQmx/SetTimingAttributeTimestamp',
                request_serializer=nidaqmx__pb2.SetTimingAttributeTimestampRequest.SerializeToString,
                response_deserializer=nidaqmx__pb2.SetTimingAttributeTimestampResponse.FromString,
                )
        self.SetTimingAttributeUInt32 = channel.unary_unary(
                '/nidaqmx_grpc.NiDAQmx/SetTimingAttributeUInt32',
                request_serializer=nidaqmx__pb2.SetTimingAttributeUInt32Request.SerializeToString,
                response_deserializer=nidaqmx__pb2.SetTimingAttributeUInt32Response.FromString,
                )
        self.SetTimingAttributeUInt64 = channel.unary_unary(
                '/nidaqmx_grpc.NiDAQmx/SetTimingAttributeUInt64',
                request_serializer=nidaqmx__pb2.SetTimingAttributeUInt64Request.SerializeToString,
                response_deserializer=nidaqmx__pb2.SetTimingAttributeUInt64Response.FromString,
                )
        self.SetTrigAttributeBool = channel.unary_unary(
                '/nidaqmx_grpc.NiDAQmx/SetTrigAttributeBool',
                request_serializer=nidaqmx__pb2.SetTrigAttributeBoolRequest.SerializeToString,
                response_deserializer=nidaqmx__pb2.SetTrigAttributeBoolResponse.FromString,
                )
        self.SetTrigAttributeDouble = channel.unary_unary(
                '/nidaqmx_grpc.NiDAQmx/SetTrigAttributeDouble',
                request_serializer=nidaqmx__pb2.SetTrigAttributeDoubleRequest.SerializeToString,
                response_deserializer=nidaqmx__pb2.SetTrigAttributeDoubleResponse.FromString,
                )
        self.SetTrigAttributeDoubleArray = channel.unary_unary(
                '/nidaqmx_grpc.NiDAQmx/SetTrigAttributeDoubleArray',
                request_serializer=nidaqmx__pb2.SetTrigAttributeDoubleArrayRequest.SerializeToString,
                response_deserializer=nidaqmx__pb2.SetTrigAttributeDoubleArrayResponse.FromString,
                )
        self.SetTrigAttributeInt32 = channel.unary_unary(
                '/nidaqmx_grpc.NiDAQmx/SetTrigAttributeInt32',
                request_serializer=nidaqmx__pb2.SetTrigAttributeInt32Request.SerializeToString,
                response_deserializer=nidaqmx__pb2.SetTrigAttributeInt32Response.FromString,
                )
        self.SetTrigAttributeInt32Array = channel.unary_unary(
                '/nidaqmx_grpc.NiDAQmx/SetTrigAttributeInt32Array',
                request_serializer=nidaqmx__pb2.SetTrigAttributeInt32ArrayRequest.SerializeToString,
                response_deserializer=nidaqmx__pb2.SetTrigAttributeInt32ArrayResponse.FromString,
                )
        self.SetTrigAttributeString = channel.unary_unary(
                '/nidaqmx_grpc.NiDAQmx/SetTrigAttributeString',
                request_serializer=nidaqmx__pb2.SetTrigAttributeStringRequest.SerializeToString,
                response_deserializer=nidaqmx__pb2.SetTrigAttributeStringResponse.FromString,
                )
        self.SetTrigAttributeTimestamp = channel.unary_unary(
                '/nidaqmx_grpc.NiDAQmx/SetTrigAttributeTimestamp',
                request_serializer=nidaqmx__pb2.SetTrigAttributeTimestampRequest.SerializeToString,
                response_deserializer=nidaqmx__pb2.SetTrigAttributeTimestampResponse.FromString,
                )
        self.SetTrigAttributeUInt32 = channel.unary_unary(
                '/nidaqmx_grpc.NiDAQmx/SetTrigAttributeUInt32',
                request_serializer=nidaqmx__pb2.SetTrigAttributeUInt32Request.SerializeToString,
                response_deserializer=nidaqmx__pb2.SetTrigAttributeUInt32Response.FromString,
                )
        self.SetWatchdogAttributeBool = channel.unary_unary(
                '/nidaqmx_grpc.NiDAQmx/SetWatchdogAttributeBool',
                request_serializer=nidaqmx__pb2.SetWatchdogAttributeBoolRequest.SerializeToString,
                response_deserializer=nidaqmx__pb2.SetWatchdogAttributeBoolResponse.FromString,
                )
        self.SetWatchdogAttributeDouble = channel.unary_unary(
                '/nidaqmx_grpc.NiDAQmx/SetWatchdogAttributeDouble',
                request_serializer=nidaqmx__pb2.SetWatchdogAttributeDoubleRequest.SerializeToString,
                response_deserializer=nidaqmx__pb2.SetWatchdogAttributeDoubleResponse.FromString,
                )
        self.SetWatchdogAttributeInt32 = channel.unary_unary(
                '/nidaqmx_grpc.NiDAQmx/SetWatchdogAttributeInt32',
                request_serializer=nidaqmx__pb2.SetWatchdogAttributeInt32Request.SerializeToString,
                response_deserializer=nidaqmx__pb2.SetWatchdogAttributeInt32Response.FromString,
                )
        self.SetWatchdogAttributeString = channel.unary_unary(
                '/nidaqmx_grpc.NiDAQmx/SetWatchdogAttributeString',
                request_serializer=nidaqmx__pb2.SetWatchdogAttributeStringRequest.SerializeToString,
                response_deserializer=nidaqmx__pb2.SetWatchdogAttributeStringResponse.FromString,
                )
        self.SetWriteAttributeBool = channel.unary_unary(
                '/nidaqmx_grpc.NiDAQmx/SetWriteAttributeBool',
                request_serializer=nidaqmx__pb2.SetWriteAttributeBoolRequest.SerializeToString,
                response_deserializer=nidaqmx__pb2.SetWriteAttributeBoolResponse.FromString,
                )
        self.SetWriteAttributeDouble = channel.unary_unary(
                '/nidaqmx_grpc.NiDAQmx/SetWriteAttributeDouble',
                request_serializer=nidaqmx__pb2.SetWriteAttributeDoubleRequest.SerializeToString,
                response_deserializer=nidaqmx__pb2.SetWriteAttributeDoubleResponse.FromString,
                )
        self.SetWriteAttributeInt32 = channel.unary_unary(
                '/nidaqmx_grpc.NiDAQmx/SetWriteAttributeInt32',
                request_serializer=nidaqmx__pb2.SetWriteAttributeInt32Request.SerializeToString,
                response_deserializer=nidaqmx__pb2.SetWriteAttributeInt32Response.FromString,
                )
        self.SetWriteAttributeString = channel.unary_unary(
                '/nidaqmx_grpc.NiDAQmx/SetWriteAttributeString',
                request_serializer=nidaqmx__pb2.SetWriteAttributeStringRequest.SerializeToString,
                response_deserializer=nidaqmx__pb2.SetWriteAttributeStringResponse.FromString,
                )
        self.SetWriteAttributeUInt32 = channel.unary_unary(
                '/nidaqmx_grpc.NiDAQmx/SetWriteAttributeUInt32',
                request_serializer=nidaqmx__pb2.SetWriteAttributeUInt32Request.SerializeToString,
                response_deserializer=nidaqmx__pb2.SetWriteAttributeUInt32Response.FromString,
                )
        self.SetWriteAttributeUInt64 = channel.unary_unary(
                '/nidaqmx_grpc.NiDAQmx/SetWriteAttributeUInt64',
                request_serializer=nidaqmx__pb2.SetWriteAttributeUInt64Request.SerializeToString,
                response_deserializer=nidaqmx__pb2.SetWriteAttributeUInt64Response.FromString,
                )
        self.StartNewFile = channel.unary_unary(
                '/nidaqmx_grpc.NiDAQmx/StartNewFile',
                request_serializer=nidaqmx__pb2.StartNewFileRequest.SerializeToString,
                response_deserializer=nidaqmx__pb2.StartNewFileResponse.FromString,
                )
        self.StartTask = channel.unary_unary(
                '/nidaqmx_grpc.NiDAQmx/StartTask',
                request_serializer=nidaqmx__pb2.StartTaskRequest.SerializeToString,
                response_deserializer=nidaqmx__pb2.StartTaskResponse.FromString,
                )
        self.StopTask = channel.unary_unary(
                '/nidaqmx_grpc.NiDAQmx/StopTask',
                request_serializer=nidaqmx__pb2.StopTaskRequest.SerializeToString,
                response_deserializer=nidaqmx__pb2.StopTaskResponse.FromString,
                )
        self.TaskControl = channel.unary_unary(
                '/nidaqmx_grpc.NiDAQmx/TaskControl',
                request_serializer=nidaqmx__pb2.TaskControlRequest.SerializeToString,
                response_deserializer=nidaqmx__pb2.TaskControlResponse.FromString,
                )
        self.TristateOutputTerm = channel.unary_unary(
                '/nidaqmx_grpc.NiDAQmx/TristateOutputTerm',
                request_serializer=nidaqmx__pb2.TristateOutputTermRequest.SerializeToString,
                response_deserializer=nidaqmx__pb2.TristateOutputTermResponse.FromString,
                )
        self.UnregisterDoneEvent = channel.unary_unary(
                '/nidaqmx_grpc.NiDAQmx/UnregisterDoneEvent',
                request_serializer=nidaqmx__pb2.UnregisterDoneEventRequest.SerializeToString,
                response_deserializer=nidaqmx__pb2.UnregisterDoneEventResponse.FromString,
                )
        self.UnregisterEveryNSamplesEvent = channel.unary_unary(
                '/nidaqmx_grpc.NiDAQmx/UnregisterEveryNSamplesEvent',
                request_serializer=nidaqmx__pb2.UnregisterEveryNSamplesEventRequest.SerializeToString,
                response_deserializer=nidaqmx__pb2.UnregisterEveryNSamplesEventResponse.FromString,
                )
        self.UnregisterSignalEvent = channel.unary_unary(
                '/nidaqmx_grpc.NiDAQmx/UnregisterSignalEvent',
                request_serializer=nidaqmx__pb2.UnregisterSignalEventRequest.SerializeToString,
                response_deserializer=nidaqmx__pb2.UnregisterSignalEventResponse.FromString,
                )
        self.UnreserveNetworkDevice = channel.unary_unary(
                '/nidaqmx_grpc.NiDAQmx/UnreserveNetworkDevice',
                request_serializer=nidaqmx__pb2.UnreserveNetworkDeviceRequest.SerializeToString,
                response_deserializer=nidaqmx__pb2.UnreserveNetworkDeviceResponse.FromString,
                )
        self.WaitForNextSampleClock = channel.unary_unary(
                '/nidaqmx_grpc.NiDAQmx/WaitForNextSampleClock',
                request_serializer=nidaqmx__pb2.WaitForNextSampleClockRequest.SerializeToString,
                response_deserializer=nidaqmx__pb2.WaitForNextSampleClockResponse.FromString,
                )
        self.WaitForValidTimestamp = channel.unary_unary(
                '/nidaqmx_grpc.NiDAQmx/WaitForValidTimestamp',
                request_serializer=nidaqmx__pb2.WaitForValidTimestampRequest.SerializeToString,
                response_deserializer=nidaqmx__pb2.WaitForValidTimestampResponse.FromString,
                )
        self.WaitUntilTaskDone = channel.unary_unary(
                '/nidaqmx_grpc.NiDAQmx/WaitUntilTaskDone',
                request_serializer=nidaqmx__pb2.WaitUntilTaskDoneRequest.SerializeToString,
                response_deserializer=nidaqmx__pb2.WaitUntilTaskDoneResponse.FromString,
                )
        self.WriteAnalogF64 = channel.unary_unary(
                '/nidaqmx_grpc.NiDAQmx/WriteAnalogF64',
                request_serializer=nidaqmx__pb2.WriteAnalogF64Request.SerializeToString,
                response_deserializer=nidaqmx__pb2.WriteAnalogF64Response.FromString,
                )
        self.WriteAnalogScalarF64 = channel.unary_unary(
                '/nidaqmx_grpc.NiDAQmx/WriteAnalogScalarF64',
                request_serializer=nidaqmx__pb2.WriteAnalogScalarF64Request.SerializeToString,
                response_deserializer=nidaqmx__pb2.WriteAnalogScalarF64Response.FromString,
                )
        self.WriteBinaryI16 = channel.unary_unary(
                '/nidaqmx_grpc.NiDAQmx/WriteBinaryI16',
                request_serializer=nidaqmx__pb2.WriteBinaryI16Request.SerializeToString,
                response_deserializer=nidaqmx__pb2.WriteBinaryI16Response.FromString,
                )
        self.WriteBinaryI32 = channel.unary_unary(
                '/nidaqmx_grpc.NiDAQmx/WriteBinaryI32',
                request_serializer=nidaqmx__pb2.WriteBinaryI32Request.SerializeToString,
                response_deserializer=nidaqmx__pb2.WriteBinaryI32Response.FromString,
                )
        self.WriteBinaryU16 = channel.unary_unary(
                '/nidaqmx_grpc.NiDAQmx/WriteBinaryU16',
                request_serializer=nidaqmx__pb2.WriteBinaryU16Request.SerializeToString,
                response_deserializer=nidaqmx__pb2.WriteBinaryU16Response.FromString,
                )
        self.WriteBinaryU32 = channel.unary_unary(
                '/nidaqmx_grpc.NiDAQmx/WriteBinaryU32',
                request_serializer=nidaqmx__pb2.WriteBinaryU32Request.SerializeToString,
                response_deserializer=nidaqmx__pb2.WriteBinaryU32Response.FromString,
                )
        self.WriteCtrFreq = channel.unary_unary(
                '/nidaqmx_grpc.NiDAQmx/WriteCtrFreq',
                request_serializer=nidaqmx__pb2.WriteCtrFreqRequest.SerializeToString,
                response_deserializer=nidaqmx__pb2.WriteCtrFreqResponse.FromString,
                )
        self.WriteCtrFreqScalar = channel.unary_unary(
                '/nidaqmx_grpc.NiDAQmx/WriteCtrFreqScalar',
                request_serializer=nidaqmx__pb2.WriteCtrFreqScalarRequest.SerializeToString,
                response_deserializer=nidaqmx__pb2.WriteCtrFreqScalarResponse.FromString,
                )
        self.WriteCtrTicks = channel.unary_unary(
                '/nidaqmx_grpc.NiDAQmx/WriteCtrTicks',
                request_serializer=nidaqmx__pb2.WriteCtrTicksRequest.SerializeToString,
                response_deserializer=nidaqmx__pb2.WriteCtrTicksResponse.FromString,
                )
        self.WriteCtrTicksScalar = channel.unary_unary(
                '/nidaqmx_grpc.NiDAQmx/WriteCtrTicksScalar',
                request_serializer=nidaqmx__pb2.WriteCtrTicksScalarRequest.SerializeToString,
                response_deserializer=nidaqmx__pb2.WriteCtrTicksScalarResponse.FromString,
                )
        self.WriteCtrTime = channel.unary_unary(
                '/nidaqmx_grpc.NiDAQmx/WriteCtrTime',
                request_serializer=nidaqmx__pb2.WriteCtrTimeRequest.SerializeToString,
                response_deserializer=nidaqmx__pb2.WriteCtrTimeResponse.FromString,
                )
        self.WriteCtrTimeScalar = channel.unary_unary(
                '/nidaqmx_grpc.NiDAQmx/WriteCtrTimeScalar',
                request_serializer=nidaqmx__pb2.WriteCtrTimeScalarRequest.SerializeToString,
                response_deserializer=nidaqmx__pb2.WriteCtrTimeScalarResponse.FromString,
                )
        self.WriteDigitalLines = channel.unary_unary(
                '/nidaqmx_grpc.NiDAQmx/WriteDigitalLines',
                request_serializer=nidaqmx__pb2.WriteDigitalLinesRequest.SerializeToString,
                response_deserializer=nidaqmx__pb2.WriteDigitalLinesResponse.FromString,
                )
        self.WriteDigitalScalarU32 = channel.unary_unary(
                '/nidaqmx_grpc.NiDAQmx/WriteDigitalScalarU32',
                request_serializer=nidaqmx__pb2.WriteDigitalScalarU32Request.SerializeToString,
                response_deserializer=nidaqmx__pb2.WriteDigitalScalarU32Response.FromString,
                )
        self.WriteDigitalU16 = channel.unary_unary(
                '/nidaqmx_grpc.NiDAQmx/WriteDigitalU16',
                request_serializer=nidaqmx__pb2.WriteDigitalU16Request.SerializeToString,
                response_deserializer=nidaqmx__pb2.WriteDigitalU16Response.FromString,
                )
        self.WriteDigitalU32 = channel.unary_unary(
                '/nidaqmx_grpc.NiDAQmx/WriteDigitalU32',
                request_serializer=nidaqmx__pb2.WriteDigitalU32Request.SerializeToString,
                response_deserializer=nidaqmx__pb2.WriteDigitalU32Response.FromString,
                )
        self.WriteDigitalU8 = channel.unary_unary(
                '/nidaqmx_grpc.NiDAQmx/WriteDigitalU8',
                request_serializer=nidaqmx__pb2.WriteDigitalU8Request.SerializeToString,
                response_deserializer=nidaqmx__pb2.WriteDigitalU8Response.FromString,
                )
        self.WriteRaw = channel.unary_unary(
                '/nidaqmx_grpc.NiDAQmx/WriteRaw',
                request_serializer=nidaqmx__pb2.WriteRawRequest.SerializeToString,
                response_deserializer=nidaqmx__pb2.WriteRawResponse.FromString,
                )
        self.WriteToTEDSFromArray = channel.unary_unary(
                '/nidaqmx_grpc.NiDAQmx/WriteToTEDSFromArray',
                request_serializer=nidaqmx__pb2.WriteToTEDSFromArrayRequest.SerializeToString,
                response_deserializer=nidaqmx__pb2.WriteToTEDSFromArrayResponse.FromString,
                )
        self.WriteToTEDSFromFile = channel.unary_unary(
                '/nidaqmx_grpc.NiDAQmx/WriteToTEDSFromFile',
                request_serializer=nidaqmx__pb2.WriteToTEDSFromFileRequest.SerializeToString,
                response_deserializer=nidaqmx__pb2.WriteToTEDSFromFileResponse.FromString,
                )


class NiDAQmxServicer(object):
    """Missing associated documentation comment in .proto file."""

    def AddCDAQSyncConnection(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def AddGlobalChansToTask(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def AddNetworkDevice(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def AreConfiguredCDAQSyncPortsDisconnected(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def AutoConfigureCDAQSyncConnections(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def CalculateReversePolyCoeff(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def CfgAnlgEdgeRefTrig(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def CfgAnlgEdgeStartTrig(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def CfgAnlgMultiEdgeRefTrig(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def CfgAnlgMultiEdgeStartTrig(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def CfgAnlgWindowRefTrig(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def CfgAnlgWindowStartTrig(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def CfgBurstHandshakingTimingExportClock(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def CfgBurstHandshakingTimingImportClock(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def CfgChangeDetectionTiming(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def CfgDigEdgeRefTrig(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def CfgDigEdgeStartTrig(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def CfgDigPatternRefTrig(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def CfgDigPatternStartTrig(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def CfgHandshakingTiming(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def CfgImplicitTiming(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def CfgInputBuffer(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def CfgOutputBuffer(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def CfgPipelinedSampClkTiming(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def CfgSampClkTiming(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def CfgTimeStartTrig(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def CfgWatchdogAOExpirStates(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def CfgWatchdogCOExpirStates(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def CfgWatchdogDOExpirStates(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def ClearTEDS(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def ClearTask(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def ConfigureLogging(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def ConfigureTEDS(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def ConnectTerms(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def ControlWatchdogTask(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def CreateAIAccel4WireDCVoltageChan(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def CreateAIAccelChan(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def CreateAIAccelChargeChan(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def CreateAIBridgeChan(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def CreateAIChargeChan(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def CreateAICurrentChan(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def CreateAICurrentRMSChan(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def CreateAIForceBridgePolynomialChan(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def CreateAIForceBridgeTableChan(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def CreateAIForceBridgeTwoPointLinChan(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def CreateAIForceIEPEChan(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def CreateAIFreqVoltageChan(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def CreateAIMicrophoneChan(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def CreateAIPosEddyCurrProxProbeChan(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def CreateAIPosLVDTChan(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def CreateAIPosRVDTChan(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def CreateAIPowerChan(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def CreateAIPressureBridgePolynomialChan(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def CreateAIPressureBridgeTableChan(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def CreateAIPressureBridgeTwoPointLinChan(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def CreateAIRTDChan(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def CreateAIResistanceChan(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def CreateAIRosetteStrainGageChan(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def CreateAIStrainGageChan(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def CreateAITempBuiltInSensorChan(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def CreateAIThrmcplChan(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def CreateAIThrmstrChanIex(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def CreateAIThrmstrChanVex(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def CreateAITorqueBridgePolynomialChan(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def CreateAITorqueBridgeTableChan(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def CreateAITorqueBridgeTwoPointLinChan(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def CreateAIVelocityIEPEChan(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def CreateAIVoltageChan(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def CreateAIVoltageChanWithExcit(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def CreateAIVoltageRMSChan(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def CreateAOCurrentChan(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def CreateAOFuncGenChan(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def CreateAOVoltageChan(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def CreateCIAngEncoderChan(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def CreateCIAngVelocityChan(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def CreateCICountEdgesChan(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def CreateCIDutyCycleChan(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def CreateCIFreqChan(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def CreateCIGPSTimestampChan(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def CreateCILinEncoderChan(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def CreateCILinVelocityChan(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def CreateCIPeriodChan(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def CreateCIPulseChanFreq(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def CreateCIPulseChanTicks(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def CreateCIPulseChanTime(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def CreateCIPulseWidthChan(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def CreateCISemiPeriodChan(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def CreateCITwoEdgeSepChan(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def CreateCOPulseChanFreq(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def CreateCOPulseChanTicks(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def CreateCOPulseChanTime(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def CreateDIChan(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def CreateDOChan(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def CreateLinScale(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def CreateMapScale(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def CreatePolynomialScale(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def CreateTEDSAIAccelChan(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def CreateTEDSAIBridgeChan(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def CreateTEDSAICurrentChan(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def CreateTEDSAIForceBridgeChan(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def CreateTEDSAIForceIEPEChan(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def CreateTEDSAIMicrophoneChan(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def CreateTEDSAIPosLVDTChan(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def CreateTEDSAIPosRVDTChan(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def CreateTEDSAIPressureBridgeChan(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def CreateTEDSAIRTDChan(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def CreateTEDSAIResistanceChan(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def CreateTEDSAIStrainGageChan(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def CreateTEDSAIThrmcplChan(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def CreateTEDSAIThrmstrChanIex(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def CreateTEDSAIThrmstrChanVex(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def CreateTEDSAITorqueBridgeChan(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def CreateTEDSAIVoltageChan(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def CreateTEDSAIVoltageChanWithExcit(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def CreateTableScale(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def CreateTask(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def CreateWatchdogTimerTask(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def CreateWatchdogTimerTaskEx(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def DeleteNetworkDevice(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def DeleteSavedGlobalChan(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def DeleteSavedScale(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def DeleteSavedTask(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def DeviceSupportsCal(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def DisableRefTrig(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def DisableStartTrig(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def DisconnectTerms(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def ExportSignal(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def GetAIChanCalCalDate(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def GetAIChanCalExpDate(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def GetAnalogPowerUpStates(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def GetAnalogPowerUpStatesWithOutputType(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def GetArmStartTrigTimestampVal(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def GetArmStartTrigTrigWhen(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def GetAutoConfiguredCDAQSyncConnections(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def GetBufferAttributeUInt32(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def GetCalInfoAttributeBool(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def GetCalInfoAttributeDouble(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def GetCalInfoAttributeString(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def GetCalInfoAttributeUInt32(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def GetChanAttributeBool(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def GetChanAttributeDouble(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def GetChanAttributeDoubleArray(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def GetChanAttributeInt32(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def GetChanAttributeString(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def GetChanAttributeUInt32(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def GetDeviceAttributeBool(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def GetDeviceAttributeDouble(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def GetDeviceAttributeDoubleArray(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def GetDeviceAttributeInt32(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def GetDeviceAttributeInt32Array(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def GetDeviceAttributeString(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def GetDeviceAttributeUInt32(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def GetDeviceAttributeUInt32Array(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def GetDigitalLogicFamilyPowerUpState(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def GetDigitalPowerUpStates(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def GetDigitalPullUpPullDownStates(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def GetDisconnectedCDAQSyncPorts(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def GetErrorString(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def GetExportedSignalAttributeBool(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def GetExportedSignalAttributeDouble(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def GetExportedSignalAttributeInt32(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def GetExportedSignalAttributeString(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def GetExportedSignalAttributeUInt32(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def GetExtCalLastDateAndTime(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def GetFirstSampClkWhen(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def GetFirstSampTimestampVal(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def GetNthTaskChannel(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def GetNthTaskDevice(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def GetNthTaskReadChannel(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def GetPersistedChanAttributeBool(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def GetPersistedChanAttributeString(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def GetPersistedScaleAttributeBool(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def GetPersistedScaleAttributeString(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def GetPersistedTaskAttributeBool(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def GetPersistedTaskAttributeString(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def GetPhysicalChanAttributeBool(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def GetPhysicalChanAttributeBytes(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def GetPhysicalChanAttributeDouble(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def GetPhysicalChanAttributeDoubleArray(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def GetPhysicalChanAttributeInt32(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def GetPhysicalChanAttributeInt32Array(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def GetPhysicalChanAttributeString(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def GetPhysicalChanAttributeUInt32(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def GetPhysicalChanAttributeUInt32Array(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def GetReadAttributeBool(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def GetReadAttributeDouble(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def GetReadAttributeInt32(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def GetReadAttributeString(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def GetReadAttributeUInt32(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def GetReadAttributeUInt64(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def GetRealTimeAttributeBool(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def GetRealTimeAttributeInt32(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def GetRealTimeAttributeUInt32(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def GetRefTrigTimestampVal(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def GetScaleAttributeDouble(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def GetScaleAttributeDoubleArray(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def GetScaleAttributeInt32(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def GetScaleAttributeString(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def GetSelfCalLastDateAndTime(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def GetStartTrigTimestampVal(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def GetStartTrigTrigWhen(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def GetSyncPulseTimeWhen(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def GetSystemInfoAttributeString(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def GetSystemInfoAttributeUInt32(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def GetTaskAttributeBool(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def GetTaskAttributeString(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def GetTaskAttributeUInt32(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def GetTimingAttributeBool(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def GetTimingAttributeDouble(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def GetTimingAttributeExBool(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def GetTimingAttributeExDouble(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def GetTimingAttributeExInt32(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def GetTimingAttributeExString(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def GetTimingAttributeExTimestamp(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def GetTimingAttributeExUInt32(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def GetTimingAttributeExUInt64(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def GetTimingAttributeInt32(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def GetTimingAttributeString(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def GetTimingAttributeTimestamp(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def GetTimingAttributeUInt32(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def GetTimingAttributeUInt64(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def GetTrigAttributeBool(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def GetTrigAttributeDouble(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def GetTrigAttributeDoubleArray(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def GetTrigAttributeInt32(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def GetTrigAttributeInt32Array(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def GetTrigAttributeString(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def GetTrigAttributeTimestamp(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def GetTrigAttributeUInt32(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def GetWatchdogAttributeBool(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def GetWatchdogAttributeDouble(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def GetWatchdogAttributeInt32(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def GetWatchdogAttributeString(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def GetWriteAttributeBool(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def GetWriteAttributeDouble(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def GetWriteAttributeInt32(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def GetWriteAttributeString(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def GetWriteAttributeUInt32(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def GetWriteAttributeUInt64(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def IsTaskDone(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def LoadTask(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def PerformBridgeOffsetNullingCalEx(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def PerformBridgeShuntCalEx(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def PerformStrainShuntCalEx(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def PerformThrmcplLeadOffsetNullingCal(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def ReadAnalogF64(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def ReadAnalogScalarF64(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def ReadBinaryI16(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def ReadBinaryI32(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def ReadBinaryU16(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def ReadBinaryU32(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def ReadCounterF64(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def ReadCounterF64Ex(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def ReadCounterScalarF64(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def ReadCounterScalarU32(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def ReadCounterU32(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def ReadCounterU32Ex(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def ReadCtrFreq(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def ReadCtrFreqScalar(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def ReadCtrTicks(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def ReadCtrTicksScalar(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def ReadCtrTime(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def ReadCtrTimeScalar(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def ReadDigitalLines(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def ReadDigitalScalarU32(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def ReadDigitalU16(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def ReadDigitalU32(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def ReadDigitalU8(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def ReadPowerBinaryI16(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def ReadPowerF64(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def ReadPowerScalarF64(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def ReadRaw(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def RegisterDoneEvent(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def RegisterEveryNSamplesEvent(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def RegisterSignalEvent(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def RemoveCDAQSyncConnection(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def ReserveNetworkDevice(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def ResetBufferAttribute(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def ResetChanAttribute(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def ResetDevice(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def ResetExportedSignalAttribute(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def ResetReadAttribute(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def ResetRealTimeAttribute(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def ResetTimingAttribute(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def ResetTimingAttributeEx(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def ResetTrigAttribute(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def ResetWatchdogAttribute(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def ResetWriteAttribute(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def RestoreLastExtCalConst(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def SaveGlobalChan(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def SaveScale(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def SaveTask(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def SelfCal(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def SelfTestDevice(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def SetAIChanCalCalDate(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def SetAIChanCalExpDate(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def SetAnalogPowerUpStates(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def SetAnalogPowerUpStatesWithOutputType(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def SetArmStartTrigTrigWhen(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def SetBufferAttributeUInt32(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def SetCalInfoAttributeBool(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def SetCalInfoAttributeDouble(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def SetCalInfoAttributeString(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def SetCalInfoAttributeUInt32(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def SetChanAttributeBool(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def SetChanAttributeDouble(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def SetChanAttributeDoubleArray(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def SetChanAttributeInt32(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def SetChanAttributeString(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def SetChanAttributeUInt32(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def SetDigitalLogicFamilyPowerUpState(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def SetDigitalPowerUpStates(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def SetDigitalPullUpPullDownStates(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def SetExportedSignalAttributeBool(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def SetExportedSignalAttributeDouble(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def SetExportedSignalAttributeInt32(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def SetExportedSignalAttributeString(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def SetExportedSignalAttributeUInt32(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def SetFirstSampClkWhen(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def SetReadAttributeBool(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def SetReadAttributeDouble(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def SetReadAttributeInt32(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def SetReadAttributeString(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def SetReadAttributeUInt32(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def SetReadAttributeUInt64(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def SetRealTimeAttributeBool(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def SetRealTimeAttributeInt32(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def SetRealTimeAttributeUInt32(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def SetScaleAttributeDouble(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def SetScaleAttributeDoubleArray(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def SetScaleAttributeInt32(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def SetScaleAttributeString(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def SetStartTrigTrigWhen(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def SetSyncPulseTimeWhen(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def SetTimingAttributeBool(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def SetTimingAttributeDouble(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def SetTimingAttributeExBool(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def SetTimingAttributeExDouble(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def SetTimingAttributeExInt32(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def SetTimingAttributeExString(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def SetTimingAttributeExTimestamp(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def SetTimingAttributeExUInt32(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def SetTimingAttributeExUInt64(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def SetTimingAttributeInt32(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def SetTimingAttributeString(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def SetTimingAttributeTimestamp(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def SetTimingAttributeUInt32(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def SetTimingAttributeUInt64(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def SetTrigAttributeBool(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def SetTrigAttributeDouble(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def SetTrigAttributeDoubleArray(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def SetTrigAttributeInt32(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def SetTrigAttributeInt32Array(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def SetTrigAttributeString(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def SetTrigAttributeTimestamp(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def SetTrigAttributeUInt32(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def SetWatchdogAttributeBool(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def SetWatchdogAttributeDouble(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def SetWatchdogAttributeInt32(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def SetWatchdogAttributeString(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def SetWriteAttributeBool(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def SetWriteAttributeDouble(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def SetWriteAttributeInt32(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def SetWriteAttributeString(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def SetWriteAttributeUInt32(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def SetWriteAttributeUInt64(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def StartNewFile(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def StartTask(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def StopTask(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def TaskControl(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def TristateOutputTerm(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def UnregisterDoneEvent(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def UnregisterEveryNSamplesEvent(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def UnregisterSignalEvent(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def UnreserveNetworkDevice(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def WaitForNextSampleClock(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def WaitForValidTimestamp(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def WaitUntilTaskDone(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def WriteAnalogF64(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def WriteAnalogScalarF64(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def WriteBinaryI16(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def WriteBinaryI32(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def WriteBinaryU16(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def WriteBinaryU32(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def WriteCtrFreq(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def WriteCtrFreqScalar(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def WriteCtrTicks(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def WriteCtrTicksScalar(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def WriteCtrTime(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def WriteCtrTimeScalar(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def WriteDigitalLines(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def WriteDigitalScalarU32(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def WriteDigitalU16(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def WriteDigitalU32(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def WriteDigitalU8(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def WriteRaw(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def WriteToTEDSFromArray(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def WriteToTEDSFromFile(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')


def add_NiDAQmxServicer_to_server(servicer, server):
    rpc_method_handlers = {
            'AddCDAQSyncConnection': grpc.unary_unary_rpc_method_handler(
                    servicer.AddCDAQSyncConnection,
                    request_deserializer=nidaqmx__pb2.AddCDAQSyncConnectionRequest.FromString,
                    response_serializer=nidaqmx__pb2.AddCDAQSyncConnectionResponse.SerializeToString,
            ),
            'AddGlobalChansToTask': grpc.unary_unary_rpc_method_handler(
                    servicer.AddGlobalChansToTask,
                    request_deserializer=nidaqmx__pb2.AddGlobalChansToTaskRequest.FromString,
                    response_serializer=nidaqmx__pb2.AddGlobalChansToTaskResponse.SerializeToString,
            ),
            'AddNetworkDevice': grpc.unary_unary_rpc_method_handler(
                    servicer.AddNetworkDevice,
                    request_deserializer=nidaqmx__pb2.AddNetworkDeviceRequest.FromString,
                    response_serializer=nidaqmx__pb2.AddNetworkDeviceResponse.SerializeToString,
            ),
            'AreConfiguredCDAQSyncPortsDisconnected': grpc.unary_unary_rpc_method_handler(
                    servicer.AreConfiguredCDAQSyncPortsDisconnected,
                    request_deserializer=nidaqmx__pb2.AreConfiguredCDAQSyncPortsDisconnectedRequest.FromString,
                    response_serializer=nidaqmx__pb2.AreConfiguredCDAQSyncPortsDisconnectedResponse.SerializeToString,
            ),
            'AutoConfigureCDAQSyncConnections': grpc.unary_unary_rpc_method_handler(
                    servicer.AutoConfigureCDAQSyncConnections,
                    request_deserializer=nidaqmx__pb2.AutoConfigureCDAQSyncConnectionsRequest.FromString,
                    response_serializer=nidaqmx__pb2.AutoConfigureCDAQSyncConnectionsResponse.SerializeToString,
            ),
            'CalculateReversePolyCoeff': grpc.unary_unary_rpc_method_handler(
                    servicer.CalculateReversePolyCoeff,
                    request_deserializer=nidaqmx__pb2.CalculateReversePolyCoeffRequest.FromString,
                    response_serializer=nidaqmx__pb2.CalculateReversePolyCoeffResponse.SerializeToString,
            ),
            'CfgAnlgEdgeRefTrig': grpc.unary_unary_rpc_method_handler(
                    servicer.CfgAnlgEdgeRefTrig,
                    request_deserializer=nidaqmx__pb2.CfgAnlgEdgeRefTrigRequest.FromString,
                    response_serializer=nidaqmx__pb2.CfgAnlgEdgeRefTrigResponse.SerializeToString,
            ),
            'CfgAnlgEdgeStartTrig': grpc.unary_unary_rpc_method_handler(
                    servicer.CfgAnlgEdgeStartTrig,
                    request_deserializer=nidaqmx__pb2.CfgAnlgEdgeStartTrigRequest.FromString,
                    response_serializer=nidaqmx__pb2.CfgAnlgEdgeStartTrigResponse.SerializeToString,
            ),
            'CfgAnlgMultiEdgeRefTrig': grpc.unary_unary_rpc_method_handler(
                    servicer.CfgAnlgMultiEdgeRefTrig,
                    request_deserializer=nidaqmx__pb2.CfgAnlgMultiEdgeRefTrigRequest.FromString,
                    response_serializer=nidaqmx__pb2.CfgAnlgMultiEdgeRefTrigResponse.SerializeToString,
            ),
            'CfgAnlgMultiEdgeStartTrig': grpc.unary_unary_rpc_method_handler(
                    servicer.CfgAnlgMultiEdgeStartTrig,
                    request_deserializer=nidaqmx__pb2.CfgAnlgMultiEdgeStartTrigRequest.FromString,
                    response_serializer=nidaqmx__pb2.CfgAnlgMultiEdgeStartTrigResponse.SerializeToString,
            ),
            'CfgAnlgWindowRefTrig': grpc.unary_unary_rpc_method_handler(
                    servicer.CfgAnlgWindowRefTrig,
                    request_deserializer=nidaqmx__pb2.CfgAnlgWindowRefTrigRequest.FromString,
                    response_serializer=nidaqmx__pb2.CfgAnlgWindowRefTrigResponse.SerializeToString,
            ),
            'CfgAnlgWindowStartTrig': grpc.unary_unary_rpc_method_handler(
                    servicer.CfgAnlgWindowStartTrig,
                    request_deserializer=nidaqmx__pb2.CfgAnlgWindowStartTrigRequest.FromString,
                    response_serializer=nidaqmx__pb2.CfgAnlgWindowStartTrigResponse.SerializeToString,
            ),
            'CfgBurstHandshakingTimingExportClock': grpc.unary_unary_rpc_method_handler(
                    servicer.CfgBurstHandshakingTimingExportClock,
                    request_deserializer=nidaqmx__pb2.CfgBurstHandshakingTimingExportClockRequest.FromString,
                    response_serializer=nidaqmx__pb2.CfgBurstHandshakingTimingExportClockResponse.SerializeToString,
            ),
            'CfgBurstHandshakingTimingImportClock': grpc.unary_unary_rpc_method_handler(
                    servicer.CfgBurstHandshakingTimingImportClock,
                    request_deserializer=nidaqmx__pb2.CfgBurstHandshakingTimingImportClockRequest.FromString,
                    response_serializer=nidaqmx__pb2.CfgBurstHandshakingTimingImportClockResponse.SerializeToString,
            ),
            'CfgChangeDetectionTiming': grpc.unary_unary_rpc_method_handler(
                    servicer.CfgChangeDetectionTiming,
                    request_deserializer=nidaqmx__pb2.CfgChangeDetectionTimingRequest.FromString,
                    response_serializer=nidaqmx__pb2.CfgChangeDetectionTimingResponse.SerializeToString,
            ),
            'CfgDigEdgeRefTrig': grpc.unary_unary_rpc_method_handler(
                    servicer.CfgDigEdgeRefTrig,
                    request_deserializer=nidaqmx__pb2.CfgDigEdgeRefTrigRequest.FromString,
                    response_serializer=nidaqmx__pb2.CfgDigEdgeRefTrigResponse.SerializeToString,
            ),
            'CfgDigEdgeStartTrig': grpc.unary_unary_rpc_method_handler(
                    servicer.CfgDigEdgeStartTrig,
                    request_deserializer=nidaqmx__pb2.CfgDigEdgeStartTrigRequest.FromString,
                    response_serializer=nidaqmx__pb2.CfgDigEdgeStartTrigResponse.SerializeToString,
            ),
            'CfgDigPatternRefTrig': grpc.unary_unary_rpc_method_handler(
                    servicer.CfgDigPatternRefTrig,
                    request_deserializer=nidaqmx__pb2.CfgDigPatternRefTrigRequest.FromString,
                    response_serializer=nidaqmx__pb2.CfgDigPatternRefTrigResponse.SerializeToString,
            ),
            'CfgDigPatternStartTrig': grpc.unary_unary_rpc_method_handler(
                    servicer.CfgDigPatternStartTrig,
                    request_deserializer=nidaqmx__pb2.CfgDigPatternStartTrigRequest.FromString,
                    response_serializer=nidaqmx__pb2.CfgDigPatternStartTrigResponse.SerializeToString,
            ),
            'CfgHandshakingTiming': grpc.unary_unary_rpc_method_handler(
                    servicer.CfgHandshakingTiming,
                    request_deserializer=nidaqmx__pb2.CfgHandshakingTimingRequest.FromString,
                    response_serializer=nidaqmx__pb2.CfgHandshakingTimingResponse.SerializeToString,
            ),
            'CfgImplicitTiming': grpc.unary_unary_rpc_method_handler(
                    servicer.CfgImplicitTiming,
                    request_deserializer=nidaqmx__pb2.CfgImplicitTimingRequest.FromString,
                    response_serializer=nidaqmx__pb2.CfgImplicitTimingResponse.SerializeToString,
            ),
            'CfgInputBuffer': grpc.unary_unary_rpc_method_handler(
                    servicer.CfgInputBuffer,
                    request_deserializer=nidaqmx__pb2.CfgInputBufferRequest.FromString,
                    response_serializer=nidaqmx__pb2.CfgInputBufferResponse.SerializeToString,
            ),
            'CfgOutputBuffer': grpc.unary_unary_rpc_method_handler(
                    servicer.CfgOutputBuffer,
                    request_deserializer=nidaqmx__pb2.CfgOutputBufferRequest.FromString,
                    response_serializer=nidaqmx__pb2.CfgOutputBufferResponse.SerializeToString,
            ),
            'CfgPipelinedSampClkTiming': grpc.unary_unary_rpc_method_handler(
                    servicer.CfgPipelinedSampClkTiming,
                    request_deserializer=nidaqmx__pb2.CfgPipelinedSampClkTimingRequest.FromString,
                    response_serializer=nidaqmx__pb2.CfgPipelinedSampClkTimingResponse.SerializeToString,
            ),
            'CfgSampClkTiming': grpc.unary_unary_rpc_method_handler(
                    servicer.CfgSampClkTiming,
                    request_deserializer=nidaqmx__pb2.CfgSampClkTimingRequest.FromString,
                    response_serializer=nidaqmx__pb2.CfgSampClkTimingResponse.SerializeToString,
            ),
            'CfgTimeStartTrig': grpc.unary_unary_rpc_method_handler(
                    servicer.CfgTimeStartTrig,
                    request_deserializer=nidaqmx__pb2.CfgTimeStartTrigRequest.FromString,
                    response_serializer=nidaqmx__pb2.CfgTimeStartTrigResponse.SerializeToString,
            ),
            'CfgWatchdogAOExpirStates': grpc.unary_unary_rpc_method_handler(
                    servicer.CfgWatchdogAOExpirStates,
                    request_deserializer=nidaqmx__pb2.CfgWatchdogAOExpirStatesRequest.FromString,
                    response_serializer=nidaqmx__pb2.CfgWatchdogAOExpirStatesResponse.SerializeToString,
            ),
            'CfgWatchdogCOExpirStates': grpc.unary_unary_rpc_method_handler(
                    servicer.CfgWatchdogCOExpirStates,
                    request_deserializer=nidaqmx__pb2.CfgWatchdogCOExpirStatesRequest.FromString,
                    response_serializer=nidaqmx__pb2.CfgWatchdogCOExpirStatesResponse.SerializeToString,
            ),
            'CfgWatchdogDOExpirStates': grpc.unary_unary_rpc_method_handler(
                    servicer.CfgWatchdogDOExpirStates,
                    request_deserializer=nidaqmx__pb2.CfgWatchdogDOExpirStatesRequest.FromString,
                    response_serializer=nidaqmx__pb2.CfgWatchdogDOExpirStatesResponse.SerializeToString,
            ),
            'ClearTEDS': grpc.unary_unary_rpc_method_handler(
                    servicer.ClearTEDS,
                    request_deserializer=nidaqmx__pb2.ClearTEDSRequest.FromString,
                    response_serializer=nidaqmx__pb2.ClearTEDSResponse.SerializeToString,
            ),
            'ClearTask': grpc.unary_unary_rpc_method_handler(
                    servicer.ClearTask,
                    request_deserializer=nidaqmx__pb2.ClearTaskRequest.FromString,
                    response_serializer=nidaqmx__pb2.ClearTaskResponse.SerializeToString,
            ),
            'ConfigureLogging': grpc.unary_unary_rpc_method_handler(
                    servicer.ConfigureLogging,
                    request_deserializer=nidaqmx__pb2.ConfigureLoggingRequest.FromString,
                    response_serializer=nidaqmx__pb2.ConfigureLoggingResponse.SerializeToString,
            ),
            'ConfigureTEDS': grpc.unary_unary_rpc_method_handler(
                    servicer.ConfigureTEDS,
                    request_deserializer=nidaqmx__pb2.ConfigureTEDSRequest.FromString,
                    response_serializer=nidaqmx__pb2.ConfigureTEDSResponse.SerializeToString,
            ),
            'ConnectTerms': grpc.unary_unary_rpc_method_handler(
                    servicer.ConnectTerms,
                    request_deserializer=nidaqmx__pb2.ConnectTermsRequest.FromString,
                    response_serializer=nidaqmx__pb2.ConnectTermsResponse.SerializeToString,
            ),
            'ControlWatchdogTask': grpc.unary_unary_rpc_method_handler(
                    servicer.ControlWatchdogTask,
                    request_deserializer=nidaqmx__pb2.ControlWatchdogTaskRequest.FromString,
                    response_serializer=nidaqmx__pb2.ControlWatchdogTaskResponse.SerializeToString,
            ),
            'CreateAIAccel4WireDCVoltageChan': grpc.unary_unary_rpc_method_handler(
                    servicer.CreateAIAccel4WireDCVoltageChan,
                    request_deserializer=nidaqmx__pb2.CreateAIAccel4WireDCVoltageChanRequest.FromString,
                    response_serializer=nidaqmx__pb2.CreateAIAccel4WireDCVoltageChanResponse.SerializeToString,
            ),
            'CreateAIAccelChan': grpc.unary_unary_rpc_method_handler(
                    servicer.CreateAIAccelChan,
                    request_deserializer=nidaqmx__pb2.CreateAIAccelChanRequest.FromString,
                    response_serializer=nidaqmx__pb2.CreateAIAccelChanResponse.SerializeToString,
            ),
            'CreateAIAccelChargeChan': grpc.unary_unary_rpc_method_handler(
                    servicer.CreateAIAccelChargeChan,
                    request_deserializer=nidaqmx__pb2.CreateAIAccelChargeChanRequest.FromString,
                    response_serializer=nidaqmx__pb2.CreateAIAccelChargeChanResponse.SerializeToString,
            ),
            'CreateAIBridgeChan': grpc.unary_unary_rpc_method_handler(
                    servicer.CreateAIBridgeChan,
                    request_deserializer=nidaqmx__pb2.CreateAIBridgeChanRequest.FromString,
                    response_serializer=nidaqmx__pb2.CreateAIBridgeChanResponse.SerializeToString,
            ),
            'CreateAIChargeChan': grpc.unary_unary_rpc_method_handler(
                    servicer.CreateAIChargeChan,
                    request_deserializer=nidaqmx__pb2.CreateAIChargeChanRequest.FromString,
                    response_serializer=nidaqmx__pb2.CreateAIChargeChanResponse.SerializeToString,
            ),
            'CreateAICurrentChan': grpc.unary_unary_rpc_method_handler(
                    servicer.CreateAICurrentChan,
                    request_deserializer=nidaqmx__pb2.CreateAICurrentChanRequest.FromString,
                    response_serializer=nidaqmx__pb2.CreateAICurrentChanResponse.SerializeToString,
            ),
            'CreateAICurrentRMSChan': grpc.unary_unary_rpc_method_handler(
                    servicer.CreateAICurrentRMSChan,
                    request_deserializer=nidaqmx__pb2.CreateAICurrentRMSChanRequest.FromString,
                    response_serializer=nidaqmx__pb2.CreateAICurrentRMSChanResponse.SerializeToString,
            ),
            'CreateAIForceBridgePolynomialChan': grpc.unary_unary_rpc_method_handler(
                    servicer.CreateAIForceBridgePolynomialChan,
                    request_deserializer=nidaqmx__pb2.CreateAIForceBridgePolynomialChanRequest.FromString,
                    response_serializer=nidaqmx__pb2.CreateAIForceBridgePolynomialChanResponse.SerializeToString,
            ),
            'CreateAIForceBridgeTableChan': grpc.unary_unary_rpc_method_handler(
                    servicer.CreateAIForceBridgeTableChan,
                    request_deserializer=nidaqmx__pb2.CreateAIForceBridgeTableChanRequest.FromString,
                    response_serializer=nidaqmx__pb2.CreateAIForceBridgeTableChanResponse.SerializeToString,
            ),
            'CreateAIForceBridgeTwoPointLinChan': grpc.unary_unary_rpc_method_handler(
                    servicer.CreateAIForceBridgeTwoPointLinChan,
                    request_deserializer=nidaqmx__pb2.CreateAIForceBridgeTwoPointLinChanRequest.FromString,
                    response_serializer=nidaqmx__pb2.CreateAIForceBridgeTwoPointLinChanResponse.SerializeToString,
            ),
            'CreateAIForceIEPEChan': grpc.unary_unary_rpc_method_handler(
                    servicer.CreateAIForceIEPEChan,
                    request_deserializer=nidaqmx__pb2.CreateAIForceIEPEChanRequest.FromString,
                    response_serializer=nidaqmx__pb2.CreateAIForceIEPEChanResponse.SerializeToString,
            ),
            'CreateAIFreqVoltageChan': grpc.unary_unary_rpc_method_handler(
                    servicer.CreateAIFreqVoltageChan,
                    request_deserializer=nidaqmx__pb2.CreateAIFreqVoltageChanRequest.FromString,
                    response_serializer=nidaqmx__pb2.CreateAIFreqVoltageChanResponse.SerializeToString,
            ),
            'CreateAIMicrophoneChan': grpc.unary_unary_rpc_method_handler(
                    servicer.CreateAIMicrophoneChan,
                    request_deserializer=nidaqmx__pb2.CreateAIMicrophoneChanRequest.FromString,
                    response_serializer=nidaqmx__pb2.CreateAIMicrophoneChanResponse.SerializeToString,
            ),
            'CreateAIPosEddyCurrProxProbeChan': grpc.unary_unary_rpc_method_handler(
                    servicer.CreateAIPosEddyCurrProxProbeChan,
                    request_deserializer=nidaqmx__pb2.CreateAIPosEddyCurrProxProbeChanRequest.FromString,
                    response_serializer=nidaqmx__pb2.CreateAIPosEddyCurrProxProbeChanResponse.SerializeToString,
            ),
            'CreateAIPosLVDTChan': grpc.unary_unary_rpc_method_handler(
                    servicer.CreateAIPosLVDTChan,
                    request_deserializer=nidaqmx__pb2.CreateAIPosLVDTChanRequest.FromString,
                    response_serializer=nidaqmx__pb2.CreateAIPosLVDTChanResponse.SerializeToString,
            ),
            'CreateAIPosRVDTChan': grpc.unary_unary_rpc_method_handler(
                    servicer.CreateAIPosRVDTChan,
                    request_deserializer=nidaqmx__pb2.CreateAIPosRVDTChanRequest.FromString,
                    response_serializer=nidaqmx__pb2.CreateAIPosRVDTChanResponse.SerializeToString,
            ),
            'CreateAIPowerChan': grpc.unary_unary_rpc_method_handler(
                    servicer.CreateAIPowerChan,
                    request_deserializer=nidaqmx__pb2.CreateAIPowerChanRequest.FromString,
                    response_serializer=nidaqmx__pb2.CreateAIPowerChanResponse.SerializeToString,
            ),
            'CreateAIPressureBridgePolynomialChan': grpc.unary_unary_rpc_method_handler(
                    servicer.CreateAIPressureBridgePolynomialChan,
                    request_deserializer=nidaqmx__pb2.CreateAIPressureBridgePolynomialChanRequest.FromString,
                    response_serializer=nidaqmx__pb2.CreateAIPressureBridgePolynomialChanResponse.SerializeToString,
            ),
            'CreateAIPressureBridgeTableChan': grpc.unary_unary_rpc_method_handler(
                    servicer.CreateAIPressureBridgeTableChan,
                    request_deserializer=nidaqmx__pb2.CreateAIPressureBridgeTableChanRequest.FromString,
                    response_serializer=nidaqmx__pb2.CreateAIPressureBridgeTableChanResponse.SerializeToString,
            ),
            'CreateAIPressureBridgeTwoPointLinChan': grpc.unary_unary_rpc_method_handler(
                    servicer.CreateAIPressureBridgeTwoPointLinChan,
                    request_deserializer=nidaqmx__pb2.CreateAIPressureBridgeTwoPointLinChanRequest.FromString,
                    response_serializer=nidaqmx__pb2.CreateAIPressureBridgeTwoPointLinChanResponse.SerializeToString,
            ),
            'CreateAIRTDChan': grpc.unary_unary_rpc_method_handler(
                    servicer.CreateAIRTDChan,
                    request_deserializer=nidaqmx__pb2.CreateAIRTDChanRequest.FromString,
                    response_serializer=nidaqmx__pb2.CreateAIRTDChanResponse.SerializeToString,
            ),
            'CreateAIResistanceChan': grpc.unary_unary_rpc_method_handler(
                    servicer.CreateAIResistanceChan,
                    request_deserializer=nidaqmx__pb2.CreateAIResistanceChanRequest.FromString,
                    response_serializer=nidaqmx__pb2.CreateAIResistanceChanResponse.SerializeToString,
            ),
            'CreateAIRosetteStrainGageChan': grpc.unary_unary_rpc_method_handler(
                    servicer.CreateAIRosetteStrainGageChan,
                    request_deserializer=nidaqmx__pb2.CreateAIRosetteStrainGageChanRequest.FromString,
                    response_serializer=nidaqmx__pb2.CreateAIRosetteStrainGageChanResponse.SerializeToString,
            ),
            'CreateAIStrainGageChan': grpc.unary_unary_rpc_method_handler(
                    servicer.CreateAIStrainGageChan,
                    request_deserializer=nidaqmx__pb2.CreateAIStrainGageChanRequest.FromString,
                    response_serializer=nidaqmx__pb2.CreateAIStrainGageChanResponse.SerializeToString,
            ),
            'CreateAITempBuiltInSensorChan': grpc.unary_unary_rpc_method_handler(
                    servicer.CreateAITempBuiltInSensorChan,
                    request_deserializer=nidaqmx__pb2.CreateAITempBuiltInSensorChanRequest.FromString,
                    response_serializer=nidaqmx__pb2.CreateAITempBuiltInSensorChanResponse.SerializeToString,
            ),
            'CreateAIThrmcplChan': grpc.unary_unary_rpc_method_handler(
                    servicer.CreateAIThrmcplChan,
                    request_deserializer=nidaqmx__pb2.CreateAIThrmcplChanRequest.FromString,
                    response_serializer=nidaqmx__pb2.CreateAIThrmcplChanResponse.SerializeToString,
            ),
            'CreateAIThrmstrChanIex': grpc.unary_unary_rpc_method_handler(
                    servicer.CreateAIThrmstrChanIex,
                    request_deserializer=nidaqmx__pb2.CreateAIThrmstrChanIexRequest.FromString,
                    response_serializer=nidaqmx__pb2.CreateAIThrmstrChanIexResponse.SerializeToString,
            ),
            'CreateAIThrmstrChanVex': grpc.unary_unary_rpc_method_handler(
                    servicer.CreateAIThrmstrChanVex,
                    request_deserializer=nidaqmx__pb2.CreateAIThrmstrChanVexRequest.FromString,
                    response_serializer=nidaqmx__pb2.CreateAIThrmstrChanVexResponse.SerializeToString,
            ),
            'CreateAITorqueBridgePolynomialChan': grpc.unary_unary_rpc_method_handler(
                    servicer.CreateAITorqueBridgePolynomialChan,
                    request_deserializer=nidaqmx__pb2.CreateAITorqueBridgePolynomialChanRequest.FromString,
                    response_serializer=nidaqmx__pb2.CreateAITorqueBridgePolynomialChanResponse.SerializeToString,
            ),
            'CreateAITorqueBridgeTableChan': grpc.unary_unary_rpc_method_handler(
                    servicer.CreateAITorqueBridgeTableChan,
                    request_deserializer=nidaqmx__pb2.CreateAITorqueBridgeTableChanRequest.FromString,
                    response_serializer=nidaqmx__pb2.CreateAITorqueBridgeTableChanResponse.SerializeToString,
            ),
            'CreateAITorqueBridgeTwoPointLinChan': grpc.unary_unary_rpc_method_handler(
                    servicer.CreateAITorqueBridgeTwoPointLinChan,
                    request_deserializer=nidaqmx__pb2.CreateAITorqueBridgeTwoPointLinChanRequest.FromString,
                    response_serializer=nidaqmx__pb2.CreateAITorqueBridgeTwoPointLinChanResponse.SerializeToString,
            ),
            'CreateAIVelocityIEPEChan': grpc.unary_unary_rpc_method_handler(
                    servicer.CreateAIVelocityIEPEChan,
                    request_deserializer=nidaqmx__pb2.CreateAIVelocityIEPEChanRequest.FromString,
                    response_serializer=nidaqmx__pb2.CreateAIVelocityIEPEChanResponse.SerializeToString,
            ),
            'CreateAIVoltageChan': grpc.unary_unary_rpc_method_handler(
                    servicer.CreateAIVoltageChan,
                    request_deserializer=nidaqmx__pb2.CreateAIVoltageChanRequest.FromString,
                    response_serializer=nidaqmx__pb2.CreateAIVoltageChanResponse.SerializeToString,
            ),
            'CreateAIVoltageChanWithExcit': grpc.unary_unary_rpc_method_handler(
                    servicer.CreateAIVoltageChanWithExcit,
                    request_deserializer=nidaqmx__pb2.CreateAIVoltageChanWithExcitRequest.FromString,
                    response_serializer=nidaqmx__pb2.CreateAIVoltageChanWithExcitResponse.SerializeToString,
            ),
            'CreateAIVoltageRMSChan': grpc.unary_unary_rpc_method_handler(
                    servicer.CreateAIVoltageRMSChan,
                    request_deserializer=nidaqmx__pb2.CreateAIVoltageRMSChanRequest.FromString,
                    response_serializer=nidaqmx__pb2.CreateAIVoltageRMSChanResponse.SerializeToString,
            ),
            'CreateAOCurrentChan': grpc.unary_unary_rpc_method_handler(
                    servicer.CreateAOCurrentChan,
                    request_deserializer=nidaqmx__pb2.CreateAOCurrentChanRequest.FromString,
                    response_serializer=nidaqmx__pb2.CreateAOCurrentChanResponse.SerializeToString,
            ),
            'CreateAOFuncGenChan': grpc.unary_unary_rpc_method_handler(
                    servicer.CreateAOFuncGenChan,
                    request_deserializer=nidaqmx__pb2.CreateAOFuncGenChanRequest.FromString,
                    response_serializer=nidaqmx__pb2.CreateAOFuncGenChanResponse.SerializeToString,
            ),
            'CreateAOVoltageChan': grpc.unary_unary_rpc_method_handler(
                    servicer.CreateAOVoltageChan,
                    request_deserializer=nidaqmx__pb2.CreateAOVoltageChanRequest.FromString,
                    response_serializer=nidaqmx__pb2.CreateAOVoltageChanResponse.SerializeToString,
            ),
            'CreateCIAngEncoderChan': grpc.unary_unary_rpc_method_handler(
                    servicer.CreateCIAngEncoderChan,
                    request_deserializer=nidaqmx__pb2.CreateCIAngEncoderChanRequest.FromString,
                    response_serializer=nidaqmx__pb2.CreateCIAngEncoderChanResponse.SerializeToString,
            ),
            'CreateCIAngVelocityChan': grpc.unary_unary_rpc_method_handler(
                    servicer.CreateCIAngVelocityChan,
                    request_deserializer=nidaqmx__pb2.CreateCIAngVelocityChanRequest.FromString,
                    response_serializer=nidaqmx__pb2.CreateCIAngVelocityChanResponse.SerializeToString,
            ),
            'CreateCICountEdgesChan': grpc.unary_unary_rpc_method_handler(
                    servicer.CreateCICountEdgesChan,
                    request_deserializer=nidaqmx__pb2.CreateCICountEdgesChanRequest.FromString,
                    response_serializer=nidaqmx__pb2.CreateCICountEdgesChanResponse.SerializeToString,
            ),
            'CreateCIDutyCycleChan': grpc.unary_unary_rpc_method_handler(
                    servicer.CreateCIDutyCycleChan,
                    request_deserializer=nidaqmx__pb2.CreateCIDutyCycleChanRequest.FromString,
                    response_serializer=nidaqmx__pb2.CreateCIDutyCycleChanResponse.SerializeToString,
            ),
            'CreateCIFreqChan': grpc.unary_unary_rpc_method_handler(
                    servicer.CreateCIFreqChan,
                    request_deserializer=nidaqmx__pb2.CreateCIFreqChanRequest.FromString,
                    response_serializer=nidaqmx__pb2.CreateCIFreqChanResponse.SerializeToString,
            ),
            'CreateCIGPSTimestampChan': grpc.unary_unary_rpc_method_handler(
                    servicer.CreateCIGPSTimestampChan,
                    request_deserializer=nidaqmx__pb2.CreateCIGPSTimestampChanRequest.FromString,
                    response_serializer=nidaqmx__pb2.CreateCIGPSTimestampChanResponse.SerializeToString,
            ),
            'CreateCILinEncoderChan': grpc.unary_unary_rpc_method_handler(
                    servicer.CreateCILinEncoderChan,
                    request_deserializer=nidaqmx__pb2.CreateCILinEncoderChanRequest.FromString,
                    response_serializer=nidaqmx__pb2.CreateCILinEncoderChanResponse.SerializeToString,
            ),
            'CreateCILinVelocityChan': grpc.unary_unary_rpc_method_handler(
                    servicer.CreateCILinVelocityChan,
                    request_deserializer=nidaqmx__pb2.CreateCILinVelocityChanRequest.FromString,
                    response_serializer=nidaqmx__pb2.CreateCILinVelocityChanResponse.SerializeToString,
            ),
            'CreateCIPeriodChan': grpc.unary_unary_rpc_method_handler(
                    servicer.CreateCIPeriodChan,
                    request_deserializer=nidaqmx__pb2.CreateCIPeriodChanRequest.FromString,
                    response_serializer=nidaqmx__pb2.CreateCIPeriodChanResponse.SerializeToString,
            ),
            'CreateCIPulseChanFreq': grpc.unary_unary_rpc_method_handler(
                    servicer.CreateCIPulseChanFreq,
                    request_deserializer=nidaqmx__pb2.CreateCIPulseChanFreqRequest.FromString,
                    response_serializer=nidaqmx__pb2.CreateCIPulseChanFreqResponse.SerializeToString,
            ),
            'CreateCIPulseChanTicks': grpc.unary_unary_rpc_method_handler(
                    servicer.CreateCIPulseChanTicks,
                    request_deserializer=nidaqmx__pb2.CreateCIPulseChanTicksRequest.FromString,
                    response_serializer=nidaqmx__pb2.CreateCIPulseChanTicksResponse.SerializeToString,
            ),
            'CreateCIPulseChanTime': grpc.unary_unary_rpc_method_handler(
                    servicer.CreateCIPulseChanTime,
                    request_deserializer=nidaqmx__pb2.CreateCIPulseChanTimeRequest.FromString,
                    response_serializer=nidaqmx__pb2.CreateCIPulseChanTimeResponse.SerializeToString,
            ),
            'CreateCIPulseWidthChan': grpc.unary_unary_rpc_method_handler(
                    servicer.CreateCIPulseWidthChan,
                    request_deserializer=nidaqmx__pb2.CreateCIPulseWidthChanRequest.FromString,
                    response_serializer=nidaqmx__pb2.CreateCIPulseWidthChanResponse.SerializeToString,
            ),
            'CreateCISemiPeriodChan': grpc.unary_unary_rpc_method_handler(
                    servicer.CreateCISemiPeriodChan,
                    request_deserializer=nidaqmx__pb2.CreateCISemiPeriodChanRequest.FromString,
                    response_serializer=nidaqmx__pb2.CreateCISemiPeriodChanResponse.SerializeToString,
            ),
            'CreateCITwoEdgeSepChan': grpc.unary_unary_rpc_method_handler(
                    servicer.CreateCITwoEdgeSepChan,
                    request_deserializer=nidaqmx__pb2.CreateCITwoEdgeSepChanRequest.FromString,
                    response_serializer=nidaqmx__pb2.CreateCITwoEdgeSepChanResponse.SerializeToString,
            ),
            'CreateCOPulseChanFreq': grpc.unary_unary_rpc_method_handler(
                    servicer.CreateCOPulseChanFreq,
                    request_deserializer=nidaqmx__pb2.CreateCOPulseChanFreqRequest.FromString,
                    response_serializer=nidaqmx__pb2.CreateCOPulseChanFreqResponse.SerializeToString,
            ),
            'CreateCOPulseChanTicks': grpc.unary_unary_rpc_method_handler(
                    servicer.CreateCOPulseChanTicks,
                    request_deserializer=nidaqmx__pb2.CreateCOPulseChanTicksRequest.FromString,
                    response_serializer=nidaqmx__pb2.CreateCOPulseChanTicksResponse.SerializeToString,
            ),
            'CreateCOPulseChanTime': grpc.unary_unary_rpc_method_handler(
                    servicer.CreateCOPulseChanTime,
                    request_deserializer=nidaqmx__pb2.CreateCOPulseChanTimeRequest.FromString,
                    response_serializer=nidaqmx__pb2.CreateCOPulseChanTimeResponse.SerializeToString,
            ),
            'CreateDIChan': grpc.unary_unary_rpc_method_handler(
                    servicer.CreateDIChan,
                    request_deserializer=nidaqmx__pb2.CreateDIChanRequest.FromString,
                    response_serializer=nidaqmx__pb2.CreateDIChanResponse.SerializeToString,
            ),
            'CreateDOChan': grpc.unary_unary_rpc_method_handler(
                    servicer.CreateDOChan,
                    request_deserializer=nidaqmx__pb2.CreateDOChanRequest.FromString,
                    response_serializer=nidaqmx__pb2.CreateDOChanResponse.SerializeToString,
            ),
            'CreateLinScale': grpc.unary_unary_rpc_method_handler(
                    servicer.CreateLinScale,
                    request_deserializer=nidaqmx__pb2.CreateLinScaleRequest.FromString,
                    response_serializer=nidaqmx__pb2.CreateLinScaleResponse.SerializeToString,
            ),
            'CreateMapScale': grpc.unary_unary_rpc_method_handler(
                    servicer.CreateMapScale,
                    request_deserializer=nidaqmx__pb2.CreateMapScaleRequest.FromString,
                    response_serializer=nidaqmx__pb2.CreateMapScaleResponse.SerializeToString,
            ),
            'CreatePolynomialScale': grpc.unary_unary_rpc_method_handler(
                    servicer.CreatePolynomialScale,
                    request_deserializer=nidaqmx__pb2.CreatePolynomialScaleRequest.FromString,
                    response_serializer=nidaqmx__pb2.CreatePolynomialScaleResponse.SerializeToString,
            ),
            'CreateTEDSAIAccelChan': grpc.unary_unary_rpc_method_handler(
                    servicer.CreateTEDSAIAccelChan,
                    request_deserializer=nidaqmx__pb2.CreateTEDSAIAccelChanRequest.FromString,
                    response_serializer=nidaqmx__pb2.CreateTEDSAIAccelChanResponse.SerializeToString,
            ),
            'CreateTEDSAIBridgeChan': grpc.unary_unary_rpc_method_handler(
                    servicer.CreateTEDSAIBridgeChan,
                    request_deserializer=nidaqmx__pb2.CreateTEDSAIBridgeChanRequest.FromString,
                    response_serializer=nidaqmx__pb2.CreateTEDSAIBridgeChanResponse.SerializeToString,
            ),
            'CreateTEDSAICurrentChan': grpc.unary_unary_rpc_method_handler(
                    servicer.CreateTEDSAICurrentChan,
                    request_deserializer=nidaqmx__pb2.CreateTEDSAICurrentChanRequest.FromString,
                    response_serializer=nidaqmx__pb2.CreateTEDSAICurrentChanResponse.SerializeToString,
            ),
            'CreateTEDSAIForceBridgeChan': grpc.unary_unary_rpc_method_handler(
                    servicer.CreateTEDSAIForceBridgeChan,
                    request_deserializer=nidaqmx__pb2.CreateTEDSAIForceBridgeChanRequest.FromString,
                    response_serializer=nidaqmx__pb2.CreateTEDSAIForceBridgeChanResponse.SerializeToString,
            ),
            'CreateTEDSAIForceIEPEChan': grpc.unary_unary_rpc_method_handler(
                    servicer.CreateTEDSAIForceIEPEChan,
                    request_deserializer=nidaqmx__pb2.CreateTEDSAIForceIEPEChanRequest.FromString,
                    response_serializer=nidaqmx__pb2.CreateTEDSAIForceIEPEChanResponse.SerializeToString,
            ),
            'CreateTEDSAIMicrophoneChan': grpc.unary_unary_rpc_method_handler(
                    servicer.CreateTEDSAIMicrophoneChan,
                    request_deserializer=nidaqmx__pb2.CreateTEDSAIMicrophoneChanRequest.FromString,
                    response_serializer=nidaqmx__pb2.CreateTEDSAIMicrophoneChanResponse.SerializeToString,
            ),
            'CreateTEDSAIPosLVDTChan': grpc.unary_unary_rpc_method_handler(
                    servicer.CreateTEDSAIPosLVDTChan,
                    request_deserializer=nidaqmx__pb2.CreateTEDSAIPosLVDTChanRequest.FromString,
                    response_serializer=nidaqmx__pb2.CreateTEDSAIPosLVDTChanResponse.SerializeToString,
            ),
            'CreateTEDSAIPosRVDTChan': grpc.unary_unary_rpc_method_handler(
                    servicer.CreateTEDSAIPosRVDTChan,
                    request_deserializer=nidaqmx__pb2.CreateTEDSAIPosRVDTChanRequest.FromString,
                    response_serializer=nidaqmx__pb2.CreateTEDSAIPosRVDTChanResponse.SerializeToString,
            ),
            'CreateTEDSAIPressureBridgeChan': grpc.unary_unary_rpc_method_handler(
                    servicer.CreateTEDSAIPressureBridgeChan,
                    request_deserializer=nidaqmx__pb2.CreateTEDSAIPressureBridgeChanRequest.FromString,
                    response_serializer=nidaqmx__pb2.CreateTEDSAIPressureBridgeChanResponse.SerializeToString,
            ),
            'CreateTEDSAIRTDChan': grpc.unary_unary_rpc_method_handler(
                    servicer.CreateTEDSAIRTDChan,
                    request_deserializer=nidaqmx__pb2.CreateTEDSAIRTDChanRequest.FromString,
                    response_serializer=nidaqmx__pb2.CreateTEDSAIRTDChanResponse.SerializeToString,
            ),
            'CreateTEDSAIResistanceChan': grpc.unary_unary_rpc_method_handler(
                    servicer.CreateTEDSAIResistanceChan,
                    request_deserializer=nidaqmx__pb2.CreateTEDSAIResistanceChanRequest.FromString,
                    response_serializer=nidaqmx__pb2.CreateTEDSAIResistanceChanResponse.SerializeToString,
            ),
            'CreateTEDSAIStrainGageChan': grpc.unary_unary_rpc_method_handler(
                    servicer.CreateTEDSAIStrainGageChan,
                    request_deserializer=nidaqmx__pb2.CreateTEDSAIStrainGageChanRequest.FromString,
                    response_serializer=nidaqmx__pb2.CreateTEDSAIStrainGageChanResponse.SerializeToString,
            ),
            'CreateTEDSAIThrmcplChan': grpc.unary_unary_rpc_method_handler(
                    servicer.CreateTEDSAIThrmcplChan,
                    request_deserializer=nidaqmx__pb2.CreateTEDSAIThrmcplChanRequest.FromString,
                    response_serializer=nidaqmx__pb2.CreateTEDSAIThrmcplChanResponse.SerializeToString,
            ),
            'CreateTEDSAIThrmstrChanIex': grpc.unary_unary_rpc_method_handler(
                    servicer.CreateTEDSAIThrmstrChanIex,
                    request_deserializer=nidaqmx__pb2.CreateTEDSAIThrmstrChanIexRequest.FromString,
                    response_serializer=nidaqmx__pb2.CreateTEDSAIThrmstrChanIexResponse.SerializeToString,
            ),
            'CreateTEDSAIThrmstrChanVex': grpc.unary_unary_rpc_method_handler(
                    servicer.CreateTEDSAIThrmstrChanVex,
                    request_deserializer=nidaqmx__pb2.CreateTEDSAIThrmstrChanVexRequest.FromString,
                    response_serializer=nidaqmx__pb2.CreateTEDSAIThrmstrChanVexResponse.SerializeToString,
            ),
            'CreateTEDSAITorqueBridgeChan': grpc.unary_unary_rpc_method_handler(
                    servicer.CreateTEDSAITorqueBridgeChan,
                    request_deserializer=nidaqmx__pb2.CreateTEDSAITorqueBridgeChanRequest.FromString,
                    response_serializer=nidaqmx__pb2.CreateTEDSAITorqueBridgeChanResponse.SerializeToString,
            ),
            'CreateTEDSAIVoltageChan': grpc.unary_unary_rpc_method_handler(
                    servicer.CreateTEDSAIVoltageChan,
                    request_deserializer=nidaqmx__pb2.CreateTEDSAIVoltageChanRequest.FromString,
                    response_serializer=nidaqmx__pb2.CreateTEDSAIVoltageChanResponse.SerializeToString,
            ),
            'CreateTEDSAIVoltageChanWithExcit': grpc.unary_unary_rpc_method_handler(
                    servicer.CreateTEDSAIVoltageChanWithExcit,
                    request_deserializer=nidaqmx__pb2.CreateTEDSAIVoltageChanWithExcitRequest.FromString,
                    response_serializer=nidaqmx__pb2.CreateTEDSAIVoltageChanWithExcitResponse.SerializeToString,
            ),
            'CreateTableScale': grpc.unary_unary_rpc_method_handler(
                    servicer.CreateTableScale,
                    request_deserializer=nidaqmx__pb2.CreateTableScaleRequest.FromString,
                    response_serializer=nidaqmx__pb2.CreateTableScaleResponse.SerializeToString,
            ),
            'CreateTask': grpc.unary_unary_rpc_method_handler(
                    servicer.CreateTask,
                    request_deserializer=nidaqmx__pb2.CreateTaskRequest.FromString,
                    response_serializer=nidaqmx__pb2.CreateTaskResponse.SerializeToString,
            ),
            'CreateWatchdogTimerTask': grpc.unary_unary_rpc_method_handler(
                    servicer.CreateWatchdogTimerTask,
                    request_deserializer=nidaqmx__pb2.CreateWatchdogTimerTaskRequest.FromString,
                    response_serializer=nidaqmx__pb2.CreateWatchdogTimerTaskResponse.SerializeToString,
            ),
            'CreateWatchdogTimerTaskEx': grpc.unary_unary_rpc_method_handler(
                    servicer.CreateWatchdogTimerTaskEx,
                    request_deserializer=nidaqmx__pb2.CreateWatchdogTimerTaskExRequest.FromString,
                    response_serializer=nidaqmx__pb2.CreateWatchdogTimerTaskExResponse.SerializeToString,
            ),
            'DeleteNetworkDevice': grpc.unary_unary_rpc_method_handler(
                    servicer.DeleteNetworkDevice,
                    request_deserializer=nidaqmx__pb2.DeleteNetworkDeviceRequest.FromString,
                    response_serializer=nidaqmx__pb2.DeleteNetworkDeviceResponse.SerializeToString,
            ),
            'DeleteSavedGlobalChan': grpc.unary_unary_rpc_method_handler(
                    servicer.DeleteSavedGlobalChan,
                    request_deserializer=nidaqmx__pb2.DeleteSavedGlobalChanRequest.FromString,
                    response_serializer=nidaqmx__pb2.DeleteSavedGlobalChanResponse.SerializeToString,
            ),
            'DeleteSavedScale': grpc.unary_unary_rpc_method_handler(
                    servicer.DeleteSavedScale,
                    request_deserializer=nidaqmx__pb2.DeleteSavedScaleRequest.FromString,
                    response_serializer=nidaqmx__pb2.DeleteSavedScaleResponse.SerializeToString,
            ),
            'DeleteSavedTask': grpc.unary_unary_rpc_method_handler(
                    servicer.DeleteSavedTask,
                    request_deserializer=nidaqmx__pb2.DeleteSavedTaskRequest.FromString,
                    response_serializer=nidaqmx__pb2.DeleteSavedTaskResponse.SerializeToString,
            ),
            'DeviceSupportsCal': grpc.unary_unary_rpc_method_handler(
                    servicer.DeviceSupportsCal,
                    request_deserializer=nidaqmx__pb2.DeviceSupportsCalRequest.FromString,
                    response_serializer=nidaqmx__pb2.DeviceSupportsCalResponse.SerializeToString,
            ),
            'DisableRefTrig': grpc.unary_unary_rpc_method_handler(
                    servicer.DisableRefTrig,
                    request_deserializer=nidaqmx__pb2.DisableRefTrigRequest.FromString,
                    response_serializer=nidaqmx__pb2.DisableRefTrigResponse.SerializeToString,
            ),
            'DisableStartTrig': grpc.unary_unary_rpc_method_handler(
                    servicer.DisableStartTrig,
                    request_deserializer=nidaqmx__pb2.DisableStartTrigRequest.FromString,
                    response_serializer=nidaqmx__pb2.DisableStartTrigResponse.SerializeToString,
            ),
            'DisconnectTerms': grpc.unary_unary_rpc_method_handler(
                    servicer.DisconnectTerms,
                    request_deserializer=nidaqmx__pb2.DisconnectTermsRequest.FromString,
                    response_serializer=nidaqmx__pb2.DisconnectTermsResponse.SerializeToString,
            ),
            'ExportSignal': grpc.unary_unary_rpc_method_handler(
                    servicer.ExportSignal,
                    request_deserializer=nidaqmx__pb2.ExportSignalRequest.FromString,
                    response_serializer=nidaqmx__pb2.ExportSignalResponse.SerializeToString,
            ),
            'GetAIChanCalCalDate': grpc.unary_unary_rpc_method_handler(
                    servicer.GetAIChanCalCalDate,
                    request_deserializer=nidaqmx__pb2.GetAIChanCalCalDateRequest.FromString,
                    response_serializer=nidaqmx__pb2.GetAIChanCalCalDateResponse.SerializeToString,
            ),
            'GetAIChanCalExpDate': grpc.unary_unary_rpc_method_handler(
                    servicer.GetAIChanCalExpDate,
                    request_deserializer=nidaqmx__pb2.GetAIChanCalExpDateRequest.FromString,
                    response_serializer=nidaqmx__pb2.GetAIChanCalExpDateResponse.SerializeToString,
            ),
            'GetAnalogPowerUpStates': grpc.unary_unary_rpc_method_handler(
                    servicer.GetAnalogPowerUpStates,
                    request_deserializer=nidaqmx__pb2.GetAnalogPowerUpStatesRequest.FromString,
                    response_serializer=nidaqmx__pb2.GetAnalogPowerUpStatesResponse.SerializeToString,
            ),
            'GetAnalogPowerUpStatesWithOutputType': grpc.unary_unary_rpc_method_handler(
                    servicer.GetAnalogPowerUpStatesWithOutputType,
                    request_deserializer=nidaqmx__pb2.GetAnalogPowerUpStatesWithOutputTypeRequest.FromString,
                    response_serializer=nidaqmx__pb2.GetAnalogPowerUpStatesWithOutputTypeResponse.SerializeToString,
            ),
            'GetArmStartTrigTimestampVal': grpc.unary_unary_rpc_method_handler(
                    servicer.GetArmStartTrigTimestampVal,
                    request_deserializer=nidaqmx__pb2.GetArmStartTrigTimestampValRequest.FromString,
                    response_serializer=nidaqmx__pb2.GetArmStartTrigTimestampValResponse.SerializeToString,
            ),
            'GetArmStartTrigTrigWhen': grpc.unary_unary_rpc_method_handler(
                    servicer.GetArmStartTrigTrigWhen,
                    request_deserializer=nidaqmx__pb2.GetArmStartTrigTrigWhenRequest.FromString,
                    response_serializer=nidaqmx__pb2.GetArmStartTrigTrigWhenResponse.SerializeToString,
            ),
            'GetAutoConfiguredCDAQSyncConnections': grpc.unary_unary_rpc_method_handler(
                    servicer.GetAutoConfiguredCDAQSyncConnections,
                    request_deserializer=nidaqmx__pb2.GetAutoConfiguredCDAQSyncConnectionsRequest.FromString,
                    response_serializer=nidaqmx__pb2.GetAutoConfiguredCDAQSyncConnectionsResponse.SerializeToString,
            ),
            'GetBufferAttributeUInt32': grpc.unary_unary_rpc_method_handler(
                    servicer.GetBufferAttributeUInt32,
                    request_deserializer=nidaqmx__pb2.GetBufferAttributeUInt32Request.FromString,
                    response_serializer=nidaqmx__pb2.GetBufferAttributeUInt32Response.SerializeToString,
            ),
            'GetCalInfoAttributeBool': grpc.unary_unary_rpc_method_handler(
                    servicer.GetCalInfoAttributeBool,
                    request_deserializer=nidaqmx__pb2.GetCalInfoAttributeBoolRequest.FromString,
                    response_serializer=nidaqmx__pb2.GetCalInfoAttributeBoolResponse.SerializeToString,
            ),
            'GetCalInfoAttributeDouble': grpc.unary_unary_rpc_method_handler(
                    servicer.GetCalInfoAttributeDouble,
                    request_deserializer=nidaqmx__pb2.GetCalInfoAttributeDoubleRequest.FromString,
                    response_serializer=nidaqmx__pb2.GetCalInfoAttributeDoubleResponse.SerializeToString,
            ),
            'GetCalInfoAttributeString': grpc.unary_unary_rpc_method_handler(
                    servicer.GetCalInfoAttributeString,
                    request_deserializer=nidaqmx__pb2.GetCalInfoAttributeStringRequest.FromString,
                    response_serializer=nidaqmx__pb2.GetCalInfoAttributeStringResponse.SerializeToString,
            ),
            'GetCalInfoAttributeUInt32': grpc.unary_unary_rpc_method_handler(
                    servicer.GetCalInfoAttributeUInt32,
                    request_deserializer=nidaqmx__pb2.GetCalInfoAttributeUInt32Request.FromString,
                    response_serializer=nidaqmx__pb2.GetCalInfoAttributeUInt32Response.SerializeToString,
            ),
            'GetChanAttributeBool': grpc.unary_unary_rpc_method_handler(
                    servicer.GetChanAttributeBool,
                    request_deserializer=nidaqmx__pb2.GetChanAttributeBoolRequest.FromString,
                    response_serializer=nidaqmx__pb2.GetChanAttributeBoolResponse.SerializeToString,
            ),
            'GetChanAttributeDouble': grpc.unary_unary_rpc_method_handler(
                    servicer.GetChanAttributeDouble,
                    request_deserializer=nidaqmx__pb2.GetChanAttributeDoubleRequest.FromString,
                    response_serializer=nidaqmx__pb2.GetChanAttributeDoubleResponse.SerializeToString,
            ),
            'GetChanAttributeDoubleArray': grpc.unary_unary_rpc_method_handler(
                    servicer.GetChanAttributeDoubleArray,
                    request_deserializer=nidaqmx__pb2.GetChanAttributeDoubleArrayRequest.FromString,
                    response_serializer=nidaqmx__pb2.GetChanAttributeDoubleArrayResponse.SerializeToString,
            ),
            'GetChanAttributeInt32': grpc.unary_unary_rpc_method_handler(
                    servicer.GetChanAttributeInt32,
                    request_deserializer=nidaqmx__pb2.GetChanAttributeInt32Request.FromString,
                    response_serializer=nidaqmx__pb2.GetChanAttributeInt32Response.SerializeToString,
            ),
            'GetChanAttributeString': grpc.unary_unary_rpc_method_handler(
                    servicer.GetChanAttributeString,
                    request_deserializer=nidaqmx__pb2.GetChanAttributeStringRequest.FromString,
                    response_serializer=nidaqmx__pb2.GetChanAttributeStringResponse.SerializeToString,
            ),
            'GetChanAttributeUInt32': grpc.unary_unary_rpc_method_handler(
                    servicer.GetChanAttributeUInt32,
                    request_deserializer=nidaqmx__pb2.GetChanAttributeUInt32Request.FromString,
                    response_serializer=nidaqmx__pb2.GetChanAttributeUInt32Response.SerializeToString,
            ),
            'GetDeviceAttributeBool': grpc.unary_unary_rpc_method_handler(
                    servicer.GetDeviceAttributeBool,
                    request_deserializer=nidaqmx__pb2.GetDeviceAttributeBoolRequest.FromString,
                    response_serializer=nidaqmx__pb2.GetDeviceAttributeBoolResponse.SerializeToString,
            ),
            'GetDeviceAttributeDouble': grpc.unary_unary_rpc_method_handler(
                    servicer.GetDeviceAttributeDouble,
                    request_deserializer=nidaqmx__pb2.GetDeviceAttributeDoubleRequest.FromString,
                    response_serializer=nidaqmx__pb2.GetDeviceAttributeDoubleResponse.SerializeToString,
            ),
            'GetDeviceAttributeDoubleArray': grpc.unary_unary_rpc_method_handler(
                    servicer.GetDeviceAttributeDoubleArray,
                    request_deserializer=nidaqmx__pb2.GetDeviceAttributeDoubleArrayRequest.FromString,
                    response_serializer=nidaqmx__pb2.GetDeviceAttributeDoubleArrayResponse.SerializeToString,
            ),
            'GetDeviceAttributeInt32': grpc.unary_unary_rpc_method_handler(
                    servicer.GetDeviceAttributeInt32,
                    request_deserializer=nidaqmx__pb2.GetDeviceAttributeInt32Request.FromString,
                    response_serializer=nidaqmx__pb2.GetDeviceAttributeInt32Response.SerializeToString,
            ),
            'GetDeviceAttributeInt32Array': grpc.unary_unary_rpc_method_handler(
                    servicer.GetDeviceAttributeInt32Array,
                    request_deserializer=nidaqmx__pb2.GetDeviceAttributeInt32ArrayRequest.FromString,
                    response_serializer=nidaqmx__pb2.GetDeviceAttributeInt32ArrayResponse.SerializeToString,
            ),
            'GetDeviceAttributeString': grpc.unary_unary_rpc_method_handler(
                    servicer.GetDeviceAttributeString,
                    request_deserializer=nidaqmx__pb2.GetDeviceAttributeStringRequest.FromString,
                    response_serializer=nidaqmx__pb2.GetDeviceAttributeStringResponse.SerializeToString,
            ),
            'GetDeviceAttributeUInt32': grpc.unary_unary_rpc_method_handler(
                    servicer.GetDeviceAttributeUInt32,
                    request_deserializer=nidaqmx__pb2.GetDeviceAttributeUInt32Request.FromString,
                    response_serializer=nidaqmx__pb2.GetDeviceAttributeUInt32Response.SerializeToString,
            ),
            'GetDeviceAttributeUInt32Array': grpc.unary_unary_rpc_method_handler(
                    servicer.GetDeviceAttributeUInt32Array,
                    request_deserializer=nidaqmx__pb2.GetDeviceAttributeUInt32ArrayRequest.FromString,
                    response_serializer=nidaqmx__pb2.GetDeviceAttributeUInt32ArrayResponse.SerializeToString,
            ),
            'GetDigitalLogicFamilyPowerUpState': grpc.unary_unary_rpc_method_handler(
                    servicer.GetDigitalLogicFamilyPowerUpState,
                    request_deserializer=nidaqmx__pb2.GetDigitalLogicFamilyPowerUpStateRequest.FromString,
                    response_serializer=nidaqmx__pb2.GetDigitalLogicFamilyPowerUpStateResponse.SerializeToString,
            ),
            'GetDigitalPowerUpStates': grpc.unary_unary_rpc_method_handler(
                    servicer.GetDigitalPowerUpStates,
                    request_deserializer=nidaqmx__pb2.GetDigitalPowerUpStatesRequest.FromString,
                    response_serializer=nidaqmx__pb2.GetDigitalPowerUpStatesResponse.SerializeToString,
            ),
            'GetDigitalPullUpPullDownStates': grpc.unary_unary_rpc_method_handler(
                    servicer.GetDigitalPullUpPullDownStates,
                    request_deserializer=nidaqmx__pb2.GetDigitalPullUpPullDownStatesRequest.FromString,
                    response_serializer=nidaqmx__pb2.GetDigitalPullUpPullDownStatesResponse.SerializeToString,
            ),
            'GetDisconnectedCDAQSyncPorts': grpc.unary_unary_rpc_method_handler(
                    servicer.GetDisconnectedCDAQSyncPorts,
                    request_deserializer=nidaqmx__pb2.GetDisconnectedCDAQSyncPortsRequest.FromString,
                    response_serializer=nidaqmx__pb2.GetDisconnectedCDAQSyncPortsResponse.SerializeToString,
            ),
            'GetErrorString': grpc.unary_unary_rpc_method_handler(
                    servicer.GetErrorString,
                    request_deserializer=nidaqmx__pb2.GetErrorStringRequest.FromString,
                    response_serializer=nidaqmx__pb2.GetErrorStringResponse.SerializeToString,
            ),
            'GetExportedSignalAttributeBool': grpc.unary_unary_rpc_method_handler(
                    servicer.GetExportedSignalAttributeBool,
                    request_deserializer=nidaqmx__pb2.GetExportedSignalAttributeBoolRequest.FromString,
                    response_serializer=nidaqmx__pb2.GetExportedSignalAttributeBoolResponse.SerializeToString,
            ),
            'GetExportedSignalAttributeDouble': grpc.unary_unary_rpc_method_handler(
                    servicer.GetExportedSignalAttributeDouble,
                    request_deserializer=nidaqmx__pb2.GetExportedSignalAttributeDoubleRequest.FromString,
                    response_serializer=nidaqmx__pb2.GetExportedSignalAttributeDoubleResponse.SerializeToString,
            ),
            'GetExportedSignalAttributeInt32': grpc.unary_unary_rpc_method_handler(
                    servicer.GetExportedSignalAttributeInt32,
                    request_deserializer=nidaqmx__pb2.GetExportedSignalAttributeInt32Request.FromString,
                    response_serializer=nidaqmx__pb2.GetExportedSignalAttributeInt32Response.SerializeToString,
            ),
            'GetExportedSignalAttributeString': grpc.unary_unary_rpc_method_handler(
                    servicer.GetExportedSignalAttributeString,
                    request_deserializer=nidaqmx__pb2.GetExportedSignalAttributeStringRequest.FromString,
                    response_serializer=nidaqmx__pb2.GetExportedSignalAttributeStringResponse.SerializeToString,
            ),
            'GetExportedSignalAttributeUInt32': grpc.unary_unary_rpc_method_handler(
                    servicer.GetExportedSignalAttributeUInt32,
                    request_deserializer=nidaqmx__pb2.GetExportedSignalAttributeUInt32Request.FromString,
                    response_serializer=nidaqmx__pb2.GetExportedSignalAttributeUInt32Response.SerializeToString,
            ),
            'GetExtCalLastDateAndTime': grpc.unary_unary_rpc_method_handler(
                    servicer.GetExtCalLastDateAndTime,
                    request_deserializer=nidaqmx__pb2.GetExtCalLastDateAndTimeRequest.FromString,
                    response_serializer=nidaqmx__pb2.GetExtCalLastDateAndTimeResponse.SerializeToString,
            ),
            'GetFirstSampClkWhen': grpc.unary_unary_rpc_method_handler(
                    servicer.GetFirstSampClkWhen,
                    request_deserializer=nidaqmx__pb2.GetFirstSampClkWhenRequest.FromString,
                    response_serializer=nidaqmx__pb2.GetFirstSampClkWhenResponse.SerializeToString,
            ),
            'GetFirstSampTimestampVal': grpc.unary_unary_rpc_method_handler(
                    servicer.GetFirstSampTimestampVal,
                    request_deserializer=nidaqmx__pb2.GetFirstSampTimestampValRequest.FromString,
                    response_serializer=nidaqmx__pb2.GetFirstSampTimestampValResponse.SerializeToString,
            ),
            'GetNthTaskChannel': grpc.unary_unary_rpc_method_handler(
                    servicer.GetNthTaskChannel,
                    request_deserializer=nidaqmx__pb2.GetNthTaskChannelRequest.FromString,
                    response_serializer=nidaqmx__pb2.GetNthTaskChannelResponse.SerializeToString,
            ),
            'GetNthTaskDevice': grpc.unary_unary_rpc_method_handler(
                    servicer.GetNthTaskDevice,
                    request_deserializer=nidaqmx__pb2.GetNthTaskDeviceRequest.FromString,
                    response_serializer=nidaqmx__pb2.GetNthTaskDeviceResponse.SerializeToString,
            ),
            'GetNthTaskReadChannel': grpc.unary_unary_rpc_method_handler(
                    servicer.GetNthTaskReadChannel,
                    request_deserializer=nidaqmx__pb2.GetNthTaskReadChannelRequest.FromString,
                    response_serializer=nidaqmx__pb2.GetNthTaskReadChannelResponse.SerializeToString,
            ),
            'GetPersistedChanAttributeBool': grpc.unary_unary_rpc_method_handler(
                    servicer.GetPersistedChanAttributeBool,
                    request_deserializer=nidaqmx__pb2.GetPersistedChanAttributeBoolRequest.FromString,
                    response_serializer=nidaqmx__pb2.GetPersistedChanAttributeBoolResponse.SerializeToString,
            ),
            'GetPersistedChanAttributeString': grpc.unary_unary_rpc_method_handler(
                    servicer.GetPersistedChanAttributeString,
                    request_deserializer=nidaqmx__pb2.GetPersistedChanAttributeStringRequest.FromString,
                    response_serializer=nidaqmx__pb2.GetPersistedChanAttributeStringResponse.SerializeToString,
            ),
            'GetPersistedScaleAttributeBool': grpc.unary_unary_rpc_method_handler(
                    servicer.GetPersistedScaleAttributeBool,
                    request_deserializer=nidaqmx__pb2.GetPersistedScaleAttributeBoolRequest.FromString,
                    response_serializer=nidaqmx__pb2.GetPersistedScaleAttributeBoolResponse.SerializeToString,
            ),
            'GetPersistedScaleAttributeString': grpc.unary_unary_rpc_method_handler(
                    servicer.GetPersistedScaleAttributeString,
                    request_deserializer=nidaqmx__pb2.GetPersistedScaleAttributeStringRequest.FromString,
                    response_serializer=nidaqmx__pb2.GetPersistedScaleAttributeStringResponse.SerializeToString,
            ),
            'GetPersistedTaskAttributeBool': grpc.unary_unary_rpc_method_handler(
                    servicer.GetPersistedTaskAttributeBool,
                    request_deserializer=nidaqmx__pb2.GetPersistedTaskAttributeBoolRequest.FromString,
                    response_serializer=nidaqmx__pb2.GetPersistedTaskAttributeBoolResponse.SerializeToString,
            ),
            'GetPersistedTaskAttributeString': grpc.unary_unary_rpc_method_handler(
                    servicer.GetPersistedTaskAttributeString,
                    request_deserializer=nidaqmx__pb2.GetPersistedTaskAttributeStringRequest.FromString,
                    response_serializer=nidaqmx__pb2.GetPersistedTaskAttributeStringResponse.SerializeToString,
            ),
            'GetPhysicalChanAttributeBool': grpc.unary_unary_rpc_method_handler(
                    servicer.GetPhysicalChanAttributeBool,
                    request_deserializer=nidaqmx__pb2.GetPhysicalChanAttributeBoolRequest.FromString,
                    response_serializer=nidaqmx__pb2.GetPhysicalChanAttributeBoolResponse.SerializeToString,
            ),
            'GetPhysicalChanAttributeBytes': grpc.unary_unary_rpc_method_handler(
                    servicer.GetPhysicalChanAttributeBytes,
                    request_deserializer=nidaqmx__pb2.GetPhysicalChanAttributeBytesRequest.FromString,
                    response_serializer=nidaqmx__pb2.GetPhysicalChanAttributeBytesResponse.SerializeToString,
            ),
            'GetPhysicalChanAttributeDouble': grpc.unary_unary_rpc_method_handler(
                    servicer.GetPhysicalChanAttributeDouble,
                    request_deserializer=nidaqmx__pb2.GetPhysicalChanAttributeDoubleRequest.FromString,
                    response_serializer=nidaqmx__pb2.GetPhysicalChanAttributeDoubleResponse.SerializeToString,
            ),
            'GetPhysicalChanAttributeDoubleArray': grpc.unary_unary_rpc_method_handler(
                    servicer.GetPhysicalChanAttributeDoubleArray,
                    request_deserializer=nidaqmx__pb2.GetPhysicalChanAttributeDoubleArrayRequest.FromString,
                    response_serializer=nidaqmx__pb2.GetPhysicalChanAttributeDoubleArrayResponse.SerializeToString,
            ),
            'GetPhysicalChanAttributeInt32': grpc.unary_unary_rpc_method_handler(
                    servicer.GetPhysicalChanAttributeInt32,
                    request_deserializer=nidaqmx__pb2.GetPhysicalChanAttributeInt32Request.FromString,
                    response_serializer=nidaqmx__pb2.GetPhysicalChanAttributeInt32Response.SerializeToString,
            ),
            'GetPhysicalChanAttributeInt32Array': grpc.unary_unary_rpc_method_handler(
                    servicer.GetPhysicalChanAttributeInt32Array,
                    request_deserializer=nidaqmx__pb2.GetPhysicalChanAttributeInt32ArrayRequest.FromString,
                    response_serializer=nidaqmx__pb2.GetPhysicalChanAttributeInt32ArrayResponse.SerializeToString,
            ),
            'GetPhysicalChanAttributeString': grpc.unary_unary_rpc_method_handler(
                    servicer.GetPhysicalChanAttributeString,
                    request_deserializer=nidaqmx__pb2.GetPhysicalChanAttributeStringRequest.FromString,
                    response_serializer=nidaqmx__pb2.GetPhysicalChanAttributeStringResponse.SerializeToString,
            ),
            'GetPhysicalChanAttributeUInt32': grpc.unary_unary_rpc_method_handler(
                    servicer.GetPhysicalChanAttributeUInt32,
                    request_deserializer=nidaqmx__pb2.GetPhysicalChanAttributeUInt32Request.FromString,
                    response_serializer=nidaqmx__pb2.GetPhysicalChanAttributeUInt32Response.SerializeToString,
            ),
            'GetPhysicalChanAttributeUInt32Array': grpc.unary_unary_rpc_method_handler(
                    servicer.GetPhysicalChanAttributeUInt32Array,
                    request_deserializer=nidaqmx__pb2.GetPhysicalChanAttributeUInt32ArrayRequest.FromString,
                    response_serializer=nidaqmx__pb2.GetPhysicalChanAttributeUInt32ArrayResponse.SerializeToString,
            ),
            'GetReadAttributeBool': grpc.unary_unary_rpc_method_handler(
                    servicer.GetReadAttributeBool,
                    request_deserializer=nidaqmx__pb2.GetReadAttributeBoolRequest.FromString,
                    response_serializer=nidaqmx__pb2.GetReadAttributeBoolResponse.SerializeToString,
            ),
            'GetReadAttributeDouble': grpc.unary_unary_rpc_method_handler(
                    servicer.GetReadAttributeDouble,
                    request_deserializer=nidaqmx__pb2.GetReadAttributeDoubleRequest.FromString,
                    response_serializer=nidaqmx__pb2.GetReadAttributeDoubleResponse.SerializeToString,
            ),
            'GetReadAttributeInt32': grpc.unary_unary_rpc_method_handler(
                    servicer.GetReadAttributeInt32,
                    request_deserializer=nidaqmx__pb2.GetReadAttributeInt32Request.FromString,
                    response_serializer=nidaqmx__pb2.GetReadAttributeInt32Response.SerializeToString,
            ),
            'GetReadAttributeString': grpc.unary_unary_rpc_method_handler(
                    servicer.GetReadAttributeString,
                    request_deserializer=nidaqmx__pb2.GetReadAttributeStringRequest.FromString,
                    response_serializer=nidaqmx__pb2.GetReadAttributeStringResponse.SerializeToString,
            ),
            'GetReadAttributeUInt32': grpc.unary_unary_rpc_method_handler(
                    servicer.GetReadAttributeUInt32,
                    request_deserializer=nidaqmx__pb2.GetReadAttributeUInt32Request.FromString,
                    response_serializer=nidaqmx__pb2.GetReadAttributeUInt32Response.SerializeToString,
            ),
            'GetReadAttributeUInt64': grpc.unary_unary_rpc_method_handler(
                    servicer.GetReadAttributeUInt64,
                    request_deserializer=nidaqmx__pb2.GetReadAttributeUInt64Request.FromString,
                    response_serializer=nidaqmx__pb2.GetReadAttributeUInt64Response.SerializeToString,
            ),
            'GetRealTimeAttributeBool': grpc.unary_unary_rpc_method_handler(
                    servicer.GetRealTimeAttributeBool,
                    request_deserializer=nidaqmx__pb2.GetRealTimeAttributeBoolRequest.FromString,
                    response_serializer=nidaqmx__pb2.GetRealTimeAttributeBoolResponse.SerializeToString,
            ),
            'GetRealTimeAttributeInt32': grpc.unary_unary_rpc_method_handler(
                    servicer.GetRealTimeAttributeInt32,
                    request_deserializer=nidaqmx__pb2.GetRealTimeAttributeInt32Request.FromString,
                    response_serializer=nidaqmx__pb2.GetRealTimeAttributeInt32Response.SerializeToString,
            ),
            'GetRealTimeAttributeUInt32': grpc.unary_unary_rpc_method_handler(
                    servicer.GetRealTimeAttributeUInt32,
                    request_deserializer=nidaqmx__pb2.GetRealTimeAttributeUInt32Request.FromString,
                    response_serializer=nidaqmx__pb2.GetRealTimeAttributeUInt32Response.SerializeToString,
            ),
            'GetRefTrigTimestampVal': grpc.unary_unary_rpc_method_handler(
                    servicer.GetRefTrigTimestampVal,
                    request_deserializer=nidaqmx__pb2.GetRefTrigTimestampValRequest.FromString,
                    response_serializer=nidaqmx__pb2.GetRefTrigTimestampValResponse.SerializeToString,
            ),
            'GetScaleAttributeDouble': grpc.unary_unary_rpc_method_handler(
                    servicer.GetScaleAttributeDouble,
                    request_deserializer=nidaqmx__pb2.GetScaleAttributeDoubleRequest.FromString,
                    response_serializer=nidaqmx__pb2.GetScaleAttributeDoubleResponse.SerializeToString,
            ),
            'GetScaleAttributeDoubleArray': grpc.unary_unary_rpc_method_handler(
                    servicer.GetScaleAttributeDoubleArray,
                    request_deserializer=nidaqmx__pb2.GetScaleAttributeDoubleArrayRequest.FromString,
                    response_serializer=nidaqmx__pb2.GetScaleAttributeDoubleArrayResponse.SerializeToString,
            ),
            'GetScaleAttributeInt32': grpc.unary_unary_rpc_method_handler(
                    servicer.GetScaleAttributeInt32,
                    request_deserializer=nidaqmx__pb2.GetScaleAttributeInt32Request.FromString,
                    response_serializer=nidaqmx__pb2.GetScaleAttributeInt32Response.SerializeToString,
            ),
            'GetScaleAttributeString': grpc.unary_unary_rpc_method_handler(
                    servicer.GetScaleAttributeString,
                    request_deserializer=nidaqmx__pb2.GetScaleAttributeStringRequest.FromString,
                    response_serializer=nidaqmx__pb2.GetScaleAttributeStringResponse.SerializeToString,
            ),
            'GetSelfCalLastDateAndTime': grpc.unary_unary_rpc_method_handler(
                    servicer.GetSelfCalLastDateAndTime,
                    request_deserializer=nidaqmx__pb2.GetSelfCalLastDateAndTimeRequest.FromString,
                    response_serializer=nidaqmx__pb2.GetSelfCalLastDateAndTimeResponse.SerializeToString,
            ),
            'GetStartTrigTimestampVal': grpc.unary_unary_rpc_method_handler(
                    servicer.GetStartTrigTimestampVal,
                    request_deserializer=nidaqmx__pb2.GetStartTrigTimestampValRequest.FromString,
                    response_serializer=nidaqmx__pb2.GetStartTrigTimestampValResponse.SerializeToString,
            ),
            'GetStartTrigTrigWhen': grpc.unary_unary_rpc_method_handler(
                    servicer.GetStartTrigTrigWhen,
                    request_deserializer=nidaqmx__pb2.GetStartTrigTrigWhenRequest.FromString,
                    response_serializer=nidaqmx__pb2.GetStartTrigTrigWhenResponse.SerializeToString,
            ),
            'GetSyncPulseTimeWhen': grpc.unary_unary_rpc_method_handler(
                    servicer.GetSyncPulseTimeWhen,
                    request_deserializer=nidaqmx__pb2.GetSyncPulseTimeWhenRequest.FromString,
                    response_serializer=nidaqmx__pb2.GetSyncPulseTimeWhenResponse.SerializeToString,
            ),
            'GetSystemInfoAttributeString': grpc.unary_unary_rpc_method_handler(
                    servicer.GetSystemInfoAttributeString,
                    request_deserializer=nidaqmx__pb2.GetSystemInfoAttributeStringRequest.FromString,
                    response_serializer=nidaqmx__pb2.GetSystemInfoAttributeStringResponse.SerializeToString,
            ),
            'GetSystemInfoAttributeUInt32': grpc.unary_unary_rpc_method_handler(
                    servicer.GetSystemInfoAttributeUInt32,
                    request_deserializer=nidaqmx__pb2.GetSystemInfoAttributeUInt32Request.FromString,
                    response_serializer=nidaqmx__pb2.GetSystemInfoAttributeUInt32Response.SerializeToString,
            ),
            'GetTaskAttributeBool': grpc.unary_unary_rpc_method_handler(
                    servicer.GetTaskAttributeBool,
                    request_deserializer=nidaqmx__pb2.GetTaskAttributeBoolRequest.FromString,
                    response_serializer=nidaqmx__pb2.GetTaskAttributeBoolResponse.SerializeToString,
            ),
            'GetTaskAttributeString': grpc.unary_unary_rpc_method_handler(
                    servicer.GetTaskAttributeString,
                    request_deserializer=nidaqmx__pb2.GetTaskAttributeStringRequest.FromString,
                    response_serializer=nidaqmx__pb2.GetTaskAttributeStringResponse.SerializeToString,
            ),
            'GetTaskAttributeUInt32': grpc.unary_unary_rpc_method_handler(
                    servicer.GetTaskAttributeUInt32,
                    request_deserializer=nidaqmx__pb2.GetTaskAttributeUInt32Request.FromString,
                    response_serializer=nidaqmx__pb2.GetTaskAttributeUInt32Response.SerializeToString,
            ),
            'GetTimingAttributeBool': grpc.unary_unary_rpc_method_handler(
                    servicer.GetTimingAttributeBool,
                    request_deserializer=nidaqmx__pb2.GetTimingAttributeBoolRequest.FromString,
                    response_serializer=nidaqmx__pb2.GetTimingAttributeBoolResponse.SerializeToString,
            ),
            'GetTimingAttributeDouble': grpc.unary_unary_rpc_method_handler(
                    servicer.GetTimingAttributeDouble,
                    request_deserializer=nidaqmx__pb2.GetTimingAttributeDoubleRequest.FromString,
                    response_serializer=nidaqmx__pb2.GetTimingAttributeDoubleResponse.SerializeToString,
            ),
            'GetTimingAttributeExBool': grpc.unary_unary_rpc_method_handler(
                    servicer.GetTimingAttributeExBool,
                    request_deserializer=nidaqmx__pb2.GetTimingAttributeExBoolRequest.FromString,
                    response_serializer=nidaqmx__pb2.GetTimingAttributeExBoolResponse.SerializeToString,
            ),
            'GetTimingAttributeExDouble': grpc.unary_unary_rpc_method_handler(
                    servicer.GetTimingAttributeExDouble,
                    request_deserializer=nidaqmx__pb2.GetTimingAttributeExDoubleRequest.FromString,
                    response_serializer=nidaqmx__pb2.GetTimingAttributeExDoubleResponse.SerializeToString,
            ),
            'GetTimingAttributeExInt32': grpc.unary_unary_rpc_method_handler(
                    servicer.GetTimingAttributeExInt32,
                    request_deserializer=nidaqmx__pb2.GetTimingAttributeExInt32Request.FromString,
                    response_serializer=nidaqmx__pb2.GetTimingAttributeExInt32Response.SerializeToString,
            ),
            'GetTimingAttributeExString': grpc.unary_unary_rpc_method_handler(
                    servicer.GetTimingAttributeExString,
                    request_deserializer=nidaqmx__pb2.GetTimingAttributeExStringRequest.FromString,
                    response_serializer=nidaqmx__pb2.GetTimingAttributeExStringResponse.SerializeToString,
            ),
            'GetTimingAttributeExTimestamp': grpc.unary_unary_rpc_method_handler(
                    servicer.GetTimingAttributeExTimestamp,
                    request_deserializer=nidaqmx__pb2.GetTimingAttributeExTimestampRequest.FromString,
                    response_serializer=nidaqmx__pb2.GetTimingAttributeExTimestampResponse.SerializeToString,
            ),
            'GetTimingAttributeExUInt32': grpc.unary_unary_rpc_method_handler(
                    servicer.GetTimingAttributeExUInt32,
                    request_deserializer=nidaqmx__pb2.GetTimingAttributeExUInt32Request.FromString,
                    response_serializer=nidaqmx__pb2.GetTimingAttributeExUInt32Response.SerializeToString,
            ),
            'GetTimingAttributeExUInt64': grpc.unary_unary_rpc_method_handler(
                    servicer.GetTimingAttributeExUInt64,
                    request_deserializer=nidaqmx__pb2.GetTimingAttributeExUInt64Request.FromString,
                    response_serializer=nidaqmx__pb2.GetTimingAttributeExUInt64Response.SerializeToString,
            ),
            'GetTimingAttributeInt32': grpc.unary_unary_rpc_method_handler(
                    servicer.GetTimingAttributeInt32,
                    request_deserializer=nidaqmx__pb2.GetTimingAttributeInt32Request.FromString,
                    response_serializer=nidaqmx__pb2.GetTimingAttributeInt32Response.SerializeToString,
            ),
            'GetTimingAttributeString': grpc.unary_unary_rpc_method_handler(
                    servicer.GetTimingAttributeString,
                    request_deserializer=nidaqmx__pb2.GetTimingAttributeStringRequest.FromString,
                    response_serializer=nidaqmx__pb2.GetTimingAttributeStringResponse.SerializeToString,
            ),
            'GetTimingAttributeTimestamp': grpc.unary_unary_rpc_method_handler(
                    servicer.GetTimingAttributeTimestamp,
                    request_deserializer=nidaqmx__pb2.GetTimingAttributeTimestampRequest.FromString,
                    response_serializer=nidaqmx__pb2.GetTimingAttributeTimestampResponse.SerializeToString,
            ),
            'GetTimingAttributeUInt32': grpc.unary_unary_rpc_method_handler(
                    servicer.GetTimingAttributeUInt32,
                    request_deserializer=nidaqmx__pb2.GetTimingAttributeUInt32Request.FromString,
                    response_serializer=nidaqmx__pb2.GetTimingAttributeUInt32Response.SerializeToString,
            ),
            'GetTimingAttributeUInt64': grpc.unary_unary_rpc_method_handler(
                    servicer.GetTimingAttributeUInt64,
                    request_deserializer=nidaqmx__pb2.GetTimingAttributeUInt64Request.FromString,
                    response_serializer=nidaqmx__pb2.GetTimingAttributeUInt64Response.SerializeToString,
            ),
            'GetTrigAttributeBool': grpc.unary_unary_rpc_method_handler(
                    servicer.GetTrigAttributeBool,
                    request_deserializer=nidaqmx__pb2.GetTrigAttributeBoolRequest.FromString,
                    response_serializer=nidaqmx__pb2.GetTrigAttributeBoolResponse.SerializeToString,
            ),
            'GetTrigAttributeDouble': grpc.unary_unary_rpc_method_handler(
                    servicer.GetTrigAttributeDouble,
                    request_deserializer=nidaqmx__pb2.GetTrigAttributeDoubleRequest.FromString,
                    response_serializer=nidaqmx__pb2.GetTrigAttributeDoubleResponse.SerializeToString,
            ),
            'GetTrigAttributeDoubleArray': grpc.unary_unary_rpc_method_handler(
                    servicer.GetTrigAttributeDoubleArray,
                    request_deserializer=nidaqmx__pb2.GetTrigAttributeDoubleArrayRequest.FromString,
                    response_serializer=nidaqmx__pb2.GetTrigAttributeDoubleArrayResponse.SerializeToString,
            ),
            'GetTrigAttributeInt32': grpc.unary_unary_rpc_method_handler(
                    servicer.GetTrigAttributeInt32,
                    request_deserializer=nidaqmx__pb2.GetTrigAttributeInt32Request.FromString,
                    response_serializer=nidaqmx__pb2.GetTrigAttributeInt32Response.SerializeToString,
            ),
            'GetTrigAttributeInt32Array': grpc.unary_unary_rpc_method_handler(
                    servicer.GetTrigAttributeInt32Array,
                    request_deserializer=nidaqmx__pb2.GetTrigAttributeInt32ArrayRequest.FromString,
                    response_serializer=nidaqmx__pb2.GetTrigAttributeInt32ArrayResponse.SerializeToString,
            ),
            'GetTrigAttributeString': grpc.unary_unary_rpc_method_handler(
                    servicer.GetTrigAttributeString,
                    request_deserializer=nidaqmx__pb2.GetTrigAttributeStringRequest.FromString,
                    response_serializer=nidaqmx__pb2.GetTrigAttributeStringResponse.SerializeToString,
            ),
            'GetTrigAttributeTimestamp': grpc.unary_unary_rpc_method_handler(
                    servicer.GetTrigAttributeTimestamp,
                    request_deserializer=nidaqmx__pb2.GetTrigAttributeTimestampRequest.FromString,
                    response_serializer=nidaqmx__pb2.GetTrigAttributeTimestampResponse.SerializeToString,
            ),
            'GetTrigAttributeUInt32': grpc.unary_unary_rpc_method_handler(
                    servicer.GetTrigAttributeUInt32,
                    request_deserializer=nidaqmx__pb2.GetTrigAttributeUInt32Request.FromString,
                    response_serializer=nidaqmx__pb2.GetTrigAttributeUInt32Response.SerializeToString,
            ),
            'GetWatchdogAttributeBool': grpc.unary_unary_rpc_method_handler(
                    servicer.GetWatchdogAttributeBool,
                    request_deserializer=nidaqmx__pb2.GetWatchdogAttributeBoolRequest.FromString,
                    response_serializer=nidaqmx__pb2.GetWatchdogAttributeBoolResponse.SerializeToString,
            ),
            'GetWatchdogAttributeDouble': grpc.unary_unary_rpc_method_handler(
                    servicer.GetWatchdogAttributeDouble,
                    request_deserializer=nidaqmx__pb2.GetWatchdogAttributeDoubleRequest.FromString,
                    response_serializer=nidaqmx__pb2.GetWatchdogAttributeDoubleResponse.SerializeToString,
            ),
            'GetWatchdogAttributeInt32': grpc.unary_unary_rpc_method_handler(
                    servicer.GetWatchdogAttributeInt32,
                    request_deserializer=nidaqmx__pb2.GetWatchdogAttributeInt32Request.FromString,
                    response_serializer=nidaqmx__pb2.GetWatchdogAttributeInt32Response.SerializeToString,
            ),
            'GetWatchdogAttributeString': grpc.unary_unary_rpc_method_handler(
                    servicer.GetWatchdogAttributeString,
                    request_deserializer=nidaqmx__pb2.GetWatchdogAttributeStringRequest.FromString,
                    response_serializer=nidaqmx__pb2.GetWatchdogAttributeStringResponse.SerializeToString,
            ),
            'GetWriteAttributeBool': grpc.unary_unary_rpc_method_handler(
                    servicer.GetWriteAttributeBool,
                    request_deserializer=nidaqmx__pb2.GetWriteAttributeBoolRequest.FromString,
                    response_serializer=nidaqmx__pb2.GetWriteAttributeBoolResponse.SerializeToString,
            ),
            'GetWriteAttributeDouble': grpc.unary_unary_rpc_method_handler(
                    servicer.GetWriteAttributeDouble,
                    request_deserializer=nidaqmx__pb2.GetWriteAttributeDoubleRequest.FromString,
                    response_serializer=nidaqmx__pb2.GetWriteAttributeDoubleResponse.SerializeToString,
            ),
            'GetWriteAttributeInt32': grpc.unary_unary_rpc_method_handler(
                    servicer.GetWriteAttributeInt32,
                    request_deserializer=nidaqmx__pb2.GetWriteAttributeInt32Request.FromString,
                    response_serializer=nidaqmx__pb2.GetWriteAttributeInt32Response.SerializeToString,
            ),
            'GetWriteAttributeString': grpc.unary_unary_rpc_method_handler(
                    servicer.GetWriteAttributeString,
                    request_deserializer=nidaqmx__pb2.GetWriteAttributeStringRequest.FromString,
                    response_serializer=nidaqmx__pb2.GetWriteAttributeStringResponse.SerializeToString,
            ),
            'GetWriteAttributeUInt32': grpc.unary_unary_rpc_method_handler(
                    servicer.GetWriteAttributeUInt32,
                    request_deserializer=nidaqmx__pb2.GetWriteAttributeUInt32Request.FromString,
                    response_serializer=nidaqmx__pb2.GetWriteAttributeUInt32Response.SerializeToString,
            ),
            'GetWriteAttributeUInt64': grpc.unary_unary_rpc_method_handler(
                    servicer.GetWriteAttributeUInt64,
                    request_deserializer=nidaqmx__pb2.GetWriteAttributeUInt64Request.FromString,
                    response_serializer=nidaqmx__pb2.GetWriteAttributeUInt64Response.SerializeToString,
            ),
            'IsTaskDone': grpc.unary_unary_rpc_method_handler(
                    servicer.IsTaskDone,
                    request_deserializer=nidaqmx__pb2.IsTaskDoneRequest.FromString,
                    response_serializer=nidaqmx__pb2.IsTaskDoneResponse.SerializeToString,
            ),
            'LoadTask': grpc.unary_unary_rpc_method_handler(
                    servicer.LoadTask,
                    request_deserializer=nidaqmx__pb2.LoadTaskRequest.FromString,
                    response_serializer=nidaqmx__pb2.LoadTaskResponse.SerializeToString,
            ),
            'PerformBridgeOffsetNullingCalEx': grpc.unary_unary_rpc_method_handler(
                    servicer.PerformBridgeOffsetNullingCalEx,
                    request_deserializer=nidaqmx__pb2.PerformBridgeOffsetNullingCalExRequest.FromString,
                    response_serializer=nidaqmx__pb2.PerformBridgeOffsetNullingCalExResponse.SerializeToString,
            ),
            'PerformBridgeShuntCalEx': grpc.unary_unary_rpc_method_handler(
                    servicer.PerformBridgeShuntCalEx,
                    request_deserializer=nidaqmx__pb2.PerformBridgeShuntCalExRequest.FromString,
                    response_serializer=nidaqmx__pb2.PerformBridgeShuntCalExResponse.SerializeToString,
            ),
            'PerformStrainShuntCalEx': grpc.unary_unary_rpc_method_handler(
                    servicer.PerformStrainShuntCalEx,
                    request_deserializer=nidaqmx__pb2.PerformStrainShuntCalExRequest.FromString,
                    response_serializer=nidaqmx__pb2.PerformStrainShuntCalExResponse.SerializeToString,
            ),
            'PerformThrmcplLeadOffsetNullingCal': grpc.unary_unary_rpc_method_handler(
                    servicer.PerformThrmcplLeadOffsetNullingCal,
                    request_deserializer=nidaqmx__pb2.PerformThrmcplLeadOffsetNullingCalRequest.FromString,
                    response_serializer=nidaqmx__pb2.PerformThrmcplLeadOffsetNullingCalResponse.SerializeToString,
            ),
            'ReadAnalogF64': grpc.unary_unary_rpc_method_handler(
                    servicer.ReadAnalogF64,
                    request_deserializer=nidaqmx__pb2.ReadAnalogF64Request.FromString,
                    response_serializer=nidaqmx__pb2.ReadAnalogF64Response.SerializeToString,
            ),
            'ReadAnalogScalarF64': grpc.unary_unary_rpc_method_handler(
                    servicer.ReadAnalogScalarF64,
                    request_deserializer=nidaqmx__pb2.ReadAnalogScalarF64Request.FromString,
                    response_serializer=nidaqmx__pb2.ReadAnalogScalarF64Response.SerializeToString,
            ),
            'ReadBinaryI16': grpc.unary_unary_rpc_method_handler(
                    servicer.ReadBinaryI16,
                    request_deserializer=nidaqmx__pb2.ReadBinaryI16Request.FromString,
                    response_serializer=nidaqmx__pb2.ReadBinaryI16Response.SerializeToString,
            ),
            'ReadBinaryI32': grpc.unary_unary_rpc_method_handler(
                    servicer.ReadBinaryI32,
                    request_deserializer=nidaqmx__pb2.ReadBinaryI32Request.FromString,
                    response_serializer=nidaqmx__pb2.ReadBinaryI32Response.SerializeToString,
            ),
            'ReadBinaryU16': grpc.unary_unary_rpc_method_handler(
                    servicer.ReadBinaryU16,
                    request_deserializer=nidaqmx__pb2.ReadBinaryU16Request.FromString,
                    response_serializer=nidaqmx__pb2.ReadBinaryU16Response.SerializeToString,
            ),
            'ReadBinaryU32': grpc.unary_unary_rpc_method_handler(
                    servicer.ReadBinaryU32,
                    request_deserializer=nidaqmx__pb2.ReadBinaryU32Request.FromString,
                    response_serializer=nidaqmx__pb2.ReadBinaryU32Response.SerializeToString,
            ),
            'ReadCounterF64': grpc.unary_unary_rpc_method_handler(
                    servicer.ReadCounterF64,
                    request_deserializer=nidaqmx__pb2.ReadCounterF64Request.FromString,
                    response_serializer=nidaqmx__pb2.ReadCounterF64Response.SerializeToString,
            ),
            'ReadCounterF64Ex': grpc.unary_unary_rpc_method_handler(
                    servicer.ReadCounterF64Ex,
                    request_deserializer=nidaqmx__pb2.ReadCounterF64ExRequest.FromString,
                    response_serializer=nidaqmx__pb2.ReadCounterF64ExResponse.SerializeToString,
            ),
            'ReadCounterScalarF64': grpc.unary_unary_rpc_method_handler(
                    servicer.ReadCounterScalarF64,
                    request_deserializer=nidaqmx__pb2.ReadCounterScalarF64Request.FromString,
                    response_serializer=nidaqmx__pb2.ReadCounterScalarF64Response.SerializeToString,
            ),
            'ReadCounterScalarU32': grpc.unary_unary_rpc_method_handler(
                    servicer.ReadCounterScalarU32,
                    request_deserializer=nidaqmx__pb2.ReadCounterScalarU32Request.FromString,
                    response_serializer=nidaqmx__pb2.ReadCounterScalarU32Response.SerializeToString,
            ),
            'ReadCounterU32': grpc.unary_unary_rpc_method_handler(
                    servicer.ReadCounterU32,
                    request_deserializer=nidaqmx__pb2.ReadCounterU32Request.FromString,
                    response_serializer=nidaqmx__pb2.ReadCounterU32Response.SerializeToString,
            ),
            'ReadCounterU32Ex': grpc.unary_unary_rpc_method_handler(
                    servicer.ReadCounterU32Ex,
                    request_deserializer=nidaqmx__pb2.ReadCounterU32ExRequest.FromString,
                    response_serializer=nidaqmx__pb2.ReadCounterU32ExResponse.SerializeToString,
            ),
            'ReadCtrFreq': grpc.unary_unary_rpc_method_handler(
                    servicer.ReadCtrFreq,
                    request_deserializer=nidaqmx__pb2.ReadCtrFreqRequest.FromString,
                    response_serializer=nidaqmx__pb2.ReadCtrFreqResponse.SerializeToString,
            ),
            'ReadCtrFreqScalar': grpc.unary_unary_rpc_method_handler(
                    servicer.ReadCtrFreqScalar,
                    request_deserializer=nidaqmx__pb2.ReadCtrFreqScalarRequest.FromString,
                    response_serializer=nidaqmx__pb2.ReadCtrFreqScalarResponse.SerializeToString,
            ),
            'ReadCtrTicks': grpc.unary_unary_rpc_method_handler(
                    servicer.ReadCtrTicks,
                    request_deserializer=nidaqmx__pb2.ReadCtrTicksRequest.FromString,
                    response_serializer=nidaqmx__pb2.ReadCtrTicksResponse.SerializeToString,
            ),
            'ReadCtrTicksScalar': grpc.unary_unary_rpc_method_handler(
                    servicer.ReadCtrTicksScalar,
                    request_deserializer=nidaqmx__pb2.ReadCtrTicksScalarRequest.FromString,
                    response_serializer=nidaqmx__pb2.ReadCtrTicksScalarResponse.SerializeToString,
            ),
            'ReadCtrTime': grpc.unary_unary_rpc_method_handler(
                    servicer.ReadCtrTime,
                    request_deserializer=nidaqmx__pb2.ReadCtrTimeRequest.FromString,
                    response_serializer=nidaqmx__pb2.ReadCtrTimeResponse.SerializeToString,
            ),
            'ReadCtrTimeScalar': grpc.unary_unary_rpc_method_handler(
                    servicer.ReadCtrTimeScalar,
                    request_deserializer=nidaqmx__pb2.ReadCtrTimeScalarRequest.FromString,
                    response_serializer=nidaqmx__pb2.ReadCtrTimeScalarResponse.SerializeToString,
            ),
            'ReadDigitalLines': grpc.unary_unary_rpc_method_handler(
                    servicer.ReadDigitalLines,
                    request_deserializer=nidaqmx__pb2.ReadDigitalLinesRequest.FromString,
                    response_serializer=nidaqmx__pb2.ReadDigitalLinesResponse.SerializeToString,
            ),
            'ReadDigitalScalarU32': grpc.unary_unary_rpc_method_handler(
                    servicer.ReadDigitalScalarU32,
                    request_deserializer=nidaqmx__pb2.ReadDigitalScalarU32Request.FromString,
                    response_serializer=nidaqmx__pb2.ReadDigitalScalarU32Response.SerializeToString,
            ),
            'ReadDigitalU16': grpc.unary_unary_rpc_method_handler(
                    servicer.ReadDigitalU16,
                    request_deserializer=nidaqmx__pb2.ReadDigitalU16Request.FromString,
                    response_serializer=nidaqmx__pb2.ReadDigitalU16Response.SerializeToString,
            ),
            'ReadDigitalU32': grpc.unary_unary_rpc_method_handler(
                    servicer.ReadDigitalU32,
                    request_deserializer=nidaqmx__pb2.ReadDigitalU32Request.FromString,
                    response_serializer=nidaqmx__pb2.ReadDigitalU32Response.SerializeToString,
            ),
            'ReadDigitalU8': grpc.unary_unary_rpc_method_handler(
                    servicer.ReadDigitalU8,
                    request_deserializer=nidaqmx__pb2.ReadDigitalU8Request.FromString,
                    response_serializer=nidaqmx__pb2.ReadDigitalU8Response.SerializeToString,
            ),
            'ReadPowerBinaryI16': grpc.unary_unary_rpc_method_handler(
                    servicer.ReadPowerBinaryI16,
                    request_deserializer=nidaqmx__pb2.ReadPowerBinaryI16Request.FromString,
                    response_serializer=nidaqmx__pb2.ReadPowerBinaryI16Response.SerializeToString,
            ),
            'ReadPowerF64': grpc.unary_unary_rpc_method_handler(
                    servicer.ReadPowerF64,
                    request_deserializer=nidaqmx__pb2.ReadPowerF64Request.FromString,
                    response_serializer=nidaqmx__pb2.ReadPowerF64Response.SerializeToString,
            ),
            'ReadPowerScalarF64': grpc.unary_unary_rpc_method_handler(
                    servicer.ReadPowerScalarF64,
                    request_deserializer=nidaqmx__pb2.ReadPowerScalarF64Request.FromString,
                    response_serializer=nidaqmx__pb2.ReadPowerScalarF64Response.SerializeToString,
            ),
            'ReadRaw': grpc.unary_unary_rpc_method_handler(
                    servicer.ReadRaw,
                    request_deserializer=nidaqmx__pb2.ReadRawRequest.FromString,
                    response_serializer=nidaqmx__pb2.ReadRawResponse.SerializeToString,
            ),
            'RegisterDoneEvent': grpc.unary_stream_rpc_method_handler(
                    servicer.RegisterDoneEvent,
                    request_deserializer=nidaqmx__pb2.RegisterDoneEventRequest.FromString,
                    response_serializer=nidaqmx__pb2.RegisterDoneEventResponse.SerializeToString,
            ),
            'RegisterEveryNSamplesEvent': grpc.unary_stream_rpc_method_handler(
                    servicer.RegisterEveryNSamplesEvent,
                    request_deserializer=nidaqmx__pb2.RegisterEveryNSamplesEventRequest.FromString,
                    response_serializer=nidaqmx__pb2.RegisterEveryNSamplesEventResponse.SerializeToString,
            ),
            'RegisterSignalEvent': grpc.unary_stream_rpc_method_handler(
                    servicer.RegisterSignalEvent,
                    request_deserializer=nidaqmx__pb2.RegisterSignalEventRequest.FromString,
                    response_serializer=nidaqmx__pb2.RegisterSignalEventResponse.SerializeToString,
            ),
            'RemoveCDAQSyncConnection': grpc.unary_unary_rpc_method_handler(
                    servicer.RemoveCDAQSyncConnection,
                    request_deserializer=nidaqmx__pb2.RemoveCDAQSyncConnectionRequest.FromString,
                    response_serializer=nidaqmx__pb2.RemoveCDAQSyncConnectionResponse.SerializeToString,
            ),
            'ReserveNetworkDevice': grpc.unary_unary_rpc_method_handler(
                    servicer.ReserveNetworkDevice,
                    request_deserializer=nidaqmx__pb2.ReserveNetworkDeviceRequest.FromString,
                    response_serializer=nidaqmx__pb2.ReserveNetworkDeviceResponse.SerializeToString,
            ),
            'ResetBufferAttribute': grpc.unary_unary_rpc_method_handler(
                    servicer.ResetBufferAttribute,
                    request_deserializer=nidaqmx__pb2.ResetBufferAttributeRequest.FromString,
                    response_serializer=nidaqmx__pb2.ResetBufferAttributeResponse.SerializeToString,
            ),
            'ResetChanAttribute': grpc.unary_unary_rpc_method_handler(
                    servicer.ResetChanAttribute,
                    request_deserializer=nidaqmx__pb2.ResetChanAttributeRequest.FromString,
                    response_serializer=nidaqmx__pb2.ResetChanAttributeResponse.SerializeToString,
            ),
            'ResetDevice': grpc.unary_unary_rpc_method_handler(
                    servicer.ResetDevice,
                    request_deserializer=nidaqmx__pb2.ResetDeviceRequest.FromString,
                    response_serializer=nidaqmx__pb2.ResetDeviceResponse.SerializeToString,
            ),
            'ResetExportedSignalAttribute': grpc.unary_unary_rpc_method_handler(
                    servicer.ResetExportedSignalAttribute,
                    request_deserializer=nidaqmx__pb2.ResetExportedSignalAttributeRequest.FromString,
                    response_serializer=nidaqmx__pb2.ResetExportedSignalAttributeResponse.SerializeToString,
            ),
            'ResetReadAttribute': grpc.unary_unary_rpc_method_handler(
                    servicer.ResetReadAttribute,
                    request_deserializer=nidaqmx__pb2.ResetReadAttributeRequest.FromString,
                    response_serializer=nidaqmx__pb2.ResetReadAttributeResponse.SerializeToString,
            ),
            'ResetRealTimeAttribute': grpc.unary_unary_rpc_method_handler(
                    servicer.ResetRealTimeAttribute,
                    request_deserializer=nidaqmx__pb2.ResetRealTimeAttributeRequest.FromString,
                    response_serializer=nidaqmx__pb2.ResetRealTimeAttributeResponse.SerializeToString,
            ),
            'ResetTimingAttribute': grpc.unary_unary_rpc_method_handler(
                    servicer.ResetTimingAttribute,
                    request_deserializer=nidaqmx__pb2.ResetTimingAttributeRequest.FromString,
                    response_serializer=nidaqmx__pb2.ResetTimingAttributeResponse.SerializeToString,
            ),
            'ResetTimingAttributeEx': grpc.unary_unary_rpc_method_handler(
                    servicer.ResetTimingAttributeEx,
                    request_deserializer=nidaqmx__pb2.ResetTimingAttributeExRequest.FromString,
                    response_serializer=nidaqmx__pb2.ResetTimingAttributeExResponse.SerializeToString,
            ),
            'ResetTrigAttribute': grpc.unary_unary_rpc_method_handler(
                    servicer.ResetTrigAttribute,
                    request_deserializer=nidaqmx__pb2.ResetTrigAttributeRequest.FromString,
                    response_serializer=nidaqmx__pb2.ResetTrigAttributeResponse.SerializeToString,
            ),
            'ResetWatchdogAttribute': grpc.unary_unary_rpc_method_handler(
                    servicer.ResetWatchdogAttribute,
                    request_deserializer=nidaqmx__pb2.ResetWatchdogAttributeRequest.FromString,
                    response_serializer=nidaqmx__pb2.ResetWatchdogAttributeResponse.SerializeToString,
            ),
            'ResetWriteAttribute': grpc.unary_unary_rpc_method_handler(
                    servicer.ResetWriteAttribute,
                    request_deserializer=nidaqmx__pb2.ResetWriteAttributeRequest.FromString,
                    response_serializer=nidaqmx__pb2.ResetWriteAttributeResponse.SerializeToString,
            ),
            'RestoreLastExtCalConst': grpc.unary_unary_rpc_method_handler(
                    servicer.RestoreLastExtCalConst,
                    request_deserializer=nidaqmx__pb2.RestoreLastExtCalConstRequest.FromString,
                    response_serializer=nidaqmx__pb2.RestoreLastExtCalConstResponse.SerializeToString,
            ),
            'SaveGlobalChan': grpc.unary_unary_rpc_method_handler(
                    servicer.SaveGlobalChan,
                    request_deserializer=nidaqmx__pb2.SaveGlobalChanRequest.FromString,
                    response_serializer=nidaqmx__pb2.SaveGlobalChanResponse.SerializeToString,
            ),
            'SaveScale': grpc.unary_unary_rpc_method_handler(
                    servicer.SaveScale,
                    request_deserializer=nidaqmx__pb2.SaveScaleRequest.FromString,
                    response_serializer=nidaqmx__pb2.SaveScaleResponse.SerializeToString,
            ),
            'SaveTask': grpc.unary_unary_rpc_method_handler(
                    servicer.SaveTask,
                    request_deserializer=nidaqmx__pb2.SaveTaskRequest.FromString,
                    response_serializer=nidaqmx__pb2.SaveTaskResponse.SerializeToString,
            ),
            'SelfCal': grpc.unary_unary_rpc_method_handler(
                    servicer.SelfCal,
                    request_deserializer=nidaqmx__pb2.SelfCalRequest.FromString,
                    response_serializer=nidaqmx__pb2.SelfCalResponse.SerializeToString,
            ),
            'SelfTestDevice': grpc.unary_unary_rpc_method_handler(
                    servicer.SelfTestDevice,
                    request_deserializer=nidaqmx__pb2.SelfTestDeviceRequest.FromString,
                    response_serializer=nidaqmx__pb2.SelfTestDeviceResponse.SerializeToString,
            ),
            'SetAIChanCalCalDate': grpc.unary_unary_rpc_method_handler(
                    servicer.SetAIChanCalCalDate,
                    request_deserializer=nidaqmx__pb2.SetAIChanCalCalDateRequest.FromString,
                    response_serializer=nidaqmx__pb2.SetAIChanCalCalDateResponse.SerializeToString,
            ),
            'SetAIChanCalExpDate': grpc.unary_unary_rpc_method_handler(
                    servicer.SetAIChanCalExpDate,
                    request_deserializer=nidaqmx__pb2.SetAIChanCalExpDateRequest.FromString,
                    response_serializer=nidaqmx__pb2.SetAIChanCalExpDateResponse.SerializeToString,
            ),
            'SetAnalogPowerUpStates': grpc.unary_unary_rpc_method_handler(
                    servicer.SetAnalogPowerUpStates,
                    request_deserializer=nidaqmx__pb2.SetAnalogPowerUpStatesRequest.FromString,
                    response_serializer=nidaqmx__pb2.SetAnalogPowerUpStatesResponse.SerializeToString,
            ),
            'SetAnalogPowerUpStatesWithOutputType': grpc.unary_unary_rpc_method_handler(
                    servicer.SetAnalogPowerUpStatesWithOutputType,
                    request_deserializer=nidaqmx__pb2.SetAnalogPowerUpStatesWithOutputTypeRequest.FromString,
                    response_serializer=nidaqmx__pb2.SetAnalogPowerUpStatesWithOutputTypeResponse.SerializeToString,
            ),
            'SetArmStartTrigTrigWhen': grpc.unary_unary_rpc_method_handler(
                    servicer.SetArmStartTrigTrigWhen,
                    request_deserializer=nidaqmx__pb2.SetArmStartTrigTrigWhenRequest.FromString,
                    response_serializer=nidaqmx__pb2.SetArmStartTrigTrigWhenResponse.SerializeToString,
            ),
            'SetBufferAttributeUInt32': grpc.unary_unary_rpc_method_handler(
                    servicer.SetBufferAttributeUInt32,
                    request_deserializer=nidaqmx__pb2.SetBufferAttributeUInt32Request.FromString,
                    response_serializer=nidaqmx__pb2.SetBufferAttributeUInt32Response.SerializeToString,
            ),
            'SetCalInfoAttributeBool': grpc.unary_unary_rpc_method_handler(
                    servicer.SetCalInfoAttributeBool,
                    request_deserializer=nidaqmx__pb2.SetCalInfoAttributeBoolRequest.FromString,
                    response_serializer=nidaqmx__pb2.SetCalInfoAttributeBoolResponse.SerializeToString,
            ),
            'SetCalInfoAttributeDouble': grpc.unary_unary_rpc_method_handler(
                    servicer.SetCalInfoAttributeDouble,
                    request_deserializer=nidaqmx__pb2.SetCalInfoAttributeDoubleRequest.FromString,
                    response_serializer=nidaqmx__pb2.SetCalInfoAttributeDoubleResponse.SerializeToString,
            ),
            'SetCalInfoAttributeString': grpc.unary_unary_rpc_method_handler(
                    servicer.SetCalInfoAttributeString,
                    request_deserializer=nidaqmx__pb2.SetCalInfoAttributeStringRequest.FromString,
                    response_serializer=nidaqmx__pb2.SetCalInfoAttributeStringResponse.SerializeToString,
            ),
            'SetCalInfoAttributeUInt32': grpc.unary_unary_rpc_method_handler(
                    servicer.SetCalInfoAttributeUInt32,
                    request_deserializer=nidaqmx__pb2.SetCalInfoAttributeUInt32Request.FromString,
                    response_serializer=nidaqmx__pb2.SetCalInfoAttributeUInt32Response.SerializeToString,
            ),
            'SetChanAttributeBool': grpc.unary_unary_rpc_method_handler(
                    servicer.SetChanAttributeBool,
                    request_deserializer=nidaqmx__pb2.SetChanAttributeBoolRequest.FromString,
                    response_serializer=nidaqmx__pb2.SetChanAttributeBoolResponse.SerializeToString,
            ),
            'SetChanAttributeDouble': grpc.unary_unary_rpc_method_handler(
                    servicer.SetChanAttributeDouble,
                    request_deserializer=nidaqmx__pb2.SetChanAttributeDoubleRequest.FromString,
                    response_serializer=nidaqmx__pb2.SetChanAttributeDoubleResponse.SerializeToString,
            ),
            'SetChanAttributeDoubleArray': grpc.unary_unary_rpc_method_handler(
                    servicer.SetChanAttributeDoubleArray,
                    request_deserializer=nidaqmx__pb2.SetChanAttributeDoubleArrayRequest.FromString,
                    response_serializer=nidaqmx__pb2.SetChanAttributeDoubleArrayResponse.SerializeToString,
            ),
            'SetChanAttributeInt32': grpc.unary_unary_rpc_method_handler(
                    servicer.SetChanAttributeInt32,
                    request_deserializer=nidaqmx__pb2.SetChanAttributeInt32Request.FromString,
                    response_serializer=nidaqmx__pb2.SetChanAttributeInt32Response.SerializeToString,
            ),
            'SetChanAttributeString': grpc.unary_unary_rpc_method_handler(
                    servicer.SetChanAttributeString,
                    request_deserializer=nidaqmx__pb2.SetChanAttributeStringRequest.FromString,
                    response_serializer=nidaqmx__pb2.SetChanAttributeStringResponse.SerializeToString,
            ),
            'SetChanAttributeUInt32': grpc.unary_unary_rpc_method_handler(
                    servicer.SetChanAttributeUInt32,
                    request_deserializer=nidaqmx__pb2.SetChanAttributeUInt32Request.FromString,
                    response_serializer=nidaqmx__pb2.SetChanAttributeUInt32Response.SerializeToString,
            ),
            'SetDigitalLogicFamilyPowerUpState': grpc.unary_unary_rpc_method_handler(
                    servicer.SetDigitalLogicFamilyPowerUpState,
                    request_deserializer=nidaqmx__pb2.SetDigitalLogicFamilyPowerUpStateRequest.FromString,
                    response_serializer=nidaqmx__pb2.SetDigitalLogicFamilyPowerUpStateResponse.SerializeToString,
            ),
            'SetDigitalPowerUpStates': grpc.unary_unary_rpc_method_handler(
                    servicer.SetDigitalPowerUpStates,
                    request_deserializer=nidaqmx__pb2.SetDigitalPowerUpStatesRequest.FromString,
                    response_serializer=nidaqmx__pb2.SetDigitalPowerUpStatesResponse.SerializeToString,
            ),
            'SetDigitalPullUpPullDownStates': grpc.unary_unary_rpc_method_handler(
                    servicer.SetDigitalPullUpPullDownStates,
                    request_deserializer=nidaqmx__pb2.SetDigitalPullUpPullDownStatesRequest.FromString,
                    response_serializer=nidaqmx__pb2.SetDigitalPullUpPullDownStatesResponse.SerializeToString,
            ),
            'SetExportedSignalAttributeBool': grpc.unary_unary_rpc_method_handler(
                    servicer.SetExportedSignalAttributeBool,
                    request_deserializer=nidaqmx__pb2.SetExportedSignalAttributeBoolRequest.FromString,
                    response_serializer=nidaqmx__pb2.SetExportedSignalAttributeBoolResponse.SerializeToString,
            ),
            'SetExportedSignalAttributeDouble': grpc.unary_unary_rpc_method_handler(
                    servicer.SetExportedSignalAttributeDouble,
                    request_deserializer=nidaqmx__pb2.SetExportedSignalAttributeDoubleRequest.FromString,
                    response_serializer=nidaqmx__pb2.SetExportedSignalAttributeDoubleResponse.SerializeToString,
            ),
            'SetExportedSignalAttributeInt32': grpc.unary_unary_rpc_method_handler(
                    servicer.SetExportedSignalAttributeInt32,
                    request_deserializer=nidaqmx__pb2.SetExportedSignalAttributeInt32Request.FromString,
                    response_serializer=nidaqmx__pb2.SetExportedSignalAttributeInt32Response.SerializeToString,
            ),
            'SetExportedSignalAttributeString': grpc.unary_unary_rpc_method_handler(
                    servicer.SetExportedSignalAttributeString,
                    request_deserializer=nidaqmx__pb2.SetExportedSignalAttributeStringRequest.FromString,
                    response_serializer=nidaqmx__pb2.SetExportedSignalAttributeStringResponse.SerializeToString,
            ),
            'SetExportedSignalAttributeUInt32': grpc.unary_unary_rpc_method_handler(
                    servicer.SetExportedSignalAttributeUInt32,
                    request_deserializer=nidaqmx__pb2.SetExportedSignalAttributeUInt32Request.FromString,
                    response_serializer=nidaqmx__pb2.SetExportedSignalAttributeUInt32Response.SerializeToString,
            ),
            'SetFirstSampClkWhen': grpc.unary_unary_rpc_method_handler(
                    servicer.SetFirstSampClkWhen,
                    request_deserializer=nidaqmx__pb2.SetFirstSampClkWhenRequest.FromString,
                    response_serializer=nidaqmx__pb2.SetFirstSampClkWhenResponse.SerializeToString,
            ),
            'SetReadAttributeBool': grpc.unary_unary_rpc_method_handler(
                    servicer.SetReadAttributeBool,
                    request_deserializer=nidaqmx__pb2.SetReadAttributeBoolRequest.FromString,
                    response_serializer=nidaqmx__pb2.SetReadAttributeBoolResponse.SerializeToString,
            ),
            'SetReadAttributeDouble': grpc.unary_unary_rpc_method_handler(
                    servicer.SetReadAttributeDouble,
                    request_deserializer=nidaqmx__pb2.SetReadAttributeDoubleRequest.FromString,
                    response_serializer=nidaqmx__pb2.SetReadAttributeDoubleResponse.SerializeToString,
            ),
            'SetReadAttributeInt32': grpc.unary_unary_rpc_method_handler(
                    servicer.SetReadAttributeInt32,
                    request_deserializer=nidaqmx__pb2.SetReadAttributeInt32Request.FromString,
                    response_serializer=nidaqmx__pb2.SetReadAttributeInt32Response.SerializeToString,
            ),
            'SetReadAttributeString': grpc.unary_unary_rpc_method_handler(
                    servicer.SetReadAttributeString,
                    request_deserializer=nidaqmx__pb2.SetReadAttributeStringRequest.FromString,
                    response_serializer=nidaqmx__pb2.SetReadAttributeStringResponse.SerializeToString,
            ),
            'SetReadAttributeUInt32': grpc.unary_unary_rpc_method_handler(
                    servicer.SetReadAttributeUInt32,
                    request_deserializer=nidaqmx__pb2.SetReadAttributeUInt32Request.FromString,
                    response_serializer=nidaqmx__pb2.SetReadAttributeUInt32Response.SerializeToString,
            ),
            'SetReadAttributeUInt64': grpc.unary_unary_rpc_method_handler(
                    servicer.SetReadAttributeUInt64,
                    request_deserializer=nidaqmx__pb2.SetReadAttributeUInt64Request.FromString,
                    response_serializer=nidaqmx__pb2.SetReadAttributeUInt64Response.SerializeToString,
            ),
            'SetRealTimeAttributeBool': grpc.unary_unary_rpc_method_handler(
                    servicer.SetRealTimeAttributeBool,
                    request_deserializer=nidaqmx__pb2.SetRealTimeAttributeBoolRequest.FromString,
                    response_serializer=nidaqmx__pb2.SetRealTimeAttributeBoolResponse.SerializeToString,
            ),
            'SetRealTimeAttributeInt32': grpc.unary_unary_rpc_method_handler(
                    servicer.SetRealTimeAttributeInt32,
                    request_deserializer=nidaqmx__pb2.SetRealTimeAttributeInt32Request.FromString,
                    response_serializer=nidaqmx__pb2.SetRealTimeAttributeInt32Response.SerializeToString,
            ),
            'SetRealTimeAttributeUInt32': grpc.unary_unary_rpc_method_handler(
                    servicer.SetRealTimeAttributeUInt32,
                    request_deserializer=nidaqmx__pb2.SetRealTimeAttributeUInt32Request.FromString,
                    response_serializer=nidaqmx__pb2.SetRealTimeAttributeUInt32Response.SerializeToString,
            ),
            'SetScaleAttributeDouble': grpc.unary_unary_rpc_method_handler(
                    servicer.SetScaleAttributeDouble,
                    request_deserializer=nidaqmx__pb2.SetScaleAttributeDoubleRequest.FromString,
                    response_serializer=nidaqmx__pb2.SetScaleAttributeDoubleResponse.SerializeToString,
            ),
            'SetScaleAttributeDoubleArray': grpc.unary_unary_rpc_method_handler(
                    servicer.SetScaleAttributeDoubleArray,
                    request_deserializer=nidaqmx__pb2.SetScaleAttributeDoubleArrayRequest.FromString,
                    response_serializer=nidaqmx__pb2.SetScaleAttributeDoubleArrayResponse.SerializeToString,
            ),
            'SetScaleAttributeInt32': grpc.unary_unary_rpc_method_handler(
                    servicer.SetScaleAttributeInt32,
                    request_deserializer=nidaqmx__pb2.SetScaleAttributeInt32Request.FromString,
                    response_serializer=nidaqmx__pb2.SetScaleAttributeInt32Response.SerializeToString,
            ),
            'SetScaleAttributeString': grpc.unary_unary_rpc_method_handler(
                    servicer.SetScaleAttributeString,
                    request_deserializer=nidaqmx__pb2.SetScaleAttributeStringRequest.FromString,
                    response_serializer=nidaqmx__pb2.SetScaleAttributeStringResponse.SerializeToString,
            ),
            'SetStartTrigTrigWhen': grpc.unary_unary_rpc_method_handler(
                    servicer.SetStartTrigTrigWhen,
                    request_deserializer=nidaqmx__pb2.SetStartTrigTrigWhenRequest.FromString,
                    response_serializer=nidaqmx__pb2.SetStartTrigTrigWhenResponse.SerializeToString,
            ),
            'SetSyncPulseTimeWhen': grpc.unary_unary_rpc_method_handler(
                    servicer.SetSyncPulseTimeWhen,
                    request_deserializer=nidaqmx__pb2.SetSyncPulseTimeWhenRequest.FromString,
                    response_serializer=nidaqmx__pb2.SetSyncPulseTimeWhenResponse.SerializeToString,
            ),
            'SetTimingAttributeBool': grpc.unary_unary_rpc_method_handler(
                    servicer.SetTimingAttributeBool,
                    request_deserializer=nidaqmx__pb2.SetTimingAttributeBoolRequest.FromString,
                    response_serializer=nidaqmx__pb2.SetTimingAttributeBoolResponse.SerializeToString,
            ),
            'SetTimingAttributeDouble': grpc.unary_unary_rpc_method_handler(
                    servicer.SetTimingAttributeDouble,
                    request_deserializer=nidaqmx__pb2.SetTimingAttributeDoubleRequest.FromString,
                    response_serializer=nidaqmx__pb2.SetTimingAttributeDoubleResponse.SerializeToString,
            ),
            'SetTimingAttributeExBool': grpc.unary_unary_rpc_method_handler(
                    servicer.SetTimingAttributeExBool,
                    request_deserializer=nidaqmx__pb2.SetTimingAttributeExBoolRequest.FromString,
                    response_serializer=nidaqmx__pb2.SetTimingAttributeExBoolResponse.SerializeToString,
            ),
            'SetTimingAttributeExDouble': grpc.unary_unary_rpc_method_handler(
                    servicer.SetTimingAttributeExDouble,
                    request_deserializer=nidaqmx__pb2.SetTimingAttributeExDoubleRequest.FromString,
                    response_serializer=nidaqmx__pb2.SetTimingAttributeExDoubleResponse.SerializeToString,
            ),
            'SetTimingAttributeExInt32': grpc.unary_unary_rpc_method_handler(
                    servicer.SetTimingAttributeExInt32,
                    request_deserializer=nidaqmx__pb2.SetTimingAttributeExInt32Request.FromString,
                    response_serializer=nidaqmx__pb2.SetTimingAttributeExInt32Response.SerializeToString,
            ),
            'SetTimingAttributeExString': grpc.unary_unary_rpc_method_handler(
                    servicer.SetTimingAttributeExString,
                    request_deserializer=nidaqmx__pb2.SetTimingAttributeExStringRequest.FromString,
                    response_serializer=nidaqmx__pb2.SetTimingAttributeExStringResponse.SerializeToString,
            ),
            'SetTimingAttributeExTimestamp': grpc.unary_unary_rpc_method_handler(
                    servicer.SetTimingAttributeExTimestamp,
                    request_deserializer=nidaqmx__pb2.SetTimingAttributeExTimestampRequest.FromString,
                    response_serializer=nidaqmx__pb2.SetTimingAttributeExTimestampResponse.SerializeToString,
            ),
            'SetTimingAttributeExUInt32': grpc.unary_unary_rpc_method_handler(
                    servicer.SetTimingAttributeExUInt32,
                    request_deserializer=nidaqmx__pb2.SetTimingAttributeExUInt32Request.FromString,
                    response_serializer=nidaqmx__pb2.SetTimingAttributeExUInt32Response.SerializeToString,
            ),
            'SetTimingAttributeExUInt64': grpc.unary_unary_rpc_method_handler(
                    servicer.SetTimingAttributeExUInt64,
                    request_deserializer=nidaqmx__pb2.SetTimingAttributeExUInt64Request.FromString,
                    response_serializer=nidaqmx__pb2.SetTimingAttributeExUInt64Response.SerializeToString,
            ),
            'SetTimingAttributeInt32': grpc.unary_unary_rpc_method_handler(
                    servicer.SetTimingAttributeInt32,
                    request_deserializer=nidaqmx__pb2.SetTimingAttributeInt32Request.FromString,
                    response_serializer=nidaqmx__pb2.SetTimingAttributeInt32Response.SerializeToString,
            ),
            'SetTimingAttributeString': grpc.unary_unary_rpc_method_handler(
                    servicer.SetTimingAttributeString,
                    request_deserializer=nidaqmx__pb2.SetTimingAttributeStringRequest.FromString,
                    response_serializer=nidaqmx__pb2.SetTimingAttributeStringResponse.SerializeToString,
            ),
            'SetTimingAttributeTimestamp': grpc.unary_unary_rpc_method_handler(
                    servicer.SetTimingAttributeTimestamp,
                    request_deserializer=nidaqmx__pb2.SetTimingAttributeTimestampRequest.FromString,
                    response_serializer=nidaqmx__pb2.SetTimingAttributeTimestampResponse.SerializeToString,
            ),
            'SetTimingAttributeUInt32': grpc.unary_unary_rpc_method_handler(
                    servicer.SetTimingAttributeUInt32,
                    request_deserializer=nidaqmx__pb2.SetTimingAttributeUInt32Request.FromString,
                    response_serializer=nidaqmx__pb2.SetTimingAttributeUInt32Response.SerializeToString,
            ),
            'SetTimingAttributeUInt64': grpc.unary_unary_rpc_method_handler(
                    servicer.SetTimingAttributeUInt64,
                    request_deserializer=nidaqmx__pb2.SetTimingAttributeUInt64Request.FromString,
                    response_serializer=nidaqmx__pb2.SetTimingAttributeUInt64Response.SerializeToString,
            ),
            'SetTrigAttributeBool': grpc.unary_unary_rpc_method_handler(
                    servicer.SetTrigAttributeBool,
                    request_deserializer=nidaqmx__pb2.SetTrigAttributeBoolRequest.FromString,
                    response_serializer=nidaqmx__pb2.SetTrigAttributeBoolResponse.SerializeToString,
            ),
            'SetTrigAttributeDouble': grpc.unary_unary_rpc_method_handler(
                    servicer.SetTrigAttributeDouble,
                    request_deserializer=nidaqmx__pb2.SetTrigAttributeDoubleRequest.FromString,
                    response_serializer=nidaqmx__pb2.SetTrigAttributeDoubleResponse.SerializeToString,
            ),
            'SetTrigAttributeDoubleArray': grpc.unary_unary_rpc_method_handler(
                    servicer.SetTrigAttributeDoubleArray,
                    request_deserializer=nidaqmx__pb2.SetTrigAttributeDoubleArrayRequest.FromString,
                    response_serializer=nidaqmx__pb2.SetTrigAttributeDoubleArrayResponse.SerializeToString,
            ),
            'SetTrigAttributeInt32': grpc.unary_unary_rpc_method_handler(
                    servicer.SetTrigAttributeInt32,
                    request_deserializer=nidaqmx__pb2.SetTrigAttributeInt32Request.FromString,
                    response_serializer=nidaqmx__pb2.SetTrigAttributeInt32Response.SerializeToString,
            ),
            'SetTrigAttributeInt32Array': grpc.unary_unary_rpc_method_handler(
                    servicer.SetTrigAttributeInt32Array,
                    request_deserializer=nidaqmx__pb2.SetTrigAttributeInt32ArrayRequest.FromString,
                    response_serializer=nidaqmx__pb2.SetTrigAttributeInt32ArrayResponse.SerializeToString,
            ),
            'SetTrigAttributeString': grpc.unary_unary_rpc_method_handler(
                    servicer.SetTrigAttributeString,
                    request_deserializer=nidaqmx__pb2.SetTrigAttributeStringRequest.FromString,
                    response_serializer=nidaqmx__pb2.SetTrigAttributeStringResponse.SerializeToString,
            ),
            'SetTrigAttributeTimestamp': grpc.unary_unary_rpc_method_handler(
                    servicer.SetTrigAttributeTimestamp,
                    request_deserializer=nidaqmx__pb2.SetTrigAttributeTimestampRequest.FromString,
                    response_serializer=nidaqmx__pb2.SetTrigAttributeTimestampResponse.SerializeToString,
            ),
            'SetTrigAttributeUInt32': grpc.unary_unary_rpc_method_handler(
                    servicer.SetTrigAttributeUInt32,
                    request_deserializer=nidaqmx__pb2.SetTrigAttributeUInt32Request.FromString,
                    response_serializer=nidaqmx__pb2.SetTrigAttributeUInt32Response.SerializeToString,
            ),
            'SetWatchdogAttributeBool': grpc.unary_unary_rpc_method_handler(
                    servicer.SetWatchdogAttributeBool,
                    request_deserializer=nidaqmx__pb2.SetWatchdogAttributeBoolRequest.FromString,
                    response_serializer=nidaqmx__pb2.SetWatchdogAttributeBoolResponse.SerializeToString,
            ),
            'SetWatchdogAttributeDouble': grpc.unary_unary_rpc_method_handler(
                    servicer.SetWatchdogAttributeDouble,
                    request_deserializer=nidaqmx__pb2.SetWatchdogAttributeDoubleRequest.FromString,
                    response_serializer=nidaqmx__pb2.SetWatchdogAttributeDoubleResponse.SerializeToString,
            ),
            'SetWatchdogAttributeInt32': grpc.unary_unary_rpc_method_handler(
                    servicer.SetWatchdogAttributeInt32,
                    request_deserializer=nidaqmx__pb2.SetWatchdogAttributeInt32Request.FromString,
                    response_serializer=nidaqmx__pb2.SetWatchdogAttributeInt32Response.SerializeToString,
            ),
            'SetWatchdogAttributeString': grpc.unary_unary_rpc_method_handler(
                    servicer.SetWatchdogAttributeString,
                    request_deserializer=nidaqmx__pb2.SetWatchdogAttributeStringRequest.FromString,
                    response_serializer=nidaqmx__pb2.SetWatchdogAttributeStringResponse.SerializeToString,
            ),
            'SetWriteAttributeBool': grpc.unary_unary_rpc_method_handler(
                    servicer.SetWriteAttributeBool,
                    request_deserializer=nidaqmx__pb2.SetWriteAttributeBoolRequest.FromString,
                    response_serializer=nidaqmx__pb2.SetWriteAttributeBoolResponse.SerializeToString,
            ),
            'SetWriteAttributeDouble': grpc.unary_unary_rpc_method_handler(
                    servicer.SetWriteAttributeDouble,
                    request_deserializer=nidaqmx__pb2.SetWriteAttributeDoubleRequest.FromString,
                    response_serializer=nidaqmx__pb2.SetWriteAttributeDoubleResponse.SerializeToString,
            ),
            'SetWriteAttributeInt32': grpc.unary_unary_rpc_method_handler(
                    servicer.SetWriteAttributeInt32,
                    request_deserializer=nidaqmx__pb2.SetWriteAttributeInt32Request.FromString,
                    response_serializer=nidaqmx__pb2.SetWriteAttributeInt32Response.SerializeToString,
            ),
            'SetWriteAttributeString': grpc.unary_unary_rpc_method_handler(
                    servicer.SetWriteAttributeString,
                    request_deserializer=nidaqmx__pb2.SetWriteAttributeStringRequest.FromString,
                    response_serializer=nidaqmx__pb2.SetWriteAttributeStringResponse.SerializeToString,
            ),
            'SetWriteAttributeUInt32': grpc.unary_unary_rpc_method_handler(
                    servicer.SetWriteAttributeUInt32,
                    request_deserializer=nidaqmx__pb2.SetWriteAttributeUInt32Request.FromString,
                    response_serializer=nidaqmx__pb2.SetWriteAttributeUInt32Response.SerializeToString,
            ),
            'SetWriteAttributeUInt64': grpc.unary_unary_rpc_method_handler(
                    servicer.SetWriteAttributeUInt64,
                    request_deserializer=nidaqmx__pb2.SetWriteAttributeUInt64Request.FromString,
                    response_serializer=nidaqmx__pb2.SetWriteAttributeUInt64Response.SerializeToString,
            ),
            'StartNewFile': grpc.unary_unary_rpc_method_handler(
                    servicer.StartNewFile,
                    request_deserializer=nidaqmx__pb2.StartNewFileRequest.FromString,
                    response_serializer=nidaqmx__pb2.StartNewFileResponse.SerializeToString,
            ),
            'StartTask': grpc.unary_unary_rpc_method_handler(
                    servicer.StartTask,
                    request_deserializer=nidaqmx__pb2.StartTaskRequest.FromString,
                    response_serializer=nidaqmx__pb2.StartTaskResponse.SerializeToString,
            ),
            'StopTask': grpc.unary_unary_rpc_method_handler(
                    servicer.StopTask,
                    request_deserializer=nidaqmx__pb2.StopTaskRequest.FromString,
                    response_serializer=nidaqmx__pb2.StopTaskResponse.SerializeToString,
            ),
            'TaskControl': grpc.unary_unary_rpc_method_handler(
                    servicer.TaskControl,
                    request_deserializer=nidaqmx__pb2.TaskControlRequest.FromString,
                    response_serializer=nidaqmx__pb2.TaskControlResponse.SerializeToString,
            ),
            'TristateOutputTerm': grpc.unary_unary_rpc_method_handler(
                    servicer.TristateOutputTerm,
                    request_deserializer=nidaqmx__pb2.TristateOutputTermRequest.FromString,
                    response_serializer=nidaqmx__pb2.TristateOutputTermResponse.SerializeToString,
            ),
            'UnregisterDoneEvent': grpc.unary_unary_rpc_method_handler(
                    servicer.UnregisterDoneEvent,
                    request_deserializer=nidaqmx__pb2.UnregisterDoneEventRequest.FromString,
                    response_serializer=nidaqmx__pb2.UnregisterDoneEventResponse.SerializeToString,
            ),
            'UnregisterEveryNSamplesEvent': grpc.unary_unary_rpc_method_handler(
                    servicer.UnregisterEveryNSamplesEvent,
                    request_deserializer=nidaqmx__pb2.UnregisterEveryNSamplesEventRequest.FromString,
                    response_serializer=nidaqmx__pb2.UnregisterEveryNSamplesEventResponse.SerializeToString,
            ),
            'UnregisterSignalEvent': grpc.unary_unary_rpc_method_handler(
                    servicer.UnregisterSignalEvent,
                    request_deserializer=nidaqmx__pb2.UnregisterSignalEventRequest.FromString,
                    response_serializer=nidaqmx__pb2.UnregisterSignalEventResponse.SerializeToString,
            ),
            'UnreserveNetworkDevice': grpc.unary_unary_rpc_method_handler(
                    servicer.UnreserveNetworkDevice,
                    request_deserializer=nidaqmx__pb2.UnreserveNetworkDeviceRequest.FromString,
                    response_serializer=nidaqmx__pb2.UnreserveNetworkDeviceResponse.SerializeToString,
            ),
            'WaitForNextSampleClock': grpc.unary_unary_rpc_method_handler(
                    servicer.WaitForNextSampleClock,
                    request_deserializer=nidaqmx__pb2.WaitForNextSampleClockRequest.FromString,
                    response_serializer=nidaqmx__pb2.WaitForNextSampleClockResponse.SerializeToString,
            ),
            'WaitForValidTimestamp': grpc.unary_unary_rpc_method_handler(
                    servicer.WaitForValidTimestamp,
                    request_deserializer=nidaqmx__pb2.WaitForValidTimestampRequest.FromString,
                    response_serializer=nidaqmx__pb2.WaitForValidTimestampResponse.SerializeToString,
            ),
            'WaitUntilTaskDone': grpc.unary_unary_rpc_method_handler(
                    servicer.WaitUntilTaskDone,
                    request_deserializer=nidaqmx__pb2.WaitUntilTaskDoneRequest.FromString,
                    response_serializer=nidaqmx__pb2.WaitUntilTaskDoneResponse.SerializeToString,
            ),
            'WriteAnalogF64': grpc.unary_unary_rpc_method_handler(
                    servicer.WriteAnalogF64,
                    request_deserializer=nidaqmx__pb2.WriteAnalogF64Request.FromString,
                    response_serializer=nidaqmx__pb2.WriteAnalogF64Response.SerializeToString,
            ),
            'WriteAnalogScalarF64': grpc.unary_unary_rpc_method_handler(
                    servicer.WriteAnalogScalarF64,
                    request_deserializer=nidaqmx__pb2.WriteAnalogScalarF64Request.FromString,
                    response_serializer=nidaqmx__pb2.WriteAnalogScalarF64Response.SerializeToString,
            ),
            'WriteBinaryI16': grpc.unary_unary_rpc_method_handler(
                    servicer.WriteBinaryI16,
                    request_deserializer=nidaqmx__pb2.WriteBinaryI16Request.FromString,
                    response_serializer=nidaqmx__pb2.WriteBinaryI16Response.SerializeToString,
            ),
            'WriteBinaryI32': grpc.unary_unary_rpc_method_handler(
                    servicer.WriteBinaryI32,
                    request_deserializer=nidaqmx__pb2.WriteBinaryI32Request.FromString,
                    response_serializer=nidaqmx__pb2.WriteBinaryI32Response.SerializeToString,
            ),
            'WriteBinaryU16': grpc.unary_unary_rpc_method_handler(
                    servicer.WriteBinaryU16,
                    request_deserializer=nidaqmx__pb2.WriteBinaryU16Request.FromString,
                    response_serializer=nidaqmx__pb2.WriteBinaryU16Response.SerializeToString,
            ),
            'WriteBinaryU32': grpc.unary_unary_rpc_method_handler(
                    servicer.WriteBinaryU32,
                    request_deserializer=nidaqmx__pb2.WriteBinaryU32Request.FromString,
                    response_serializer=nidaqmx__pb2.WriteBinaryU32Response.SerializeToString,
            ),
            'WriteCtrFreq': grpc.unary_unary_rpc_method_handler(
                    servicer.WriteCtrFreq,
                    request_deserializer=nidaqmx__pb2.WriteCtrFreqRequest.FromString,
                    response_serializer=nidaqmx__pb2.WriteCtrFreqResponse.SerializeToString,
            ),
            'WriteCtrFreqScalar': grpc.unary_unary_rpc_method_handler(
                    servicer.WriteCtrFreqScalar,
                    request_deserializer=nidaqmx__pb2.WriteCtrFreqScalarRequest.FromString,
                    response_serializer=nidaqmx__pb2.WriteCtrFreqScalarResponse.SerializeToString,
            ),
            'WriteCtrTicks': grpc.unary_unary_rpc_method_handler(
                    servicer.WriteCtrTicks,
                    request_deserializer=nidaqmx__pb2.WriteCtrTicksRequest.FromString,
                    response_serializer=nidaqmx__pb2.WriteCtrTicksResponse.SerializeToString,
            ),
            'WriteCtrTicksScalar': grpc.unary_unary_rpc_method_handler(
                    servicer.WriteCtrTicksScalar,
                    request_deserializer=nidaqmx__pb2.WriteCtrTicksScalarRequest.FromString,
                    response_serializer=nidaqmx__pb2.WriteCtrTicksScalarResponse.SerializeToString,
            ),
            'WriteCtrTime': grpc.unary_unary_rpc_method_handler(
                    servicer.WriteCtrTime,
                    request_deserializer=nidaqmx__pb2.WriteCtrTimeRequest.FromString,
                    response_serializer=nidaqmx__pb2.WriteCtrTimeResponse.SerializeToString,
            ),
            'WriteCtrTimeScalar': grpc.unary_unary_rpc_method_handler(
                    servicer.WriteCtrTimeScalar,
                    request_deserializer=nidaqmx__pb2.WriteCtrTimeScalarRequest.FromString,
                    response_serializer=nidaqmx__pb2.WriteCtrTimeScalarResponse.SerializeToString,
            ),
            'WriteDigitalLines': grpc.unary_unary_rpc_method_handler(
                    servicer.WriteDigitalLines,
                    request_deserializer=nidaqmx__pb2.WriteDigitalLinesRequest.FromString,
                    response_serializer=nidaqmx__pb2.WriteDigitalLinesResponse.SerializeToString,
            ),
            'WriteDigitalScalarU32': grpc.unary_unary_rpc_method_handler(
                    servicer.WriteDigitalScalarU32,
                    request_deserializer=nidaqmx__pb2.WriteDigitalScalarU32Request.FromString,
                    response_serializer=nidaqmx__pb2.WriteDigitalScalarU32Response.SerializeToString,
            ),
            'WriteDigitalU16': grpc.unary_unary_rpc_method_handler(
                    servicer.WriteDigitalU16,
                    request_deserializer=nidaqmx__pb2.WriteDigitalU16Request.FromString,
                    response_serializer=nidaqmx__pb2.WriteDigitalU16Response.SerializeToString,
            ),
            'WriteDigitalU32': grpc.unary_unary_rpc_method_handler(
                    servicer.WriteDigitalU32,
                    request_deserializer=nidaqmx__pb2.WriteDigitalU32Request.FromString,
                    response_serializer=nidaqmx__pb2.WriteDigitalU32Response.SerializeToString,
            ),
            'WriteDigitalU8': grpc.unary_unary_rpc_method_handler(
                    servicer.WriteDigitalU8,
                    request_deserializer=nidaqmx__pb2.WriteDigitalU8Request.FromString,
                    response_serializer=nidaqmx__pb2.WriteDigitalU8Response.SerializeToString,
            ),
            'WriteRaw': grpc.unary_unary_rpc_method_handler(
                    servicer.WriteRaw,
                    request_deserializer=nidaqmx__pb2.WriteRawRequest.FromString,
                    response_serializer=nidaqmx__pb2.WriteRawResponse.SerializeToString,
            ),
            'WriteToTEDSFromArray': grpc.unary_unary_rpc_method_handler(
                    servicer.WriteToTEDSFromArray,
                    request_deserializer=nidaqmx__pb2.WriteToTEDSFromArrayRequest.FromString,
                    response_serializer=nidaqmx__pb2.WriteToTEDSFromArrayResponse.SerializeToString,
            ),
            'WriteToTEDSFromFile': grpc.unary_unary_rpc_method_handler(
                    servicer.WriteToTEDSFromFile,
                    request_deserializer=nidaqmx__pb2.WriteToTEDSFromFileRequest.FromString,
                    response_serializer=nidaqmx__pb2.WriteToTEDSFromFileResponse.SerializeToString,
            ),
    }
    generic_handler = grpc.method_handlers_generic_handler(
            'nidaqmx_grpc.NiDAQmx', rpc_method_handlers)
    server.add_generic_rpc_handlers((generic_handler,))


 # This class is part of an EXPERIMENTAL API.
class NiDAQmx(object):
    """Missing associated documentation comment in .proto file."""

    @staticmethod
    def AddCDAQSyncConnection(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/nidaqmx_grpc.NiDAQmx/AddCDAQSyncConnection',
            nidaqmx__pb2.AddCDAQSyncConnectionRequest.SerializeToString,
            nidaqmx__pb2.AddCDAQSyncConnectionResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def AddGlobalChansToTask(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/nidaqmx_grpc.NiDAQmx/AddGlobalChansToTask',
            nidaqmx__pb2.AddGlobalChansToTaskRequest.SerializeToString,
            nidaqmx__pb2.AddGlobalChansToTaskResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def AddNetworkDevice(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/nidaqmx_grpc.NiDAQmx/AddNetworkDevice',
            nidaqmx__pb2.AddNetworkDeviceRequest.SerializeToString,
            nidaqmx__pb2.AddNetworkDeviceResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def AreConfiguredCDAQSyncPortsDisconnected(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/nidaqmx_grpc.NiDAQmx/AreConfiguredCDAQSyncPortsDisconnected',
            nidaqmx__pb2.AreConfiguredCDAQSyncPortsDisconnectedRequest.SerializeToString,
            nidaqmx__pb2.AreConfiguredCDAQSyncPortsDisconnectedResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def AutoConfigureCDAQSyncConnections(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/nidaqmx_grpc.NiDAQmx/AutoConfigureCDAQSyncConnections',
            nidaqmx__pb2.AutoConfigureCDAQSyncConnectionsRequest.SerializeToString,
            nidaqmx__pb2.AutoConfigureCDAQSyncConnectionsResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def CalculateReversePolyCoeff(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/nidaqmx_grpc.NiDAQmx/CalculateReversePolyCoeff',
            nidaqmx__pb2.CalculateReversePolyCoeffRequest.SerializeToString,
            nidaqmx__pb2.CalculateReversePolyCoeffResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def CfgAnlgEdgeRefTrig(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/nidaqmx_grpc.NiDAQmx/CfgAnlgEdgeRefTrig',
            nidaqmx__pb2.CfgAnlgEdgeRefTrigRequest.SerializeToString,
            nidaqmx__pb2.CfgAnlgEdgeRefTrigResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def CfgAnlgEdgeStartTrig(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/nidaqmx_grpc.NiDAQmx/CfgAnlgEdgeStartTrig',
            nidaqmx__pb2.CfgAnlgEdgeStartTrigRequest.SerializeToString,
            nidaqmx__pb2.CfgAnlgEdgeStartTrigResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def CfgAnlgMultiEdgeRefTrig(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/nidaqmx_grpc.NiDAQmx/CfgAnlgMultiEdgeRefTrig',
            nidaqmx__pb2.CfgAnlgMultiEdgeRefTrigRequest.SerializeToString,
            nidaqmx__pb2.CfgAnlgMultiEdgeRefTrigResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def CfgAnlgMultiEdgeStartTrig(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/nidaqmx_grpc.NiDAQmx/CfgAnlgMultiEdgeStartTrig',
            nidaqmx__pb2.CfgAnlgMultiEdgeStartTrigRequest.SerializeToString,
            nidaqmx__pb2.CfgAnlgMultiEdgeStartTrigResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def CfgAnlgWindowRefTrig(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/nidaqmx_grpc.NiDAQmx/CfgAnlgWindowRefTrig',
            nidaqmx__pb2.CfgAnlgWindowRefTrigRequest.SerializeToString,
            nidaqmx__pb2.CfgAnlgWindowRefTrigResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def CfgAnlgWindowStartTrig(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/nidaqmx_grpc.NiDAQmx/CfgAnlgWindowStartTrig',
            nidaqmx__pb2.CfgAnlgWindowStartTrigRequest.SerializeToString,
            nidaqmx__pb2.CfgAnlgWindowStartTrigResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def CfgBurstHandshakingTimingExportClock(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/nidaqmx_grpc.NiDAQmx/CfgBurstHandshakingTimingExportClock',
            nidaqmx__pb2.CfgBurstHandshakingTimingExportClockRequest.SerializeToString,
            nidaqmx__pb2.CfgBurstHandshakingTimingExportClockResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def CfgBurstHandshakingTimingImportClock(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/nidaqmx_grpc.NiDAQmx/CfgBurstHandshakingTimingImportClock',
            nidaqmx__pb2.CfgBurstHandshakingTimingImportClockRequest.SerializeToString,
            nidaqmx__pb2.CfgBurstHandshakingTimingImportClockResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def CfgChangeDetectionTiming(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/nidaqmx_grpc.NiDAQmx/CfgChangeDetectionTiming',
            nidaqmx__pb2.CfgChangeDetectionTimingRequest.SerializeToString,
            nidaqmx__pb2.CfgChangeDetectionTimingResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def CfgDigEdgeRefTrig(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/nidaqmx_grpc.NiDAQmx/CfgDigEdgeRefTrig',
            nidaqmx__pb2.CfgDigEdgeRefTrigRequest.SerializeToString,
            nidaqmx__pb2.CfgDigEdgeRefTrigResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def CfgDigEdgeStartTrig(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/nidaqmx_grpc.NiDAQmx/CfgDigEdgeStartTrig',
            nidaqmx__pb2.CfgDigEdgeStartTrigRequest.SerializeToString,
            nidaqmx__pb2.CfgDigEdgeStartTrigResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def CfgDigPatternRefTrig(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/nidaqmx_grpc.NiDAQmx/CfgDigPatternRefTrig',
            nidaqmx__pb2.CfgDigPatternRefTrigRequest.SerializeToString,
            nidaqmx__pb2.CfgDigPatternRefTrigResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def CfgDigPatternStartTrig(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/nidaqmx_grpc.NiDAQmx/CfgDigPatternStartTrig',
            nidaqmx__pb2.CfgDigPatternStartTrigRequest.SerializeToString,
            nidaqmx__pb2.CfgDigPatternStartTrigResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def CfgHandshakingTiming(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/nidaqmx_grpc.NiDAQmx/CfgHandshakingTiming',
            nidaqmx__pb2.CfgHandshakingTimingRequest.SerializeToString,
            nidaqmx__pb2.CfgHandshakingTimingResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def CfgImplicitTiming(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/nidaqmx_grpc.NiDAQmx/CfgImplicitTiming',
            nidaqmx__pb2.CfgImplicitTimingRequest.SerializeToString,
            nidaqmx__pb2.CfgImplicitTimingResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def CfgInputBuffer(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/nidaqmx_grpc.NiDAQmx/CfgInputBuffer',
            nidaqmx__pb2.CfgInputBufferRequest.SerializeToString,
            nidaqmx__pb2.CfgInputBufferResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def CfgOutputBuffer(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/nidaqmx_grpc.NiDAQmx/CfgOutputBuffer',
            nidaqmx__pb2.CfgOutputBufferRequest.SerializeToString,
            nidaqmx__pb2.CfgOutputBufferResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def CfgPipelinedSampClkTiming(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/nidaqmx_grpc.NiDAQmx/CfgPipelinedSampClkTiming',
            nidaqmx__pb2.CfgPipelinedSampClkTimingRequest.SerializeToString,
            nidaqmx__pb2.CfgPipelinedSampClkTimingResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def CfgSampClkTiming(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/nidaqmx_grpc.NiDAQmx/CfgSampClkTiming',
            nidaqmx__pb2.CfgSampClkTimingRequest.SerializeToString,
            nidaqmx__pb2.CfgSampClkTimingResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def CfgTimeStartTrig(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/nidaqmx_grpc.NiDAQmx/CfgTimeStartTrig',
            nidaqmx__pb2.CfgTimeStartTrigRequest.SerializeToString,
            nidaqmx__pb2.CfgTimeStartTrigResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def CfgWatchdogAOExpirStates(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/nidaqmx_grpc.NiDAQmx/CfgWatchdogAOExpirStates',
            nidaqmx__pb2.CfgWatchdogAOExpirStatesRequest.SerializeToString,
            nidaqmx__pb2.CfgWatchdogAOExpirStatesResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def CfgWatchdogCOExpirStates(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/nidaqmx_grpc.NiDAQmx/CfgWatchdogCOExpirStates',
            nidaqmx__pb2.CfgWatchdogCOExpirStatesRequest.SerializeToString,
            nidaqmx__pb2.CfgWatchdogCOExpirStatesResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def CfgWatchdogDOExpirStates(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/nidaqmx_grpc.NiDAQmx/CfgWatchdogDOExpirStates',
            nidaqmx__pb2.CfgWatchdogDOExpirStatesRequest.SerializeToString,
            nidaqmx__pb2.CfgWatchdogDOExpirStatesResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def ClearTEDS(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/nidaqmx_grpc.NiDAQmx/ClearTEDS',
            nidaqmx__pb2.ClearTEDSRequest.SerializeToString,
            nidaqmx__pb2.ClearTEDSResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def ClearTask(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/nidaqmx_grpc.NiDAQmx/ClearTask',
            nidaqmx__pb2.ClearTaskRequest.SerializeToString,
            nidaqmx__pb2.ClearTaskResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def ConfigureLogging(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/nidaqmx_grpc.NiDAQmx/ConfigureLogging',
            nidaqmx__pb2.ConfigureLoggingRequest.SerializeToString,
            nidaqmx__pb2.ConfigureLoggingResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def ConfigureTEDS(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/nidaqmx_grpc.NiDAQmx/ConfigureTEDS',
            nidaqmx__pb2.ConfigureTEDSRequest.SerializeToString,
            nidaqmx__pb2.ConfigureTEDSResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def ConnectTerms(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/nidaqmx_grpc.NiDAQmx/ConnectTerms',
            nidaqmx__pb2.ConnectTermsRequest.SerializeToString,
            nidaqmx__pb2.ConnectTermsResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def ControlWatchdogTask(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/nidaqmx_grpc.NiDAQmx/ControlWatchdogTask',
            nidaqmx__pb2.ControlWatchdogTaskRequest.SerializeToString,
            nidaqmx__pb2.ControlWatchdogTaskResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def CreateAIAccel4WireDCVoltageChan(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/nidaqmx_grpc.NiDAQmx/CreateAIAccel4WireDCVoltageChan',
            nidaqmx__pb2.CreateAIAccel4WireDCVoltageChanRequest.SerializeToString,
            nidaqmx__pb2.CreateAIAccel4WireDCVoltageChanResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def CreateAIAccelChan(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/nidaqmx_grpc.NiDAQmx/CreateAIAccelChan',
            nidaqmx__pb2.CreateAIAccelChanRequest.SerializeToString,
            nidaqmx__pb2.CreateAIAccelChanResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def CreateAIAccelChargeChan(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/nidaqmx_grpc.NiDAQmx/CreateAIAccelChargeChan',
            nidaqmx__pb2.CreateAIAccelChargeChanRequest.SerializeToString,
            nidaqmx__pb2.CreateAIAccelChargeChanResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def CreateAIBridgeChan(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/nidaqmx_grpc.NiDAQmx/CreateAIBridgeChan',
            nidaqmx__pb2.CreateAIBridgeChanRequest.SerializeToString,
            nidaqmx__pb2.CreateAIBridgeChanResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def CreateAIChargeChan(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/nidaqmx_grpc.NiDAQmx/CreateAIChargeChan',
            nidaqmx__pb2.CreateAIChargeChanRequest.SerializeToString,
            nidaqmx__pb2.CreateAIChargeChanResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def CreateAICurrentChan(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/nidaqmx_grpc.NiDAQmx/CreateAICurrentChan',
            nidaqmx__pb2.CreateAICurrentChanRequest.SerializeToString,
            nidaqmx__pb2.CreateAICurrentChanResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def CreateAICurrentRMSChan(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/nidaqmx_grpc.NiDAQmx/CreateAICurrentRMSChan',
            nidaqmx__pb2.CreateAICurrentRMSChanRequest.SerializeToString,
            nidaqmx__pb2.CreateAICurrentRMSChanResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def CreateAIForceBridgePolynomialChan(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/nidaqmx_grpc.NiDAQmx/CreateAIForceBridgePolynomialChan',
            nidaqmx__pb2.CreateAIForceBridgePolynomialChanRequest.SerializeToString,
            nidaqmx__pb2.CreateAIForceBridgePolynomialChanResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def CreateAIForceBridgeTableChan(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/nidaqmx_grpc.NiDAQmx/CreateAIForceBridgeTableChan',
            nidaqmx__pb2.CreateAIForceBridgeTableChanRequest.SerializeToString,
            nidaqmx__pb2.CreateAIForceBridgeTableChanResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def CreateAIForceBridgeTwoPointLinChan(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/nidaqmx_grpc.NiDAQmx/CreateAIForceBridgeTwoPointLinChan',
            nidaqmx__pb2.CreateAIForceBridgeTwoPointLinChanRequest.SerializeToString,
            nidaqmx__pb2.CreateAIForceBridgeTwoPointLinChanResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def CreateAIForceIEPEChan(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/nidaqmx_grpc.NiDAQmx/CreateAIForceIEPEChan',
            nidaqmx__pb2.CreateAIForceIEPEChanRequest.SerializeToString,
            nidaqmx__pb2.CreateAIForceIEPEChanResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def CreateAIFreqVoltageChan(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/nidaqmx_grpc.NiDAQmx/CreateAIFreqVoltageChan',
            nidaqmx__pb2.CreateAIFreqVoltageChanRequest.SerializeToString,
            nidaqmx__pb2.CreateAIFreqVoltageChanResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def CreateAIMicrophoneChan(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/nidaqmx_grpc.NiDAQmx/CreateAIMicrophoneChan',
            nidaqmx__pb2.CreateAIMicrophoneChanRequest.SerializeToString,
            nidaqmx__pb2.CreateAIMicrophoneChanResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def CreateAIPosEddyCurrProxProbeChan(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/nidaqmx_grpc.NiDAQmx/CreateAIPosEddyCurrProxProbeChan',
            nidaqmx__pb2.CreateAIPosEddyCurrProxProbeChanRequest.SerializeToString,
            nidaqmx__pb2.CreateAIPosEddyCurrProxProbeChanResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def CreateAIPosLVDTChan(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/nidaqmx_grpc.NiDAQmx/CreateAIPosLVDTChan',
            nidaqmx__pb2.CreateAIPosLVDTChanRequest.SerializeToString,
            nidaqmx__pb2.CreateAIPosLVDTChanResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def CreateAIPosRVDTChan(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/nidaqmx_grpc.NiDAQmx/CreateAIPosRVDTChan',
            nidaqmx__pb2.CreateAIPosRVDTChanRequest.SerializeToString,
            nidaqmx__pb2.CreateAIPosRVDTChanResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def CreateAIPowerChan(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/nidaqmx_grpc.NiDAQmx/CreateAIPowerChan',
            nidaqmx__pb2.CreateAIPowerChanRequest.SerializeToString,
            nidaqmx__pb2.CreateAIPowerChanResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def CreateAIPressureBridgePolynomialChan(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/nidaqmx_grpc.NiDAQmx/CreateAIPressureBridgePolynomialChan',
            nidaqmx__pb2.CreateAIPressureBridgePolynomialChanRequest.SerializeToString,
            nidaqmx__pb2.CreateAIPressureBridgePolynomialChanResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def CreateAIPressureBridgeTableChan(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/nidaqmx_grpc.NiDAQmx/CreateAIPressureBridgeTableChan',
            nidaqmx__pb2.CreateAIPressureBridgeTableChanRequest.SerializeToString,
            nidaqmx__pb2.CreateAIPressureBridgeTableChanResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def CreateAIPressureBridgeTwoPointLinChan(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/nidaqmx_grpc.NiDAQmx/CreateAIPressureBridgeTwoPointLinChan',
            nidaqmx__pb2.CreateAIPressureBridgeTwoPointLinChanRequest.SerializeToString,
            nidaqmx__pb2.CreateAIPressureBridgeTwoPointLinChanResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def CreateAIRTDChan(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/nidaqmx_grpc.NiDAQmx/CreateAIRTDChan',
            nidaqmx__pb2.CreateAIRTDChanRequest.SerializeToString,
            nidaqmx__pb2.CreateAIRTDChanResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def CreateAIResistanceChan(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/nidaqmx_grpc.NiDAQmx/CreateAIResistanceChan',
            nidaqmx__pb2.CreateAIResistanceChanRequest.SerializeToString,
            nidaqmx__pb2.CreateAIResistanceChanResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def CreateAIRosetteStrainGageChan(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/nidaqmx_grpc.NiDAQmx/CreateAIRosetteStrainGageChan',
            nidaqmx__pb2.CreateAIRosetteStrainGageChanRequest.SerializeToString,
            nidaqmx__pb2.CreateAIRosetteStrainGageChanResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def CreateAIStrainGageChan(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/nidaqmx_grpc.NiDAQmx/CreateAIStrainGageChan',
            nidaqmx__pb2.CreateAIStrainGageChanRequest.SerializeToString,
            nidaqmx__pb2.CreateAIStrainGageChanResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def CreateAITempBuiltInSensorChan(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/nidaqmx_grpc.NiDAQmx/CreateAITempBuiltInSensorChan',
            nidaqmx__pb2.CreateAITempBuiltInSensorChanRequest.SerializeToString,
            nidaqmx__pb2.CreateAITempBuiltInSensorChanResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def CreateAIThrmcplChan(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/nidaqmx_grpc.NiDAQmx/CreateAIThrmcplChan',
            nidaqmx__pb2.CreateAIThrmcplChanRequest.SerializeToString,
            nidaqmx__pb2.CreateAIThrmcplChanResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def CreateAIThrmstrChanIex(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/nidaqmx_grpc.NiDAQmx/CreateAIThrmstrChanIex',
            nidaqmx__pb2.CreateAIThrmstrChanIexRequest.SerializeToString,
            nidaqmx__pb2.CreateAIThrmstrChanIexResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def CreateAIThrmstrChanVex(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/nidaqmx_grpc.NiDAQmx/CreateAIThrmstrChanVex',
            nidaqmx__pb2.CreateAIThrmstrChanVexRequest.SerializeToString,
            nidaqmx__pb2.CreateAIThrmstrChanVexResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def CreateAITorqueBridgePolynomialChan(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/nidaqmx_grpc.NiDAQmx/CreateAITorqueBridgePolynomialChan',
            nidaqmx__pb2.CreateAITorqueBridgePolynomialChanRequest.SerializeToString,
            nidaqmx__pb2.CreateAITorqueBridgePolynomialChanResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def CreateAITorqueBridgeTableChan(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/nidaqmx_grpc.NiDAQmx/CreateAITorqueBridgeTableChan',
            nidaqmx__pb2.CreateAITorqueBridgeTableChanRequest.SerializeToString,
            nidaqmx__pb2.CreateAITorqueBridgeTableChanResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def CreateAITorqueBridgeTwoPointLinChan(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/nidaqmx_grpc.NiDAQmx/CreateAITorqueBridgeTwoPointLinChan',
            nidaqmx__pb2.CreateAITorqueBridgeTwoPointLinChanRequest.SerializeToString,
            nidaqmx__pb2.CreateAITorqueBridgeTwoPointLinChanResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def CreateAIVelocityIEPEChan(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/nidaqmx_grpc.NiDAQmx/CreateAIVelocityIEPEChan',
            nidaqmx__pb2.CreateAIVelocityIEPEChanRequest.SerializeToString,
            nidaqmx__pb2.CreateAIVelocityIEPEChanResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def CreateAIVoltageChan(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/nidaqmx_grpc.NiDAQmx/CreateAIVoltageChan',
            nidaqmx__pb2.CreateAIVoltageChanRequest.SerializeToString,
            nidaqmx__pb2.CreateAIVoltageChanResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def CreateAIVoltageChanWithExcit(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/nidaqmx_grpc.NiDAQmx/CreateAIVoltageChanWithExcit',
            nidaqmx__pb2.CreateAIVoltageChanWithExcitRequest.SerializeToString,
            nidaqmx__pb2.CreateAIVoltageChanWithExcitResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def CreateAIVoltageRMSChan(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/nidaqmx_grpc.NiDAQmx/CreateAIVoltageRMSChan',
            nidaqmx__pb2.CreateAIVoltageRMSChanRequest.SerializeToString,
            nidaqmx__pb2.CreateAIVoltageRMSChanResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def CreateAOCurrentChan(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/nidaqmx_grpc.NiDAQmx/CreateAOCurrentChan',
            nidaqmx__pb2.CreateAOCurrentChanRequest.SerializeToString,
            nidaqmx__pb2.CreateAOCurrentChanResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def CreateAOFuncGenChan(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/nidaqmx_grpc.NiDAQmx/CreateAOFuncGenChan',
            nidaqmx__pb2.CreateAOFuncGenChanRequest.SerializeToString,
            nidaqmx__pb2.CreateAOFuncGenChanResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def CreateAOVoltageChan(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/nidaqmx_grpc.NiDAQmx/CreateAOVoltageChan',
            nidaqmx__pb2.CreateAOVoltageChanRequest.SerializeToString,
            nidaqmx__pb2.CreateAOVoltageChanResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def CreateCIAngEncoderChan(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/nidaqmx_grpc.NiDAQmx/CreateCIAngEncoderChan',
            nidaqmx__pb2.CreateCIAngEncoderChanRequest.SerializeToString,
            nidaqmx__pb2.CreateCIAngEncoderChanResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def CreateCIAngVelocityChan(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/nidaqmx_grpc.NiDAQmx/CreateCIAngVelocityChan',
            nidaqmx__pb2.CreateCIAngVelocityChanRequest.SerializeToString,
            nidaqmx__pb2.CreateCIAngVelocityChanResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def CreateCICountEdgesChan(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/nidaqmx_grpc.NiDAQmx/CreateCICountEdgesChan',
            nidaqmx__pb2.CreateCICountEdgesChanRequest.SerializeToString,
            nidaqmx__pb2.CreateCICountEdgesChanResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def CreateCIDutyCycleChan(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/nidaqmx_grpc.NiDAQmx/CreateCIDutyCycleChan',
            nidaqmx__pb2.CreateCIDutyCycleChanRequest.SerializeToString,
            nidaqmx__pb2.CreateCIDutyCycleChanResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def CreateCIFreqChan(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/nidaqmx_grpc.NiDAQmx/CreateCIFreqChan',
            nidaqmx__pb2.CreateCIFreqChanRequest.SerializeToString,
            nidaqmx__pb2.CreateCIFreqChanResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def CreateCIGPSTimestampChan(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/nidaqmx_grpc.NiDAQmx/CreateCIGPSTimestampChan',
            nidaqmx__pb2.CreateCIGPSTimestampChanRequest.SerializeToString,
            nidaqmx__pb2.CreateCIGPSTimestampChanResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def CreateCILinEncoderChan(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/nidaqmx_grpc.NiDAQmx/CreateCILinEncoderChan',
            nidaqmx__pb2.CreateCILinEncoderChanRequest.SerializeToString,
            nidaqmx__pb2.CreateCILinEncoderChanResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def CreateCILinVelocityChan(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/nidaqmx_grpc.NiDAQmx/CreateCILinVelocityChan',
            nidaqmx__pb2.CreateCILinVelocityChanRequest.SerializeToString,
            nidaqmx__pb2.CreateCILinVelocityChanResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def CreateCIPeriodChan(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/nidaqmx_grpc.NiDAQmx/CreateCIPeriodChan',
            nidaqmx__pb2.CreateCIPeriodChanRequest.SerializeToString,
            nidaqmx__pb2.CreateCIPeriodChanResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def CreateCIPulseChanFreq(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/nidaqmx_grpc.NiDAQmx/CreateCIPulseChanFreq',
            nidaqmx__pb2.CreateCIPulseChanFreqRequest.SerializeToString,
            nidaqmx__pb2.CreateCIPulseChanFreqResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def CreateCIPulseChanTicks(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/nidaqmx_grpc.NiDAQmx/CreateCIPulseChanTicks',
            nidaqmx__pb2.CreateCIPulseChanTicksRequest.SerializeToString,
            nidaqmx__pb2.CreateCIPulseChanTicksResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def CreateCIPulseChanTime(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/nidaqmx_grpc.NiDAQmx/CreateCIPulseChanTime',
            nidaqmx__pb2.CreateCIPulseChanTimeRequest.SerializeToString,
            nidaqmx__pb2.CreateCIPulseChanTimeResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def CreateCIPulseWidthChan(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/nidaqmx_grpc.NiDAQmx/CreateCIPulseWidthChan',
            nidaqmx__pb2.CreateCIPulseWidthChanRequest.SerializeToString,
            nidaqmx__pb2.CreateCIPulseWidthChanResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def CreateCISemiPeriodChan(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/nidaqmx_grpc.NiDAQmx/CreateCISemiPeriodChan',
            nidaqmx__pb2.CreateCISemiPeriodChanRequest.SerializeToString,
            nidaqmx__pb2.CreateCISemiPeriodChanResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def CreateCITwoEdgeSepChan(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/nidaqmx_grpc.NiDAQmx/CreateCITwoEdgeSepChan',
            nidaqmx__pb2.CreateCITwoEdgeSepChanRequest.SerializeToString,
            nidaqmx__pb2.CreateCITwoEdgeSepChanResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def CreateCOPulseChanFreq(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/nidaqmx_grpc.NiDAQmx/CreateCOPulseChanFreq',
            nidaqmx__pb2.CreateCOPulseChanFreqRequest.SerializeToString,
            nidaqmx__pb2.CreateCOPulseChanFreqResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def CreateCOPulseChanTicks(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/nidaqmx_grpc.NiDAQmx/CreateCOPulseChanTicks',
            nidaqmx__pb2.CreateCOPulseChanTicksRequest.SerializeToString,
            nidaqmx__pb2.CreateCOPulseChanTicksResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def CreateCOPulseChanTime(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/nidaqmx_grpc.NiDAQmx/CreateCOPulseChanTime',
            nidaqmx__pb2.CreateCOPulseChanTimeRequest.SerializeToString,
            nidaqmx__pb2.CreateCOPulseChanTimeResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def CreateDIChan(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/nidaqmx_grpc.NiDAQmx/CreateDIChan',
            nidaqmx__pb2.CreateDIChanRequest.SerializeToString,
            nidaqmx__pb2.CreateDIChanResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def CreateDOChan(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/nidaqmx_grpc.NiDAQmx/CreateDOChan',
            nidaqmx__pb2.CreateDOChanRequest.SerializeToString,
            nidaqmx__pb2.CreateDOChanResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def CreateLinScale(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/nidaqmx_grpc.NiDAQmx/CreateLinScale',
            nidaqmx__pb2.CreateLinScaleRequest.SerializeToString,
            nidaqmx__pb2.CreateLinScaleResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def CreateMapScale(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/nidaqmx_grpc.NiDAQmx/CreateMapScale',
            nidaqmx__pb2.CreateMapScaleRequest.SerializeToString,
            nidaqmx__pb2.CreateMapScaleResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def CreatePolynomialScale(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/nidaqmx_grpc.NiDAQmx/CreatePolynomialScale',
            nidaqmx__pb2.CreatePolynomialScaleRequest.SerializeToString,
            nidaqmx__pb2.CreatePolynomialScaleResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def CreateTEDSAIAccelChan(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/nidaqmx_grpc.NiDAQmx/CreateTEDSAIAccelChan',
            nidaqmx__pb2.CreateTEDSAIAccelChanRequest.SerializeToString,
            nidaqmx__pb2.CreateTEDSAIAccelChanResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def CreateTEDSAIBridgeChan(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/nidaqmx_grpc.NiDAQmx/CreateTEDSAIBridgeChan',
            nidaqmx__pb2.CreateTEDSAIBridgeChanRequest.SerializeToString,
            nidaqmx__pb2.CreateTEDSAIBridgeChanResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def CreateTEDSAICurrentChan(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/nidaqmx_grpc.NiDAQmx/CreateTEDSAICurrentChan',
            nidaqmx__pb2.CreateTEDSAICurrentChanRequest.SerializeToString,
            nidaqmx__pb2.CreateTEDSAICurrentChanResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def CreateTEDSAIForceBridgeChan(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/nidaqmx_grpc.NiDAQmx/CreateTEDSAIForceBridgeChan',
            nidaqmx__pb2.CreateTEDSAIForceBridgeChanRequest.SerializeToString,
            nidaqmx__pb2.CreateTEDSAIForceBridgeChanResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def CreateTEDSAIForceIEPEChan(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/nidaqmx_grpc.NiDAQmx/CreateTEDSAIForceIEPEChan',
            nidaqmx__pb2.CreateTEDSAIForceIEPEChanRequest.SerializeToString,
            nidaqmx__pb2.CreateTEDSAIForceIEPEChanResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def CreateTEDSAIMicrophoneChan(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/nidaqmx_grpc.NiDAQmx/CreateTEDSAIMicrophoneChan',
            nidaqmx__pb2.CreateTEDSAIMicrophoneChanRequest.SerializeToString,
            nidaqmx__pb2.CreateTEDSAIMicrophoneChanResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def CreateTEDSAIPosLVDTChan(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/nidaqmx_grpc.NiDAQmx/CreateTEDSAIPosLVDTChan',
            nidaqmx__pb2.CreateTEDSAIPosLVDTChanRequest.SerializeToString,
            nidaqmx__pb2.CreateTEDSAIPosLVDTChanResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def CreateTEDSAIPosRVDTChan(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/nidaqmx_grpc.NiDAQmx/CreateTEDSAIPosRVDTChan',
            nidaqmx__pb2.CreateTEDSAIPosRVDTChanRequest.SerializeToString,
            nidaqmx__pb2.CreateTEDSAIPosRVDTChanResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def CreateTEDSAIPressureBridgeChan(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/nidaqmx_grpc.NiDAQmx/CreateTEDSAIPressureBridgeChan',
            nidaqmx__pb2.CreateTEDSAIPressureBridgeChanRequest.SerializeToString,
            nidaqmx__pb2.CreateTEDSAIPressureBridgeChanResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def CreateTEDSAIRTDChan(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/nidaqmx_grpc.NiDAQmx/CreateTEDSAIRTDChan',
            nidaqmx__pb2.CreateTEDSAIRTDChanRequest.SerializeToString,
            nidaqmx__pb2.CreateTEDSAIRTDChanResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def CreateTEDSAIResistanceChan(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/nidaqmx_grpc.NiDAQmx/CreateTEDSAIResistanceChan',
            nidaqmx__pb2.CreateTEDSAIResistanceChanRequest.SerializeToString,
            nidaqmx__pb2.CreateTEDSAIResistanceChanResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def CreateTEDSAIStrainGageChan(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/nidaqmx_grpc.NiDAQmx/CreateTEDSAIStrainGageChan',
            nidaqmx__pb2.CreateTEDSAIStrainGageChanRequest.SerializeToString,
            nidaqmx__pb2.CreateTEDSAIStrainGageChanResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def CreateTEDSAIThrmcplChan(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/nidaqmx_grpc.NiDAQmx/CreateTEDSAIThrmcplChan',
            nidaqmx__pb2.CreateTEDSAIThrmcplChanRequest.SerializeToString,
            nidaqmx__pb2.CreateTEDSAIThrmcplChanResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def CreateTEDSAIThrmstrChanIex(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/nidaqmx_grpc.NiDAQmx/CreateTEDSAIThrmstrChanIex',
            nidaqmx__pb2.CreateTEDSAIThrmstrChanIexRequest.SerializeToString,
            nidaqmx__pb2.CreateTEDSAIThrmstrChanIexResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def CreateTEDSAIThrmstrChanVex(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/nidaqmx_grpc.NiDAQmx/CreateTEDSAIThrmstrChanVex',
            nidaqmx__pb2.CreateTEDSAIThrmstrChanVexRequest.SerializeToString,
            nidaqmx__pb2.CreateTEDSAIThrmstrChanVexResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def CreateTEDSAITorqueBridgeChan(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/nidaqmx_grpc.NiDAQmx/CreateTEDSAITorqueBridgeChan',
            nidaqmx__pb2.CreateTEDSAITorqueBridgeChanRequest.SerializeToString,
            nidaqmx__pb2.CreateTEDSAITorqueBridgeChanResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def CreateTEDSAIVoltageChan(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/nidaqmx_grpc.NiDAQmx/CreateTEDSAIVoltageChan',
            nidaqmx__pb2.CreateTEDSAIVoltageChanRequest.SerializeToString,
            nidaqmx__pb2.CreateTEDSAIVoltageChanResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def CreateTEDSAIVoltageChanWithExcit(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/nidaqmx_grpc.NiDAQmx/CreateTEDSAIVoltageChanWithExcit',
            nidaqmx__pb2.CreateTEDSAIVoltageChanWithExcitRequest.SerializeToString,
            nidaqmx__pb2.CreateTEDSAIVoltageChanWithExcitResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def CreateTableScale(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/nidaqmx_grpc.NiDAQmx/CreateTableScale',
            nidaqmx__pb2.CreateTableScaleRequest.SerializeToString,
            nidaqmx__pb2.CreateTableScaleResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def CreateTask(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/nidaqmx_grpc.NiDAQmx/CreateTask',
            nidaqmx__pb2.CreateTaskRequest.SerializeToString,
            nidaqmx__pb2.CreateTaskResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def CreateWatchdogTimerTask(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/nidaqmx_grpc.NiDAQmx/CreateWatchdogTimerTask',
            nidaqmx__pb2.CreateWatchdogTimerTaskRequest.SerializeToString,
            nidaqmx__pb2.CreateWatchdogTimerTaskResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def CreateWatchdogTimerTaskEx(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/nidaqmx_grpc.NiDAQmx/CreateWatchdogTimerTaskEx',
            nidaqmx__pb2.CreateWatchdogTimerTaskExRequest.SerializeToString,
            nidaqmx__pb2.CreateWatchdogTimerTaskExResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def DeleteNetworkDevice(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/nidaqmx_grpc.NiDAQmx/DeleteNetworkDevice',
            nidaqmx__pb2.DeleteNetworkDeviceRequest.SerializeToString,
            nidaqmx__pb2.DeleteNetworkDeviceResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def DeleteSavedGlobalChan(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/nidaqmx_grpc.NiDAQmx/DeleteSavedGlobalChan',
            nidaqmx__pb2.DeleteSavedGlobalChanRequest.SerializeToString,
            nidaqmx__pb2.DeleteSavedGlobalChanResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def DeleteSavedScale(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/nidaqmx_grpc.NiDAQmx/DeleteSavedScale',
            nidaqmx__pb2.DeleteSavedScaleRequest.SerializeToString,
            nidaqmx__pb2.DeleteSavedScaleResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def DeleteSavedTask(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/nidaqmx_grpc.NiDAQmx/DeleteSavedTask',
            nidaqmx__pb2.DeleteSavedTaskRequest.SerializeToString,
            nidaqmx__pb2.DeleteSavedTaskResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def DeviceSupportsCal(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/nidaqmx_grpc.NiDAQmx/DeviceSupportsCal',
            nidaqmx__pb2.DeviceSupportsCalRequest.SerializeToString,
            nidaqmx__pb2.DeviceSupportsCalResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def DisableRefTrig(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/nidaqmx_grpc.NiDAQmx/DisableRefTrig',
            nidaqmx__pb2.DisableRefTrigRequest.SerializeToString,
            nidaqmx__pb2.DisableRefTrigResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def DisableStartTrig(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/nidaqmx_grpc.NiDAQmx/DisableStartTrig',
            nidaqmx__pb2.DisableStartTrigRequest.SerializeToString,
            nidaqmx__pb2.DisableStartTrigResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def DisconnectTerms(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/nidaqmx_grpc.NiDAQmx/DisconnectTerms',
            nidaqmx__pb2.DisconnectTermsRequest.SerializeToString,
            nidaqmx__pb2.DisconnectTermsResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def ExportSignal(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/nidaqmx_grpc.NiDAQmx/ExportSignal',
            nidaqmx__pb2.ExportSignalRequest.SerializeToString,
            nidaqmx__pb2.ExportSignalResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def GetAIChanCalCalDate(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/nidaqmx_grpc.NiDAQmx/GetAIChanCalCalDate',
            nidaqmx__pb2.GetAIChanCalCalDateRequest.SerializeToString,
            nidaqmx__pb2.GetAIChanCalCalDateResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def GetAIChanCalExpDate(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/nidaqmx_grpc.NiDAQmx/GetAIChanCalExpDate',
            nidaqmx__pb2.GetAIChanCalExpDateRequest.SerializeToString,
            nidaqmx__pb2.GetAIChanCalExpDateResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def GetAnalogPowerUpStates(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/nidaqmx_grpc.NiDAQmx/GetAnalogPowerUpStates',
            nidaqmx__pb2.GetAnalogPowerUpStatesRequest.SerializeToString,
            nidaqmx__pb2.GetAnalogPowerUpStatesResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def GetAnalogPowerUpStatesWithOutputType(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/nidaqmx_grpc.NiDAQmx/GetAnalogPowerUpStatesWithOutputType',
            nidaqmx__pb2.GetAnalogPowerUpStatesWithOutputTypeRequest.SerializeToString,
            nidaqmx__pb2.GetAnalogPowerUpStatesWithOutputTypeResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def GetArmStartTrigTimestampVal(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/nidaqmx_grpc.NiDAQmx/GetArmStartTrigTimestampVal',
            nidaqmx__pb2.GetArmStartTrigTimestampValRequest.SerializeToString,
            nidaqmx__pb2.GetArmStartTrigTimestampValResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def GetArmStartTrigTrigWhen(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/nidaqmx_grpc.NiDAQmx/GetArmStartTrigTrigWhen',
            nidaqmx__pb2.GetArmStartTrigTrigWhenRequest.SerializeToString,
            nidaqmx__pb2.GetArmStartTrigTrigWhenResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def GetAutoConfiguredCDAQSyncConnections(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/nidaqmx_grpc.NiDAQmx/GetAutoConfiguredCDAQSyncConnections',
            nidaqmx__pb2.GetAutoConfiguredCDAQSyncConnectionsRequest.SerializeToString,
            nidaqmx__pb2.GetAutoConfiguredCDAQSyncConnectionsResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def GetBufferAttributeUInt32(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/nidaqmx_grpc.NiDAQmx/GetBufferAttributeUInt32',
            nidaqmx__pb2.GetBufferAttributeUInt32Request.SerializeToString,
            nidaqmx__pb2.GetBufferAttributeUInt32Response.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def GetCalInfoAttributeBool(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/nidaqmx_grpc.NiDAQmx/GetCalInfoAttributeBool',
            nidaqmx__pb2.GetCalInfoAttributeBoolRequest.SerializeToString,
            nidaqmx__pb2.GetCalInfoAttributeBoolResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def GetCalInfoAttributeDouble(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/nidaqmx_grpc.NiDAQmx/GetCalInfoAttributeDouble',
            nidaqmx__pb2.GetCalInfoAttributeDoubleRequest.SerializeToString,
            nidaqmx__pb2.GetCalInfoAttributeDoubleResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def GetCalInfoAttributeString(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/nidaqmx_grpc.NiDAQmx/GetCalInfoAttributeString',
            nidaqmx__pb2.GetCalInfoAttributeStringRequest.SerializeToString,
            nidaqmx__pb2.GetCalInfoAttributeStringResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def GetCalInfoAttributeUInt32(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/nidaqmx_grpc.NiDAQmx/GetCalInfoAttributeUInt32',
            nidaqmx__pb2.GetCalInfoAttributeUInt32Request.SerializeToString,
            nidaqmx__pb2.GetCalInfoAttributeUInt32Response.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def GetChanAttributeBool(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/nidaqmx_grpc.NiDAQmx/GetChanAttributeBool',
            nidaqmx__pb2.GetChanAttributeBoolRequest.SerializeToString,
            nidaqmx__pb2.GetChanAttributeBoolResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def GetChanAttributeDouble(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/nidaqmx_grpc.NiDAQmx/GetChanAttributeDouble',
            nidaqmx__pb2.GetChanAttributeDoubleRequest.SerializeToString,
            nidaqmx__pb2.GetChanAttributeDoubleResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def GetChanAttributeDoubleArray(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/nidaqmx_grpc.NiDAQmx/GetChanAttributeDoubleArray',
            nidaqmx__pb2.GetChanAttributeDoubleArrayRequest.SerializeToString,
            nidaqmx__pb2.GetChanAttributeDoubleArrayResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def GetChanAttributeInt32(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/nidaqmx_grpc.NiDAQmx/GetChanAttributeInt32',
            nidaqmx__pb2.GetChanAttributeInt32Request.SerializeToString,
            nidaqmx__pb2.GetChanAttributeInt32Response.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def GetChanAttributeString(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/nidaqmx_grpc.NiDAQmx/GetChanAttributeString',
            nidaqmx__pb2.GetChanAttributeStringRequest.SerializeToString,
            nidaqmx__pb2.GetChanAttributeStringResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def GetChanAttributeUInt32(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/nidaqmx_grpc.NiDAQmx/GetChanAttributeUInt32',
            nidaqmx__pb2.GetChanAttributeUInt32Request.SerializeToString,
            nidaqmx__pb2.GetChanAttributeUInt32Response.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def GetDeviceAttributeBool(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/nidaqmx_grpc.NiDAQmx/GetDeviceAttributeBool',
            nidaqmx__pb2.GetDeviceAttributeBoolRequest.SerializeToString,
            nidaqmx__pb2.GetDeviceAttributeBoolResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def GetDeviceAttributeDouble(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/nidaqmx_grpc.NiDAQmx/GetDeviceAttributeDouble',
            nidaqmx__pb2.GetDeviceAttributeDoubleRequest.SerializeToString,
            nidaqmx__pb2.GetDeviceAttributeDoubleResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def GetDeviceAttributeDoubleArray(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/nidaqmx_grpc.NiDAQmx/GetDeviceAttributeDoubleArray',
            nidaqmx__pb2.GetDeviceAttributeDoubleArrayRequest.SerializeToString,
            nidaqmx__pb2.GetDeviceAttributeDoubleArrayResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def GetDeviceAttributeInt32(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/nidaqmx_grpc.NiDAQmx/GetDeviceAttributeInt32',
            nidaqmx__pb2.GetDeviceAttributeInt32Request.SerializeToString,
            nidaqmx__pb2.GetDeviceAttributeInt32Response.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def GetDeviceAttributeInt32Array(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/nidaqmx_grpc.NiDAQmx/GetDeviceAttributeInt32Array',
            nidaqmx__pb2.GetDeviceAttributeInt32ArrayRequest.SerializeToString,
            nidaqmx__pb2.GetDeviceAttributeInt32ArrayResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def GetDeviceAttributeString(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/nidaqmx_grpc.NiDAQmx/GetDeviceAttributeString',
            nidaqmx__pb2.GetDeviceAttributeStringRequest.SerializeToString,
            nidaqmx__pb2.GetDeviceAttributeStringResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def GetDeviceAttributeUInt32(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/nidaqmx_grpc.NiDAQmx/GetDeviceAttributeUInt32',
            nidaqmx__pb2.GetDeviceAttributeUInt32Request.SerializeToString,
            nidaqmx__pb2.GetDeviceAttributeUInt32Response.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def GetDeviceAttributeUInt32Array(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/nidaqmx_grpc.NiDAQmx/GetDeviceAttributeUInt32Array',
            nidaqmx__pb2.GetDeviceAttributeUInt32ArrayRequest.SerializeToString,
            nidaqmx__pb2.GetDeviceAttributeUInt32ArrayResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def GetDigitalLogicFamilyPowerUpState(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/nidaqmx_grpc.NiDAQmx/GetDigitalLogicFamilyPowerUpState',
            nidaqmx__pb2.GetDigitalLogicFamilyPowerUpStateRequest.SerializeToString,
            nidaqmx__pb2.GetDigitalLogicFamilyPowerUpStateResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def GetDigitalPowerUpStates(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/nidaqmx_grpc.NiDAQmx/GetDigitalPowerUpStates',
            nidaqmx__pb2.GetDigitalPowerUpStatesRequest.SerializeToString,
            nidaqmx__pb2.GetDigitalPowerUpStatesResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def GetDigitalPullUpPullDownStates(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/nidaqmx_grpc.NiDAQmx/GetDigitalPullUpPullDownStates',
            nidaqmx__pb2.GetDigitalPullUpPullDownStatesRequest.SerializeToString,
            nidaqmx__pb2.GetDigitalPullUpPullDownStatesResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def GetDisconnectedCDAQSyncPorts(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/nidaqmx_grpc.NiDAQmx/GetDisconnectedCDAQSyncPorts',
            nidaqmx__pb2.GetDisconnectedCDAQSyncPortsRequest.SerializeToString,
            nidaqmx__pb2.GetDisconnectedCDAQSyncPortsResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def GetErrorString(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/nidaqmx_grpc.NiDAQmx/GetErrorString',
            nidaqmx__pb2.GetErrorStringRequest.SerializeToString,
            nidaqmx__pb2.GetErrorStringResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def GetExportedSignalAttributeBool(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/nidaqmx_grpc.NiDAQmx/GetExportedSignalAttributeBool',
            nidaqmx__pb2.GetExportedSignalAttributeBoolRequest.SerializeToString,
            nidaqmx__pb2.GetExportedSignalAttributeBoolResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def GetExportedSignalAttributeDouble(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/nidaqmx_grpc.NiDAQmx/GetExportedSignalAttributeDouble',
            nidaqmx__pb2.GetExportedSignalAttributeDoubleRequest.SerializeToString,
            nidaqmx__pb2.GetExportedSignalAttributeDoubleResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def GetExportedSignalAttributeInt32(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/nidaqmx_grpc.NiDAQmx/GetExportedSignalAttributeInt32',
            nidaqmx__pb2.GetExportedSignalAttributeInt32Request.SerializeToString,
            nidaqmx__pb2.GetExportedSignalAttributeInt32Response.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def GetExportedSignalAttributeString(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/nidaqmx_grpc.NiDAQmx/GetExportedSignalAttributeString',
            nidaqmx__pb2.GetExportedSignalAttributeStringRequest.SerializeToString,
            nidaqmx__pb2.GetExportedSignalAttributeStringResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def GetExportedSignalAttributeUInt32(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/nidaqmx_grpc.NiDAQmx/GetExportedSignalAttributeUInt32',
            nidaqmx__pb2.GetExportedSignalAttributeUInt32Request.SerializeToString,
            nidaqmx__pb2.GetExportedSignalAttributeUInt32Response.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def GetExtCalLastDateAndTime(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/nidaqmx_grpc.NiDAQmx/GetExtCalLastDateAndTime',
            nidaqmx__pb2.GetExtCalLastDateAndTimeRequest.SerializeToString,
            nidaqmx__pb2.GetExtCalLastDateAndTimeResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def GetFirstSampClkWhen(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/nidaqmx_grpc.NiDAQmx/GetFirstSampClkWhen',
            nidaqmx__pb2.GetFirstSampClkWhenRequest.SerializeToString,
            nidaqmx__pb2.GetFirstSampClkWhenResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def GetFirstSampTimestampVal(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/nidaqmx_grpc.NiDAQmx/GetFirstSampTimestampVal',
            nidaqmx__pb2.GetFirstSampTimestampValRequest.SerializeToString,
            nidaqmx__pb2.GetFirstSampTimestampValResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def GetNthTaskChannel(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/nidaqmx_grpc.NiDAQmx/GetNthTaskChannel',
            nidaqmx__pb2.GetNthTaskChannelRequest.SerializeToString,
            nidaqmx__pb2.GetNthTaskChannelResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def GetNthTaskDevice(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/nidaqmx_grpc.NiDAQmx/GetNthTaskDevice',
            nidaqmx__pb2.GetNthTaskDeviceRequest.SerializeToString,
            nidaqmx__pb2.GetNthTaskDeviceResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def GetNthTaskReadChannel(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/nidaqmx_grpc.NiDAQmx/GetNthTaskReadChannel',
            nidaqmx__pb2.GetNthTaskReadChannelRequest.SerializeToString,
            nidaqmx__pb2.GetNthTaskReadChannelResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def GetPersistedChanAttributeBool(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/nidaqmx_grpc.NiDAQmx/GetPersistedChanAttributeBool',
            nidaqmx__pb2.GetPersistedChanAttributeBoolRequest.SerializeToString,
            nidaqmx__pb2.GetPersistedChanAttributeBoolResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def GetPersistedChanAttributeString(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/nidaqmx_grpc.NiDAQmx/GetPersistedChanAttributeString',
            nidaqmx__pb2.GetPersistedChanAttributeStringRequest.SerializeToString,
            nidaqmx__pb2.GetPersistedChanAttributeStringResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def GetPersistedScaleAttributeBool(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/nidaqmx_grpc.NiDAQmx/GetPersistedScaleAttributeBool',
            nidaqmx__pb2.GetPersistedScaleAttributeBoolRequest.SerializeToString,
            nidaqmx__pb2.GetPersistedScaleAttributeBoolResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def GetPersistedScaleAttributeString(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/nidaqmx_grpc.NiDAQmx/GetPersistedScaleAttributeString',
            nidaqmx__pb2.GetPersistedScaleAttributeStringRequest.SerializeToString,
            nidaqmx__pb2.GetPersistedScaleAttributeStringResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def GetPersistedTaskAttributeBool(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/nidaqmx_grpc.NiDAQmx/GetPersistedTaskAttributeBool',
            nidaqmx__pb2.GetPersistedTaskAttributeBoolRequest.SerializeToString,
            nidaqmx__pb2.GetPersistedTaskAttributeBoolResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def GetPersistedTaskAttributeString(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/nidaqmx_grpc.NiDAQmx/GetPersistedTaskAttributeString',
            nidaqmx__pb2.GetPersistedTaskAttributeStringRequest.SerializeToString,
            nidaqmx__pb2.GetPersistedTaskAttributeStringResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def GetPhysicalChanAttributeBool(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/nidaqmx_grpc.NiDAQmx/GetPhysicalChanAttributeBool',
            nidaqmx__pb2.GetPhysicalChanAttributeBoolRequest.SerializeToString,
            nidaqmx__pb2.GetPhysicalChanAttributeBoolResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def GetPhysicalChanAttributeBytes(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/nidaqmx_grpc.NiDAQmx/GetPhysicalChanAttributeBytes',
            nidaqmx__pb2.GetPhysicalChanAttributeBytesRequest.SerializeToString,
            nidaqmx__pb2.GetPhysicalChanAttributeBytesResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def GetPhysicalChanAttributeDouble(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/nidaqmx_grpc.NiDAQmx/GetPhysicalChanAttributeDouble',
            nidaqmx__pb2.GetPhysicalChanAttributeDoubleRequest.SerializeToString,
            nidaqmx__pb2.GetPhysicalChanAttributeDoubleResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def GetPhysicalChanAttributeDoubleArray(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/nidaqmx_grpc.NiDAQmx/GetPhysicalChanAttributeDoubleArray',
            nidaqmx__pb2.GetPhysicalChanAttributeDoubleArrayRequest.SerializeToString,
            nidaqmx__pb2.GetPhysicalChanAttributeDoubleArrayResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def GetPhysicalChanAttributeInt32(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/nidaqmx_grpc.NiDAQmx/GetPhysicalChanAttributeInt32',
            nidaqmx__pb2.GetPhysicalChanAttributeInt32Request.SerializeToString,
            nidaqmx__pb2.GetPhysicalChanAttributeInt32Response.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def GetPhysicalChanAttributeInt32Array(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/nidaqmx_grpc.NiDAQmx/GetPhysicalChanAttributeInt32Array',
            nidaqmx__pb2.GetPhysicalChanAttributeInt32ArrayRequest.SerializeToString,
            nidaqmx__pb2.GetPhysicalChanAttributeInt32ArrayResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def GetPhysicalChanAttributeString(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/nidaqmx_grpc.NiDAQmx/GetPhysicalChanAttributeString',
            nidaqmx__pb2.GetPhysicalChanAttributeStringRequest.SerializeToString,
            nidaqmx__pb2.GetPhysicalChanAttributeStringResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def GetPhysicalChanAttributeUInt32(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/nidaqmx_grpc.NiDAQmx/GetPhysicalChanAttributeUInt32',
            nidaqmx__pb2.GetPhysicalChanAttributeUInt32Request.SerializeToString,
            nidaqmx__pb2.GetPhysicalChanAttributeUInt32Response.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def GetPhysicalChanAttributeUInt32Array(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/nidaqmx_grpc.NiDAQmx/GetPhysicalChanAttributeUInt32Array',
            nidaqmx__pb2.GetPhysicalChanAttributeUInt32ArrayRequest.SerializeToString,
            nidaqmx__pb2.GetPhysicalChanAttributeUInt32ArrayResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def GetReadAttributeBool(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/nidaqmx_grpc.NiDAQmx/GetReadAttributeBool',
            nidaqmx__pb2.GetReadAttributeBoolRequest.SerializeToString,
            nidaqmx__pb2.GetReadAttributeBoolResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def GetReadAttributeDouble(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/nidaqmx_grpc.NiDAQmx/GetReadAttributeDouble',
            nidaqmx__pb2.GetReadAttributeDoubleRequest.SerializeToString,
            nidaqmx__pb2.GetReadAttributeDoubleResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def GetReadAttributeInt32(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/nidaqmx_grpc.NiDAQmx/GetReadAttributeInt32',
            nidaqmx__pb2.GetReadAttributeInt32Request.SerializeToString,
            nidaqmx__pb2.GetReadAttributeInt32Response.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def GetReadAttributeString(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/nidaqmx_grpc.NiDAQmx/GetReadAttributeString',
            nidaqmx__pb2.GetReadAttributeStringRequest.SerializeToString,
            nidaqmx__pb2.GetReadAttributeStringResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def GetReadAttributeUInt32(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/nidaqmx_grpc.NiDAQmx/GetReadAttributeUInt32',
            nidaqmx__pb2.GetReadAttributeUInt32Request.SerializeToString,
            nidaqmx__pb2.GetReadAttributeUInt32Response.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def GetReadAttributeUInt64(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/nidaqmx_grpc.NiDAQmx/GetReadAttributeUInt64',
            nidaqmx__pb2.GetReadAttributeUInt64Request.SerializeToString,
            nidaqmx__pb2.GetReadAttributeUInt64Response.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def GetRealTimeAttributeBool(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/nidaqmx_grpc.NiDAQmx/GetRealTimeAttributeBool',
            nidaqmx__pb2.GetRealTimeAttributeBoolRequest.SerializeToString,
            nidaqmx__pb2.GetRealTimeAttributeBoolResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def GetRealTimeAttributeInt32(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/nidaqmx_grpc.NiDAQmx/GetRealTimeAttributeInt32',
            nidaqmx__pb2.GetRealTimeAttributeInt32Request.SerializeToString,
            nidaqmx__pb2.GetRealTimeAttributeInt32Response.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def GetRealTimeAttributeUInt32(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/nidaqmx_grpc.NiDAQmx/GetRealTimeAttributeUInt32',
            nidaqmx__pb2.GetRealTimeAttributeUInt32Request.SerializeToString,
            nidaqmx__pb2.GetRealTimeAttributeUInt32Response.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def GetRefTrigTimestampVal(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/nidaqmx_grpc.NiDAQmx/GetRefTrigTimestampVal',
            nidaqmx__pb2.GetRefTrigTimestampValRequest.SerializeToString,
            nidaqmx__pb2.GetRefTrigTimestampValResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def GetScaleAttributeDouble(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/nidaqmx_grpc.NiDAQmx/GetScaleAttributeDouble',
            nidaqmx__pb2.GetScaleAttributeDoubleRequest.SerializeToString,
            nidaqmx__pb2.GetScaleAttributeDoubleResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def GetScaleAttributeDoubleArray(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/nidaqmx_grpc.NiDAQmx/GetScaleAttributeDoubleArray',
            nidaqmx__pb2.GetScaleAttributeDoubleArrayRequest.SerializeToString,
            nidaqmx__pb2.GetScaleAttributeDoubleArrayResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def GetScaleAttributeInt32(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/nidaqmx_grpc.NiDAQmx/GetScaleAttributeInt32',
            nidaqmx__pb2.GetScaleAttributeInt32Request.SerializeToString,
            nidaqmx__pb2.GetScaleAttributeInt32Response.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def GetScaleAttributeString(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/nidaqmx_grpc.NiDAQmx/GetScaleAttributeString',
            nidaqmx__pb2.GetScaleAttributeStringRequest.SerializeToString,
            nidaqmx__pb2.GetScaleAttributeStringResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def GetSelfCalLastDateAndTime(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/nidaqmx_grpc.NiDAQmx/GetSelfCalLastDateAndTime',
            nidaqmx__pb2.GetSelfCalLastDateAndTimeRequest.SerializeToString,
            nidaqmx__pb2.GetSelfCalLastDateAndTimeResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def GetStartTrigTimestampVal(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/nidaqmx_grpc.NiDAQmx/GetStartTrigTimestampVal',
            nidaqmx__pb2.GetStartTrigTimestampValRequest.SerializeToString,
            nidaqmx__pb2.GetStartTrigTimestampValResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def GetStartTrigTrigWhen(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/nidaqmx_grpc.NiDAQmx/GetStartTrigTrigWhen',
            nidaqmx__pb2.GetStartTrigTrigWhenRequest.SerializeToString,
            nidaqmx__pb2.GetStartTrigTrigWhenResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def GetSyncPulseTimeWhen(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/nidaqmx_grpc.NiDAQmx/GetSyncPulseTimeWhen',
            nidaqmx__pb2.GetSyncPulseTimeWhenRequest.SerializeToString,
            nidaqmx__pb2.GetSyncPulseTimeWhenResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def GetSystemInfoAttributeString(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/nidaqmx_grpc.NiDAQmx/GetSystemInfoAttributeString',
            nidaqmx__pb2.GetSystemInfoAttributeStringRequest.SerializeToString,
            nidaqmx__pb2.GetSystemInfoAttributeStringResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def GetSystemInfoAttributeUInt32(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/nidaqmx_grpc.NiDAQmx/GetSystemInfoAttributeUInt32',
            nidaqmx__pb2.GetSystemInfoAttributeUInt32Request.SerializeToString,
            nidaqmx__pb2.GetSystemInfoAttributeUInt32Response.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def GetTaskAttributeBool(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/nidaqmx_grpc.NiDAQmx/GetTaskAttributeBool',
            nidaqmx__pb2.GetTaskAttributeBoolRequest.SerializeToString,
            nidaqmx__pb2.GetTaskAttributeBoolResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def GetTaskAttributeString(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/nidaqmx_grpc.NiDAQmx/GetTaskAttributeString',
            nidaqmx__pb2.GetTaskAttributeStringRequest.SerializeToString,
            nidaqmx__pb2.GetTaskAttributeStringResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def GetTaskAttributeUInt32(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/nidaqmx_grpc.NiDAQmx/GetTaskAttributeUInt32',
            nidaqmx__pb2.GetTaskAttributeUInt32Request.SerializeToString,
            nidaqmx__pb2.GetTaskAttributeUInt32Response.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def GetTimingAttributeBool(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/nidaqmx_grpc.NiDAQmx/GetTimingAttributeBool',
            nidaqmx__pb2.GetTimingAttributeBoolRequest.SerializeToString,
            nidaqmx__pb2.GetTimingAttributeBoolResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def GetTimingAttributeDouble(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/nidaqmx_grpc.NiDAQmx/GetTimingAttributeDouble',
            nidaqmx__pb2.GetTimingAttributeDoubleRequest.SerializeToString,
            nidaqmx__pb2.GetTimingAttributeDoubleResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def GetTimingAttributeExBool(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/nidaqmx_grpc.NiDAQmx/GetTimingAttributeExBool',
            nidaqmx__pb2.GetTimingAttributeExBoolRequest.SerializeToString,
            nidaqmx__pb2.GetTimingAttributeExBoolResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def GetTimingAttributeExDouble(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/nidaqmx_grpc.NiDAQmx/GetTimingAttributeExDouble',
            nidaqmx__pb2.GetTimingAttributeExDoubleRequest.SerializeToString,
            nidaqmx__pb2.GetTimingAttributeExDoubleResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def GetTimingAttributeExInt32(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/nidaqmx_grpc.NiDAQmx/GetTimingAttributeExInt32',
            nidaqmx__pb2.GetTimingAttributeExInt32Request.SerializeToString,
            nidaqmx__pb2.GetTimingAttributeExInt32Response.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def GetTimingAttributeExString(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/nidaqmx_grpc.NiDAQmx/GetTimingAttributeExString',
            nidaqmx__pb2.GetTimingAttributeExStringRequest.SerializeToString,
            nidaqmx__pb2.GetTimingAttributeExStringResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def GetTimingAttributeExTimestamp(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/nidaqmx_grpc.NiDAQmx/GetTimingAttributeExTimestamp',
            nidaqmx__pb2.GetTimingAttributeExTimestampRequest.SerializeToString,
            nidaqmx__pb2.GetTimingAttributeExTimestampResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def GetTimingAttributeExUInt32(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/nidaqmx_grpc.NiDAQmx/GetTimingAttributeExUInt32',
            nidaqmx__pb2.GetTimingAttributeExUInt32Request.SerializeToString,
            nidaqmx__pb2.GetTimingAttributeExUInt32Response.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def GetTimingAttributeExUInt64(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/nidaqmx_grpc.NiDAQmx/GetTimingAttributeExUInt64',
            nidaqmx__pb2.GetTimingAttributeExUInt64Request.SerializeToString,
            nidaqmx__pb2.GetTimingAttributeExUInt64Response.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def GetTimingAttributeInt32(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/nidaqmx_grpc.NiDAQmx/GetTimingAttributeInt32',
            nidaqmx__pb2.GetTimingAttributeInt32Request.SerializeToString,
            nidaqmx__pb2.GetTimingAttributeInt32Response.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def GetTimingAttributeString(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/nidaqmx_grpc.NiDAQmx/GetTimingAttributeString',
            nidaqmx__pb2.GetTimingAttributeStringRequest.SerializeToString,
            nidaqmx__pb2.GetTimingAttributeStringResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def GetTimingAttributeTimestamp(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/nidaqmx_grpc.NiDAQmx/GetTimingAttributeTimestamp',
            nidaqmx__pb2.GetTimingAttributeTimestampRequest.SerializeToString,
            nidaqmx__pb2.GetTimingAttributeTimestampResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def GetTimingAttributeUInt32(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/nidaqmx_grpc.NiDAQmx/GetTimingAttributeUInt32',
            nidaqmx__pb2.GetTimingAttributeUInt32Request.SerializeToString,
            nidaqmx__pb2.GetTimingAttributeUInt32Response.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def GetTimingAttributeUInt64(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/nidaqmx_grpc.NiDAQmx/GetTimingAttributeUInt64',
            nidaqmx__pb2.GetTimingAttributeUInt64Request.SerializeToString,
            nidaqmx__pb2.GetTimingAttributeUInt64Response.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def GetTrigAttributeBool(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/nidaqmx_grpc.NiDAQmx/GetTrigAttributeBool',
            nidaqmx__pb2.GetTrigAttributeBoolRequest.SerializeToString,
            nidaqmx__pb2.GetTrigAttributeBoolResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def GetTrigAttributeDouble(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/nidaqmx_grpc.NiDAQmx/GetTrigAttributeDouble',
            nidaqmx__pb2.GetTrigAttributeDoubleRequest.SerializeToString,
            nidaqmx__pb2.GetTrigAttributeDoubleResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def GetTrigAttributeDoubleArray(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/nidaqmx_grpc.NiDAQmx/GetTrigAttributeDoubleArray',
            nidaqmx__pb2.GetTrigAttributeDoubleArrayRequest.SerializeToString,
            nidaqmx__pb2.GetTrigAttributeDoubleArrayResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def GetTrigAttributeInt32(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/nidaqmx_grpc.NiDAQmx/GetTrigAttributeInt32',
            nidaqmx__pb2.GetTrigAttributeInt32Request.SerializeToString,
            nidaqmx__pb2.GetTrigAttributeInt32Response.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def GetTrigAttributeInt32Array(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/nidaqmx_grpc.NiDAQmx/GetTrigAttributeInt32Array',
            nidaqmx__pb2.GetTrigAttributeInt32ArrayRequest.SerializeToString,
            nidaqmx__pb2.GetTrigAttributeInt32ArrayResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def GetTrigAttributeString(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/nidaqmx_grpc.NiDAQmx/GetTrigAttributeString',
            nidaqmx__pb2.GetTrigAttributeStringRequest.SerializeToString,
            nidaqmx__pb2.GetTrigAttributeStringResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def GetTrigAttributeTimestamp(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/nidaqmx_grpc.NiDAQmx/GetTrigAttributeTimestamp',
            nidaqmx__pb2.GetTrigAttributeTimestampRequest.SerializeToString,
            nidaqmx__pb2.GetTrigAttributeTimestampResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def GetTrigAttributeUInt32(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/nidaqmx_grpc.NiDAQmx/GetTrigAttributeUInt32',
            nidaqmx__pb2.GetTrigAttributeUInt32Request.SerializeToString,
            nidaqmx__pb2.GetTrigAttributeUInt32Response.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def GetWatchdogAttributeBool(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/nidaqmx_grpc.NiDAQmx/GetWatchdogAttributeBool',
            nidaqmx__pb2.GetWatchdogAttributeBoolRequest.SerializeToString,
            nidaqmx__pb2.GetWatchdogAttributeBoolResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def GetWatchdogAttributeDouble(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/nidaqmx_grpc.NiDAQmx/GetWatchdogAttributeDouble',
            nidaqmx__pb2.GetWatchdogAttributeDoubleRequest.SerializeToString,
            nidaqmx__pb2.GetWatchdogAttributeDoubleResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def GetWatchdogAttributeInt32(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/nidaqmx_grpc.NiDAQmx/GetWatchdogAttributeInt32',
            nidaqmx__pb2.GetWatchdogAttributeInt32Request.SerializeToString,
            nidaqmx__pb2.GetWatchdogAttributeInt32Response.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def GetWatchdogAttributeString(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/nidaqmx_grpc.NiDAQmx/GetWatchdogAttributeString',
            nidaqmx__pb2.GetWatchdogAttributeStringRequest.SerializeToString,
            nidaqmx__pb2.GetWatchdogAttributeStringResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def GetWriteAttributeBool(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/nidaqmx_grpc.NiDAQmx/GetWriteAttributeBool',
            nidaqmx__pb2.GetWriteAttributeBoolRequest.SerializeToString,
            nidaqmx__pb2.GetWriteAttributeBoolResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def GetWriteAttributeDouble(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/nidaqmx_grpc.NiDAQmx/GetWriteAttributeDouble',
            nidaqmx__pb2.GetWriteAttributeDoubleRequest.SerializeToString,
            nidaqmx__pb2.GetWriteAttributeDoubleResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def GetWriteAttributeInt32(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/nidaqmx_grpc.NiDAQmx/GetWriteAttributeInt32',
            nidaqmx__pb2.GetWriteAttributeInt32Request.SerializeToString,
            nidaqmx__pb2.GetWriteAttributeInt32Response.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def GetWriteAttributeString(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/nidaqmx_grpc.NiDAQmx/GetWriteAttributeString',
            nidaqmx__pb2.GetWriteAttributeStringRequest.SerializeToString,
            nidaqmx__pb2.GetWriteAttributeStringResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def GetWriteAttributeUInt32(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/nidaqmx_grpc.NiDAQmx/GetWriteAttributeUInt32',
            nidaqmx__pb2.GetWriteAttributeUInt32Request.SerializeToString,
            nidaqmx__pb2.GetWriteAttributeUInt32Response.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def GetWriteAttributeUInt64(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/nidaqmx_grpc.NiDAQmx/GetWriteAttributeUInt64',
            nidaqmx__pb2.GetWriteAttributeUInt64Request.SerializeToString,
            nidaqmx__pb2.GetWriteAttributeUInt64Response.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def IsTaskDone(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/nidaqmx_grpc.NiDAQmx/IsTaskDone',
            nidaqmx__pb2.IsTaskDoneRequest.SerializeToString,
            nidaqmx__pb2.IsTaskDoneResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def LoadTask(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/nidaqmx_grpc.NiDAQmx/LoadTask',
            nidaqmx__pb2.LoadTaskRequest.SerializeToString,
            nidaqmx__pb2.LoadTaskResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def PerformBridgeOffsetNullingCalEx(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/nidaqmx_grpc.NiDAQmx/PerformBridgeOffsetNullingCalEx',
            nidaqmx__pb2.PerformBridgeOffsetNullingCalExRequest.SerializeToString,
            nidaqmx__pb2.PerformBridgeOffsetNullingCalExResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def PerformBridgeShuntCalEx(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/nidaqmx_grpc.NiDAQmx/PerformBridgeShuntCalEx',
            nidaqmx__pb2.PerformBridgeShuntCalExRequest.SerializeToString,
            nidaqmx__pb2.PerformBridgeShuntCalExResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def PerformStrainShuntCalEx(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/nidaqmx_grpc.NiDAQmx/PerformStrainShuntCalEx',
            nidaqmx__pb2.PerformStrainShuntCalExRequest.SerializeToString,
            nidaqmx__pb2.PerformStrainShuntCalExResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def PerformThrmcplLeadOffsetNullingCal(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/nidaqmx_grpc.NiDAQmx/PerformThrmcplLeadOffsetNullingCal',
            nidaqmx__pb2.PerformThrmcplLeadOffsetNullingCalRequest.SerializeToString,
            nidaqmx__pb2.PerformThrmcplLeadOffsetNullingCalResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def ReadAnalogF64(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/nidaqmx_grpc.NiDAQmx/ReadAnalogF64',
            nidaqmx__pb2.ReadAnalogF64Request.SerializeToString,
            nidaqmx__pb2.ReadAnalogF64Response.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def ReadAnalogScalarF64(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/nidaqmx_grpc.NiDAQmx/ReadAnalogScalarF64',
            nidaqmx__pb2.ReadAnalogScalarF64Request.SerializeToString,
            nidaqmx__pb2.ReadAnalogScalarF64Response.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def ReadBinaryI16(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/nidaqmx_grpc.NiDAQmx/ReadBinaryI16',
            nidaqmx__pb2.ReadBinaryI16Request.SerializeToString,
            nidaqmx__pb2.ReadBinaryI16Response.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def ReadBinaryI32(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/nidaqmx_grpc.NiDAQmx/ReadBinaryI32',
            nidaqmx__pb2.ReadBinaryI32Request.SerializeToString,
            nidaqmx__pb2.ReadBinaryI32Response.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def ReadBinaryU16(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/nidaqmx_grpc.NiDAQmx/ReadBinaryU16',
            nidaqmx__pb2.ReadBinaryU16Request.SerializeToString,
            nidaqmx__pb2.ReadBinaryU16Response.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def ReadBinaryU32(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/nidaqmx_grpc.NiDAQmx/ReadBinaryU32',
            nidaqmx__pb2.ReadBinaryU32Request.SerializeToString,
            nidaqmx__pb2.ReadBinaryU32Response.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def ReadCounterF64(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/nidaqmx_grpc.NiDAQmx/ReadCounterF64',
            nidaqmx__pb2.ReadCounterF64Request.SerializeToString,
            nidaqmx__pb2.ReadCounterF64Response.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def ReadCounterF64Ex(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/nidaqmx_grpc.NiDAQmx/ReadCounterF64Ex',
            nidaqmx__pb2.ReadCounterF64ExRequest.SerializeToString,
            nidaqmx__pb2.ReadCounterF64ExResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def ReadCounterScalarF64(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/nidaqmx_grpc.NiDAQmx/ReadCounterScalarF64',
            nidaqmx__pb2.ReadCounterScalarF64Request.SerializeToString,
            nidaqmx__pb2.ReadCounterScalarF64Response.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def ReadCounterScalarU32(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/nidaqmx_grpc.NiDAQmx/ReadCounterScalarU32',
            nidaqmx__pb2.ReadCounterScalarU32Request.SerializeToString,
            nidaqmx__pb2.ReadCounterScalarU32Response.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def ReadCounterU32(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/nidaqmx_grpc.NiDAQmx/ReadCounterU32',
            nidaqmx__pb2.ReadCounterU32Request.SerializeToString,
            nidaqmx__pb2.ReadCounterU32Response.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def ReadCounterU32Ex(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/nidaqmx_grpc.NiDAQmx/ReadCounterU32Ex',
            nidaqmx__pb2.ReadCounterU32ExRequest.SerializeToString,
            nidaqmx__pb2.ReadCounterU32ExResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def ReadCtrFreq(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/nidaqmx_grpc.NiDAQmx/ReadCtrFreq',
            nidaqmx__pb2.ReadCtrFreqRequest.SerializeToString,
            nidaqmx__pb2.ReadCtrFreqResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def ReadCtrFreqScalar(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/nidaqmx_grpc.NiDAQmx/ReadCtrFreqScalar',
            nidaqmx__pb2.ReadCtrFreqScalarRequest.SerializeToString,
            nidaqmx__pb2.ReadCtrFreqScalarResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def ReadCtrTicks(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/nidaqmx_grpc.NiDAQmx/ReadCtrTicks',
            nidaqmx__pb2.ReadCtrTicksRequest.SerializeToString,
            nidaqmx__pb2.ReadCtrTicksResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def ReadCtrTicksScalar(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/nidaqmx_grpc.NiDAQmx/ReadCtrTicksScalar',
            nidaqmx__pb2.ReadCtrTicksScalarRequest.SerializeToString,
            nidaqmx__pb2.ReadCtrTicksScalarResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def ReadCtrTime(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/nidaqmx_grpc.NiDAQmx/ReadCtrTime',
            nidaqmx__pb2.ReadCtrTimeRequest.SerializeToString,
            nidaqmx__pb2.ReadCtrTimeResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def ReadCtrTimeScalar(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/nidaqmx_grpc.NiDAQmx/ReadCtrTimeScalar',
            nidaqmx__pb2.ReadCtrTimeScalarRequest.SerializeToString,
            nidaqmx__pb2.ReadCtrTimeScalarResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def ReadDigitalLines(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/nidaqmx_grpc.NiDAQmx/ReadDigitalLines',
            nidaqmx__pb2.ReadDigitalLinesRequest.SerializeToString,
            nidaqmx__pb2.ReadDigitalLinesResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def ReadDigitalScalarU32(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/nidaqmx_grpc.NiDAQmx/ReadDigitalScalarU32',
            nidaqmx__pb2.ReadDigitalScalarU32Request.SerializeToString,
            nidaqmx__pb2.ReadDigitalScalarU32Response.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def ReadDigitalU16(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/nidaqmx_grpc.NiDAQmx/ReadDigitalU16',
            nidaqmx__pb2.ReadDigitalU16Request.SerializeToString,
            nidaqmx__pb2.ReadDigitalU16Response.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def ReadDigitalU32(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/nidaqmx_grpc.NiDAQmx/ReadDigitalU32',
            nidaqmx__pb2.ReadDigitalU32Request.SerializeToString,
            nidaqmx__pb2.ReadDigitalU32Response.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def ReadDigitalU8(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/nidaqmx_grpc.NiDAQmx/ReadDigitalU8',
            nidaqmx__pb2.ReadDigitalU8Request.SerializeToString,
            nidaqmx__pb2.ReadDigitalU8Response.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def ReadPowerBinaryI16(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/nidaqmx_grpc.NiDAQmx/ReadPowerBinaryI16',
            nidaqmx__pb2.ReadPowerBinaryI16Request.SerializeToString,
            nidaqmx__pb2.ReadPowerBinaryI16Response.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def ReadPowerF64(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/nidaqmx_grpc.NiDAQmx/ReadPowerF64',
            nidaqmx__pb2.ReadPowerF64Request.SerializeToString,
            nidaqmx__pb2.ReadPowerF64Response.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def ReadPowerScalarF64(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/nidaqmx_grpc.NiDAQmx/ReadPowerScalarF64',
            nidaqmx__pb2.ReadPowerScalarF64Request.SerializeToString,
            nidaqmx__pb2.ReadPowerScalarF64Response.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def ReadRaw(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/nidaqmx_grpc.NiDAQmx/ReadRaw',
            nidaqmx__pb2.ReadRawRequest.SerializeToString,
            nidaqmx__pb2.ReadRawResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def RegisterDoneEvent(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_stream(request, target, '/nidaqmx_grpc.NiDAQmx/RegisterDoneEvent',
            nidaqmx__pb2.RegisterDoneEventRequest.SerializeToString,
            nidaqmx__pb2.RegisterDoneEventResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def RegisterEveryNSamplesEvent(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_stream(request, target, '/nidaqmx_grpc.NiDAQmx/RegisterEveryNSamplesEvent',
            nidaqmx__pb2.RegisterEveryNSamplesEventRequest.SerializeToString,
            nidaqmx__pb2.RegisterEveryNSamplesEventResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def RegisterSignalEvent(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_stream(request, target, '/nidaqmx_grpc.NiDAQmx/RegisterSignalEvent',
            nidaqmx__pb2.RegisterSignalEventRequest.SerializeToString,
            nidaqmx__pb2.RegisterSignalEventResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def RemoveCDAQSyncConnection(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/nidaqmx_grpc.NiDAQmx/RemoveCDAQSyncConnection',
            nidaqmx__pb2.RemoveCDAQSyncConnectionRequest.SerializeToString,
            nidaqmx__pb2.RemoveCDAQSyncConnectionResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def ReserveNetworkDevice(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/nidaqmx_grpc.NiDAQmx/ReserveNetworkDevice',
            nidaqmx__pb2.ReserveNetworkDeviceRequest.SerializeToString,
            nidaqmx__pb2.ReserveNetworkDeviceResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def ResetBufferAttribute(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/nidaqmx_grpc.NiDAQmx/ResetBufferAttribute',
            nidaqmx__pb2.ResetBufferAttributeRequest.SerializeToString,
            nidaqmx__pb2.ResetBufferAttributeResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def ResetChanAttribute(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/nidaqmx_grpc.NiDAQmx/ResetChanAttribute',
            nidaqmx__pb2.ResetChanAttributeRequest.SerializeToString,
            nidaqmx__pb2.ResetChanAttributeResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def ResetDevice(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/nidaqmx_grpc.NiDAQmx/ResetDevice',
            nidaqmx__pb2.ResetDeviceRequest.SerializeToString,
            nidaqmx__pb2.ResetDeviceResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def ResetExportedSignalAttribute(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/nidaqmx_grpc.NiDAQmx/ResetExportedSignalAttribute',
            nidaqmx__pb2.ResetExportedSignalAttributeRequest.SerializeToString,
            nidaqmx__pb2.ResetExportedSignalAttributeResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def ResetReadAttribute(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/nidaqmx_grpc.NiDAQmx/ResetReadAttribute',
            nidaqmx__pb2.ResetReadAttributeRequest.SerializeToString,
            nidaqmx__pb2.ResetReadAttributeResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def ResetRealTimeAttribute(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/nidaqmx_grpc.NiDAQmx/ResetRealTimeAttribute',
            nidaqmx__pb2.ResetRealTimeAttributeRequest.SerializeToString,
            nidaqmx__pb2.ResetRealTimeAttributeResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def ResetTimingAttribute(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/nidaqmx_grpc.NiDAQmx/ResetTimingAttribute',
            nidaqmx__pb2.ResetTimingAttributeRequest.SerializeToString,
            nidaqmx__pb2.ResetTimingAttributeResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def ResetTimingAttributeEx(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/nidaqmx_grpc.NiDAQmx/ResetTimingAttributeEx',
            nidaqmx__pb2.ResetTimingAttributeExRequest.SerializeToString,
            nidaqmx__pb2.ResetTimingAttributeExResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def ResetTrigAttribute(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/nidaqmx_grpc.NiDAQmx/ResetTrigAttribute',
            nidaqmx__pb2.ResetTrigAttributeRequest.SerializeToString,
            nidaqmx__pb2.ResetTrigAttributeResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def ResetWatchdogAttribute(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/nidaqmx_grpc.NiDAQmx/ResetWatchdogAttribute',
            nidaqmx__pb2.ResetWatchdogAttributeRequest.SerializeToString,
            nidaqmx__pb2.ResetWatchdogAttributeResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def ResetWriteAttribute(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/nidaqmx_grpc.NiDAQmx/ResetWriteAttribute',
            nidaqmx__pb2.ResetWriteAttributeRequest.SerializeToString,
            nidaqmx__pb2.ResetWriteAttributeResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def RestoreLastExtCalConst(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/nidaqmx_grpc.NiDAQmx/RestoreLastExtCalConst',
            nidaqmx__pb2.RestoreLastExtCalConstRequest.SerializeToString,
            nidaqmx__pb2.RestoreLastExtCalConstResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def SaveGlobalChan(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/nidaqmx_grpc.NiDAQmx/SaveGlobalChan',
            nidaqmx__pb2.SaveGlobalChanRequest.SerializeToString,
            nidaqmx__pb2.SaveGlobalChanResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def SaveScale(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/nidaqmx_grpc.NiDAQmx/SaveScale',
            nidaqmx__pb2.SaveScaleRequest.SerializeToString,
            nidaqmx__pb2.SaveScaleResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def SaveTask(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/nidaqmx_grpc.NiDAQmx/SaveTask',
            nidaqmx__pb2.SaveTaskRequest.SerializeToString,
            nidaqmx__pb2.SaveTaskResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def SelfCal(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/nidaqmx_grpc.NiDAQmx/SelfCal',
            nidaqmx__pb2.SelfCalRequest.SerializeToString,
            nidaqmx__pb2.SelfCalResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def SelfTestDevice(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/nidaqmx_grpc.NiDAQmx/SelfTestDevice',
            nidaqmx__pb2.SelfTestDeviceRequest.SerializeToString,
            nidaqmx__pb2.SelfTestDeviceResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def SetAIChanCalCalDate(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/nidaqmx_grpc.NiDAQmx/SetAIChanCalCalDate',
            nidaqmx__pb2.SetAIChanCalCalDateRequest.SerializeToString,
            nidaqmx__pb2.SetAIChanCalCalDateResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def SetAIChanCalExpDate(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/nidaqmx_grpc.NiDAQmx/SetAIChanCalExpDate',
            nidaqmx__pb2.SetAIChanCalExpDateRequest.SerializeToString,
            nidaqmx__pb2.SetAIChanCalExpDateResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def SetAnalogPowerUpStates(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/nidaqmx_grpc.NiDAQmx/SetAnalogPowerUpStates',
            nidaqmx__pb2.SetAnalogPowerUpStatesRequest.SerializeToString,
            nidaqmx__pb2.SetAnalogPowerUpStatesResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def SetAnalogPowerUpStatesWithOutputType(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/nidaqmx_grpc.NiDAQmx/SetAnalogPowerUpStatesWithOutputType',
            nidaqmx__pb2.SetAnalogPowerUpStatesWithOutputTypeRequest.SerializeToString,
            nidaqmx__pb2.SetAnalogPowerUpStatesWithOutputTypeResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def SetArmStartTrigTrigWhen(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/nidaqmx_grpc.NiDAQmx/SetArmStartTrigTrigWhen',
            nidaqmx__pb2.SetArmStartTrigTrigWhenRequest.SerializeToString,
            nidaqmx__pb2.SetArmStartTrigTrigWhenResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def SetBufferAttributeUInt32(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/nidaqmx_grpc.NiDAQmx/SetBufferAttributeUInt32',
            nidaqmx__pb2.SetBufferAttributeUInt32Request.SerializeToString,
            nidaqmx__pb2.SetBufferAttributeUInt32Response.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def SetCalInfoAttributeBool(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/nidaqmx_grpc.NiDAQmx/SetCalInfoAttributeBool',
            nidaqmx__pb2.SetCalInfoAttributeBoolRequest.SerializeToString,
            nidaqmx__pb2.SetCalInfoAttributeBoolResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def SetCalInfoAttributeDouble(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/nidaqmx_grpc.NiDAQmx/SetCalInfoAttributeDouble',
            nidaqmx__pb2.SetCalInfoAttributeDoubleRequest.SerializeToString,
            nidaqmx__pb2.SetCalInfoAttributeDoubleResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def SetCalInfoAttributeString(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/nidaqmx_grpc.NiDAQmx/SetCalInfoAttributeString',
            nidaqmx__pb2.SetCalInfoAttributeStringRequest.SerializeToString,
            nidaqmx__pb2.SetCalInfoAttributeStringResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def SetCalInfoAttributeUInt32(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/nidaqmx_grpc.NiDAQmx/SetCalInfoAttributeUInt32',
            nidaqmx__pb2.SetCalInfoAttributeUInt32Request.SerializeToString,
            nidaqmx__pb2.SetCalInfoAttributeUInt32Response.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def SetChanAttributeBool(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/nidaqmx_grpc.NiDAQmx/SetChanAttributeBool',
            nidaqmx__pb2.SetChanAttributeBoolRequest.SerializeToString,
            nidaqmx__pb2.SetChanAttributeBoolResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def SetChanAttributeDouble(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/nidaqmx_grpc.NiDAQmx/SetChanAttributeDouble',
            nidaqmx__pb2.SetChanAttributeDoubleRequest.SerializeToString,
            nidaqmx__pb2.SetChanAttributeDoubleResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def SetChanAttributeDoubleArray(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/nidaqmx_grpc.NiDAQmx/SetChanAttributeDoubleArray',
            nidaqmx__pb2.SetChanAttributeDoubleArrayRequest.SerializeToString,
            nidaqmx__pb2.SetChanAttributeDoubleArrayResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def SetChanAttributeInt32(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/nidaqmx_grpc.NiDAQmx/SetChanAttributeInt32',
            nidaqmx__pb2.SetChanAttributeInt32Request.SerializeToString,
            nidaqmx__pb2.SetChanAttributeInt32Response.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def SetChanAttributeString(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/nidaqmx_grpc.NiDAQmx/SetChanAttributeString',
            nidaqmx__pb2.SetChanAttributeStringRequest.SerializeToString,
            nidaqmx__pb2.SetChanAttributeStringResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def SetChanAttributeUInt32(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/nidaqmx_grpc.NiDAQmx/SetChanAttributeUInt32',
            nidaqmx__pb2.SetChanAttributeUInt32Request.SerializeToString,
            nidaqmx__pb2.SetChanAttributeUInt32Response.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def SetDigitalLogicFamilyPowerUpState(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/nidaqmx_grpc.NiDAQmx/SetDigitalLogicFamilyPowerUpState',
            nidaqmx__pb2.SetDigitalLogicFamilyPowerUpStateRequest.SerializeToString,
            nidaqmx__pb2.SetDigitalLogicFamilyPowerUpStateResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def SetDigitalPowerUpStates(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/nidaqmx_grpc.NiDAQmx/SetDigitalPowerUpStates',
            nidaqmx__pb2.SetDigitalPowerUpStatesRequest.SerializeToString,
            nidaqmx__pb2.SetDigitalPowerUpStatesResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def SetDigitalPullUpPullDownStates(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/nidaqmx_grpc.NiDAQmx/SetDigitalPullUpPullDownStates',
            nidaqmx__pb2.SetDigitalPullUpPullDownStatesRequest.SerializeToString,
            nidaqmx__pb2.SetDigitalPullUpPullDownStatesResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def SetExportedSignalAttributeBool(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/nidaqmx_grpc.NiDAQmx/SetExportedSignalAttributeBool',
            nidaqmx__pb2.SetExportedSignalAttributeBoolRequest.SerializeToString,
            nidaqmx__pb2.SetExportedSignalAttributeBoolResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def SetExportedSignalAttributeDouble(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/nidaqmx_grpc.NiDAQmx/SetExportedSignalAttributeDouble',
            nidaqmx__pb2.SetExportedSignalAttributeDoubleRequest.SerializeToString,
            nidaqmx__pb2.SetExportedSignalAttributeDoubleResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def SetExportedSignalAttributeInt32(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/nidaqmx_grpc.NiDAQmx/SetExportedSignalAttributeInt32',
            nidaqmx__pb2.SetExportedSignalAttributeInt32Request.SerializeToString,
            nidaqmx__pb2.SetExportedSignalAttributeInt32Response.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def SetExportedSignalAttributeString(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/nidaqmx_grpc.NiDAQmx/SetExportedSignalAttributeString',
            nidaqmx__pb2.SetExportedSignalAttributeStringRequest.SerializeToString,
            nidaqmx__pb2.SetExportedSignalAttributeStringResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def SetExportedSignalAttributeUInt32(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/nidaqmx_grpc.NiDAQmx/SetExportedSignalAttributeUInt32',
            nidaqmx__pb2.SetExportedSignalAttributeUInt32Request.SerializeToString,
            nidaqmx__pb2.SetExportedSignalAttributeUInt32Response.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def SetFirstSampClkWhen(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/nidaqmx_grpc.NiDAQmx/SetFirstSampClkWhen',
            nidaqmx__pb2.SetFirstSampClkWhenRequest.SerializeToString,
            nidaqmx__pb2.SetFirstSampClkWhenResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def SetReadAttributeBool(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/nidaqmx_grpc.NiDAQmx/SetReadAttributeBool',
            nidaqmx__pb2.SetReadAttributeBoolRequest.SerializeToString,
            nidaqmx__pb2.SetReadAttributeBoolResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def SetReadAttributeDouble(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/nidaqmx_grpc.NiDAQmx/SetReadAttributeDouble',
            nidaqmx__pb2.SetReadAttributeDoubleRequest.SerializeToString,
            nidaqmx__pb2.SetReadAttributeDoubleResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def SetReadAttributeInt32(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/nidaqmx_grpc.NiDAQmx/SetReadAttributeInt32',
            nidaqmx__pb2.SetReadAttributeInt32Request.SerializeToString,
            nidaqmx__pb2.SetReadAttributeInt32Response.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def SetReadAttributeString(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/nidaqmx_grpc.NiDAQmx/SetReadAttributeString',
            nidaqmx__pb2.SetReadAttributeStringRequest.SerializeToString,
            nidaqmx__pb2.SetReadAttributeStringResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def SetReadAttributeUInt32(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/nidaqmx_grpc.NiDAQmx/SetReadAttributeUInt32',
            nidaqmx__pb2.SetReadAttributeUInt32Request.SerializeToString,
            nidaqmx__pb2.SetReadAttributeUInt32Response.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def SetReadAttributeUInt64(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/nidaqmx_grpc.NiDAQmx/SetReadAttributeUInt64',
            nidaqmx__pb2.SetReadAttributeUInt64Request.SerializeToString,
            nidaqmx__pb2.SetReadAttributeUInt64Response.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def SetRealTimeAttributeBool(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/nidaqmx_grpc.NiDAQmx/SetRealTimeAttributeBool',
            nidaqmx__pb2.SetRealTimeAttributeBoolRequest.SerializeToString,
            nidaqmx__pb2.SetRealTimeAttributeBoolResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def SetRealTimeAttributeInt32(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/nidaqmx_grpc.NiDAQmx/SetRealTimeAttributeInt32',
            nidaqmx__pb2.SetRealTimeAttributeInt32Request.SerializeToString,
            nidaqmx__pb2.SetRealTimeAttributeInt32Response.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def SetRealTimeAttributeUInt32(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/nidaqmx_grpc.NiDAQmx/SetRealTimeAttributeUInt32',
            nidaqmx__pb2.SetRealTimeAttributeUInt32Request.SerializeToString,
            nidaqmx__pb2.SetRealTimeAttributeUInt32Response.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def SetScaleAttributeDouble(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/nidaqmx_grpc.NiDAQmx/SetScaleAttributeDouble',
            nidaqmx__pb2.SetScaleAttributeDoubleRequest.SerializeToString,
            nidaqmx__pb2.SetScaleAttributeDoubleResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def SetScaleAttributeDoubleArray(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/nidaqmx_grpc.NiDAQmx/SetScaleAttributeDoubleArray',
            nidaqmx__pb2.SetScaleAttributeDoubleArrayRequest.SerializeToString,
            nidaqmx__pb2.SetScaleAttributeDoubleArrayResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def SetScaleAttributeInt32(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/nidaqmx_grpc.NiDAQmx/SetScaleAttributeInt32',
            nidaqmx__pb2.SetScaleAttributeInt32Request.SerializeToString,
            nidaqmx__pb2.SetScaleAttributeInt32Response.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def SetScaleAttributeString(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/nidaqmx_grpc.NiDAQmx/SetScaleAttributeString',
            nidaqmx__pb2.SetScaleAttributeStringRequest.SerializeToString,
            nidaqmx__pb2.SetScaleAttributeStringResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def SetStartTrigTrigWhen(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/nidaqmx_grpc.NiDAQmx/SetStartTrigTrigWhen',
            nidaqmx__pb2.SetStartTrigTrigWhenRequest.SerializeToString,
            nidaqmx__pb2.SetStartTrigTrigWhenResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def SetSyncPulseTimeWhen(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/nidaqmx_grpc.NiDAQmx/SetSyncPulseTimeWhen',
            nidaqmx__pb2.SetSyncPulseTimeWhenRequest.SerializeToString,
            nidaqmx__pb2.SetSyncPulseTimeWhenResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def SetTimingAttributeBool(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/nidaqmx_grpc.NiDAQmx/SetTimingAttributeBool',
            nidaqmx__pb2.SetTimingAttributeBoolRequest.SerializeToString,
            nidaqmx__pb2.SetTimingAttributeBoolResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def SetTimingAttributeDouble(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/nidaqmx_grpc.NiDAQmx/SetTimingAttributeDouble',
            nidaqmx__pb2.SetTimingAttributeDoubleRequest.SerializeToString,
            nidaqmx__pb2.SetTimingAttributeDoubleResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def SetTimingAttributeExBool(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/nidaqmx_grpc.NiDAQmx/SetTimingAttributeExBool',
            nidaqmx__pb2.SetTimingAttributeExBoolRequest.SerializeToString,
            nidaqmx__pb2.SetTimingAttributeExBoolResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def SetTimingAttributeExDouble(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/nidaqmx_grpc.NiDAQmx/SetTimingAttributeExDouble',
            nidaqmx__pb2.SetTimingAttributeExDoubleRequest.SerializeToString,
            nidaqmx__pb2.SetTimingAttributeExDoubleResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def SetTimingAttributeExInt32(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/nidaqmx_grpc.NiDAQmx/SetTimingAttributeExInt32',
            nidaqmx__pb2.SetTimingAttributeExInt32Request.SerializeToString,
            nidaqmx__pb2.SetTimingAttributeExInt32Response.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def SetTimingAttributeExString(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/nidaqmx_grpc.NiDAQmx/SetTimingAttributeExString',
            nidaqmx__pb2.SetTimingAttributeExStringRequest.SerializeToString,
            nidaqmx__pb2.SetTimingAttributeExStringResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def SetTimingAttributeExTimestamp(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/nidaqmx_grpc.NiDAQmx/SetTimingAttributeExTimestamp',
            nidaqmx__pb2.SetTimingAttributeExTimestampRequest.SerializeToString,
            nidaqmx__pb2.SetTimingAttributeExTimestampResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def SetTimingAttributeExUInt32(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/nidaqmx_grpc.NiDAQmx/SetTimingAttributeExUInt32',
            nidaqmx__pb2.SetTimingAttributeExUInt32Request.SerializeToString,
            nidaqmx__pb2.SetTimingAttributeExUInt32Response.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def SetTimingAttributeExUInt64(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/nidaqmx_grpc.NiDAQmx/SetTimingAttributeExUInt64',
            nidaqmx__pb2.SetTimingAttributeExUInt64Request.SerializeToString,
            nidaqmx__pb2.SetTimingAttributeExUInt64Response.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def SetTimingAttributeInt32(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/nidaqmx_grpc.NiDAQmx/SetTimingAttributeInt32',
            nidaqmx__pb2.SetTimingAttributeInt32Request.SerializeToString,
            nidaqmx__pb2.SetTimingAttributeInt32Response.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def SetTimingAttributeString(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/nidaqmx_grpc.NiDAQmx/SetTimingAttributeString',
            nidaqmx__pb2.SetTimingAttributeStringRequest.SerializeToString,
            nidaqmx__pb2.SetTimingAttributeStringResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def SetTimingAttributeTimestamp(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/nidaqmx_grpc.NiDAQmx/SetTimingAttributeTimestamp',
            nidaqmx__pb2.SetTimingAttributeTimestampRequest.SerializeToString,
            nidaqmx__pb2.SetTimingAttributeTimestampResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def SetTimingAttributeUInt32(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/nidaqmx_grpc.NiDAQmx/SetTimingAttributeUInt32',
            nidaqmx__pb2.SetTimingAttributeUInt32Request.SerializeToString,
            nidaqmx__pb2.SetTimingAttributeUInt32Response.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def SetTimingAttributeUInt64(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/nidaqmx_grpc.NiDAQmx/SetTimingAttributeUInt64',
            nidaqmx__pb2.SetTimingAttributeUInt64Request.SerializeToString,
            nidaqmx__pb2.SetTimingAttributeUInt64Response.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def SetTrigAttributeBool(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/nidaqmx_grpc.NiDAQmx/SetTrigAttributeBool',
            nidaqmx__pb2.SetTrigAttributeBoolRequest.SerializeToString,
            nidaqmx__pb2.SetTrigAttributeBoolResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def SetTrigAttributeDouble(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/nidaqmx_grpc.NiDAQmx/SetTrigAttributeDouble',
            nidaqmx__pb2.SetTrigAttributeDoubleRequest.SerializeToString,
            nidaqmx__pb2.SetTrigAttributeDoubleResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def SetTrigAttributeDoubleArray(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/nidaqmx_grpc.NiDAQmx/SetTrigAttributeDoubleArray',
            nidaqmx__pb2.SetTrigAttributeDoubleArrayRequest.SerializeToString,
            nidaqmx__pb2.SetTrigAttributeDoubleArrayResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def SetTrigAttributeInt32(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/nidaqmx_grpc.NiDAQmx/SetTrigAttributeInt32',
            nidaqmx__pb2.SetTrigAttributeInt32Request.SerializeToString,
            nidaqmx__pb2.SetTrigAttributeInt32Response.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def SetTrigAttributeInt32Array(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/nidaqmx_grpc.NiDAQmx/SetTrigAttributeInt32Array',
            nidaqmx__pb2.SetTrigAttributeInt32ArrayRequest.SerializeToString,
            nidaqmx__pb2.SetTrigAttributeInt32ArrayResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def SetTrigAttributeString(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/nidaqmx_grpc.NiDAQmx/SetTrigAttributeString',
            nidaqmx__pb2.SetTrigAttributeStringRequest.SerializeToString,
            nidaqmx__pb2.SetTrigAttributeStringResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def SetTrigAttributeTimestamp(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/nidaqmx_grpc.NiDAQmx/SetTrigAttributeTimestamp',
            nidaqmx__pb2.SetTrigAttributeTimestampRequest.SerializeToString,
            nidaqmx__pb2.SetTrigAttributeTimestampResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def SetTrigAttributeUInt32(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/nidaqmx_grpc.NiDAQmx/SetTrigAttributeUInt32',
            nidaqmx__pb2.SetTrigAttributeUInt32Request.SerializeToString,
            nidaqmx__pb2.SetTrigAttributeUInt32Response.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def SetWatchdogAttributeBool(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/nidaqmx_grpc.NiDAQmx/SetWatchdogAttributeBool',
            nidaqmx__pb2.SetWatchdogAttributeBoolRequest.SerializeToString,
            nidaqmx__pb2.SetWatchdogAttributeBoolResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def SetWatchdogAttributeDouble(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/nidaqmx_grpc.NiDAQmx/SetWatchdogAttributeDouble',
            nidaqmx__pb2.SetWatchdogAttributeDoubleRequest.SerializeToString,
            nidaqmx__pb2.SetWatchdogAttributeDoubleResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def SetWatchdogAttributeInt32(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/nidaqmx_grpc.NiDAQmx/SetWatchdogAttributeInt32',
            nidaqmx__pb2.SetWatchdogAttributeInt32Request.SerializeToString,
            nidaqmx__pb2.SetWatchdogAttributeInt32Response.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def SetWatchdogAttributeString(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/nidaqmx_grpc.NiDAQmx/SetWatchdogAttributeString',
            nidaqmx__pb2.SetWatchdogAttributeStringRequest.SerializeToString,
            nidaqmx__pb2.SetWatchdogAttributeStringResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def SetWriteAttributeBool(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/nidaqmx_grpc.NiDAQmx/SetWriteAttributeBool',
            nidaqmx__pb2.SetWriteAttributeBoolRequest.SerializeToString,
            nidaqmx__pb2.SetWriteAttributeBoolResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def SetWriteAttributeDouble(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/nidaqmx_grpc.NiDAQmx/SetWriteAttributeDouble',
            nidaqmx__pb2.SetWriteAttributeDoubleRequest.SerializeToString,
            nidaqmx__pb2.SetWriteAttributeDoubleResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def SetWriteAttributeInt32(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/nidaqmx_grpc.NiDAQmx/SetWriteAttributeInt32',
            nidaqmx__pb2.SetWriteAttributeInt32Request.SerializeToString,
            nidaqmx__pb2.SetWriteAttributeInt32Response.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def SetWriteAttributeString(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/nidaqmx_grpc.NiDAQmx/SetWriteAttributeString',
            nidaqmx__pb2.SetWriteAttributeStringRequest.SerializeToString,
            nidaqmx__pb2.SetWriteAttributeStringResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def SetWriteAttributeUInt32(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/nidaqmx_grpc.NiDAQmx/SetWriteAttributeUInt32',
            nidaqmx__pb2.SetWriteAttributeUInt32Request.SerializeToString,
            nidaqmx__pb2.SetWriteAttributeUInt32Response.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def SetWriteAttributeUInt64(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/nidaqmx_grpc.NiDAQmx/SetWriteAttributeUInt64',
            nidaqmx__pb2.SetWriteAttributeUInt64Request.SerializeToString,
            nidaqmx__pb2.SetWriteAttributeUInt64Response.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def StartNewFile(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/nidaqmx_grpc.NiDAQmx/StartNewFile',
            nidaqmx__pb2.StartNewFileRequest.SerializeToString,
            nidaqmx__pb2.StartNewFileResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def StartTask(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/nidaqmx_grpc.NiDAQmx/StartTask',
            nidaqmx__pb2.StartTaskRequest.SerializeToString,
            nidaqmx__pb2.StartTaskResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def StopTask(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/nidaqmx_grpc.NiDAQmx/StopTask',
            nidaqmx__pb2.StopTaskRequest.SerializeToString,
            nidaqmx__pb2.StopTaskResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def TaskControl(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/nidaqmx_grpc.NiDAQmx/TaskControl',
            nidaqmx__pb2.TaskControlRequest.SerializeToString,
            nidaqmx__pb2.TaskControlResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def TristateOutputTerm(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/nidaqmx_grpc.NiDAQmx/TristateOutputTerm',
            nidaqmx__pb2.TristateOutputTermRequest.SerializeToString,
            nidaqmx__pb2.TristateOutputTermResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def UnregisterDoneEvent(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/nidaqmx_grpc.NiDAQmx/UnregisterDoneEvent',
            nidaqmx__pb2.UnregisterDoneEventRequest.SerializeToString,
            nidaqmx__pb2.UnregisterDoneEventResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def UnregisterEveryNSamplesEvent(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/nidaqmx_grpc.NiDAQmx/UnregisterEveryNSamplesEvent',
            nidaqmx__pb2.UnregisterEveryNSamplesEventRequest.SerializeToString,
            nidaqmx__pb2.UnregisterEveryNSamplesEventResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def UnregisterSignalEvent(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/nidaqmx_grpc.NiDAQmx/UnregisterSignalEvent',
            nidaqmx__pb2.UnregisterSignalEventRequest.SerializeToString,
            nidaqmx__pb2.UnregisterSignalEventResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def UnreserveNetworkDevice(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/nidaqmx_grpc.NiDAQmx/UnreserveNetworkDevice',
            nidaqmx__pb2.UnreserveNetworkDeviceRequest.SerializeToString,
            nidaqmx__pb2.UnreserveNetworkDeviceResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def WaitForNextSampleClock(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/nidaqmx_grpc.NiDAQmx/WaitForNextSampleClock',
            nidaqmx__pb2.WaitForNextSampleClockRequest.SerializeToString,
            nidaqmx__pb2.WaitForNextSampleClockResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def WaitForValidTimestamp(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/nidaqmx_grpc.NiDAQmx/WaitForValidTimestamp',
            nidaqmx__pb2.WaitForValidTimestampRequest.SerializeToString,
            nidaqmx__pb2.WaitForValidTimestampResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def WaitUntilTaskDone(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/nidaqmx_grpc.NiDAQmx/WaitUntilTaskDone',
            nidaqmx__pb2.WaitUntilTaskDoneRequest.SerializeToString,
            nidaqmx__pb2.WaitUntilTaskDoneResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def WriteAnalogF64(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/nidaqmx_grpc.NiDAQmx/WriteAnalogF64',
            nidaqmx__pb2.WriteAnalogF64Request.SerializeToString,
            nidaqmx__pb2.WriteAnalogF64Response.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def WriteAnalogScalarF64(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/nidaqmx_grpc.NiDAQmx/WriteAnalogScalarF64',
            nidaqmx__pb2.WriteAnalogScalarF64Request.SerializeToString,
            nidaqmx__pb2.WriteAnalogScalarF64Response.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def WriteBinaryI16(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/nidaqmx_grpc.NiDAQmx/WriteBinaryI16',
            nidaqmx__pb2.WriteBinaryI16Request.SerializeToString,
            nidaqmx__pb2.WriteBinaryI16Response.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def WriteBinaryI32(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/nidaqmx_grpc.NiDAQmx/WriteBinaryI32',
            nidaqmx__pb2.WriteBinaryI32Request.SerializeToString,
            nidaqmx__pb2.WriteBinaryI32Response.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def WriteBinaryU16(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/nidaqmx_grpc.NiDAQmx/WriteBinaryU16',
            nidaqmx__pb2.WriteBinaryU16Request.SerializeToString,
            nidaqmx__pb2.WriteBinaryU16Response.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def WriteBinaryU32(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/nidaqmx_grpc.NiDAQmx/WriteBinaryU32',
            nidaqmx__pb2.WriteBinaryU32Request.SerializeToString,
            nidaqmx__pb2.WriteBinaryU32Response.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def WriteCtrFreq(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/nidaqmx_grpc.NiDAQmx/WriteCtrFreq',
            nidaqmx__pb2.WriteCtrFreqRequest.SerializeToString,
            nidaqmx__pb2.WriteCtrFreqResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def WriteCtrFreqScalar(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/nidaqmx_grpc.NiDAQmx/WriteCtrFreqScalar',
            nidaqmx__pb2.WriteCtrFreqScalarRequest.SerializeToString,
            nidaqmx__pb2.WriteCtrFreqScalarResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def WriteCtrTicks(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/nidaqmx_grpc.NiDAQmx/WriteCtrTicks',
            nidaqmx__pb2.WriteCtrTicksRequest.SerializeToString,
            nidaqmx__pb2.WriteCtrTicksResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def WriteCtrTicksScalar(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/nidaqmx_grpc.NiDAQmx/WriteCtrTicksScalar',
            nidaqmx__pb2.WriteCtrTicksScalarRequest.SerializeToString,
            nidaqmx__pb2.WriteCtrTicksScalarResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def WriteCtrTime(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/nidaqmx_grpc.NiDAQmx/WriteCtrTime',
            nidaqmx__pb2.WriteCtrTimeRequest.SerializeToString,
            nidaqmx__pb2.WriteCtrTimeResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def WriteCtrTimeScalar(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/nidaqmx_grpc.NiDAQmx/WriteCtrTimeScalar',
            nidaqmx__pb2.WriteCtrTimeScalarRequest.SerializeToString,
            nidaqmx__pb2.WriteCtrTimeScalarResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def WriteDigitalLines(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/nidaqmx_grpc.NiDAQmx/WriteDigitalLines',
            nidaqmx__pb2.WriteDigitalLinesRequest.SerializeToString,
            nidaqmx__pb2.WriteDigitalLinesResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def WriteDigitalScalarU32(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/nidaqmx_grpc.NiDAQmx/WriteDigitalScalarU32',
            nidaqmx__pb2.WriteDigitalScalarU32Request.SerializeToString,
            nidaqmx__pb2.WriteDigitalScalarU32Response.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def WriteDigitalU16(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/nidaqmx_grpc.NiDAQmx/WriteDigitalU16',
            nidaqmx__pb2.WriteDigitalU16Request.SerializeToString,
            nidaqmx__pb2.WriteDigitalU16Response.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def WriteDigitalU32(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/nidaqmx_grpc.NiDAQmx/WriteDigitalU32',
            nidaqmx__pb2.WriteDigitalU32Request.SerializeToString,
            nidaqmx__pb2.WriteDigitalU32Response.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def WriteDigitalU8(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/nidaqmx_grpc.NiDAQmx/WriteDigitalU8',
            nidaqmx__pb2.WriteDigitalU8Request.SerializeToString,
            nidaqmx__pb2.WriteDigitalU8Response.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def WriteRaw(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/nidaqmx_grpc.NiDAQmx/WriteRaw',
            nidaqmx__pb2.WriteRawRequest.SerializeToString,
            nidaqmx__pb2.WriteRawResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def WriteToTEDSFromArray(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/nidaqmx_grpc.NiDAQmx/WriteToTEDSFromArray',
            nidaqmx__pb2.WriteToTEDSFromArrayRequest.SerializeToString,
            nidaqmx__pb2.WriteToTEDSFromArrayResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def WriteToTEDSFromFile(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/nidaqmx_grpc.NiDAQmx/WriteToTEDSFromFile',
            nidaqmx__pb2.WriteToTEDSFromFileRequest.SerializeToString,
            nidaqmx__pb2.WriteToTEDSFromFileResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)
